/**
*  Title:           ColabCreateDB for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 2000
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     Define and Populate MySQL Unix database tables for COLAB
*/

package colab2000;

import java.util.*;
import java.net.URL;
import java.sql.*;

import pow2000.DatabaseMgr;

import pow2000.ChangeMgr;


/**
initialize standard tables
*/
public class ColabCreateDB {
    final int numOfTables = 14;
    String[] tableNames = new String[numOfTables];
    DatabaseMgr databaseMgr1;
    ChangeMgr changeMgr1;
    Connection con;
    Statement stmt;
    Vector DBtableName;

    final String tableSpaces = "create table spaces ( sid bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " name varchar(100), driver varchar(100), url varchar(100), username varchar(100), password varchar(100), " +
    " dbms varchar(100), comment varchar(255), PRIMARY KEY (sid) )";

    final String tableNodes = "create table nodes ( nid bigint(21) DEFAULT '0' NOT NULL auto_increment, " +
    " kind bigint(21) DEFAULT '0' NOT NULL, perspective bigint(21)DEFAULT '0' NOT NULL, " +
    " suptable bigint(21) DEFAULT '0' NOT NULL, content bigint(21), name varchar(100), " +
    " description varchar(255), author bigint(21), created timestamp(14), editor bigint(21), " +
    " modified timestamp(14), PRIMARY KEY (nid), " +
    " KEY (kind), KEY (perspective), KEY (suptable) )";

    final String tableLinks = "create table links ( lid bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " type bigint(21) DEFAULT '0' NOT NULL, perspective bigint(21)DEFAULT '0' NOT NULL, fromnode bigint(21) DEFAULT '0' NOT NULL," +
    " tonode  bigint(21)DEFAULT '0' NOT NULL, seq int(11), author bigint(21), created timestamp(14), PRIMARY KEY (lid), KEY (type)," +
    " KEY (perspective), KEY (fromnode), KEY (tonode) )";

    final String tablePeople = "create table people ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), username varchar(20), password varchar(20), perspective bigint(21), fullname varchar(100), first varchar(20)," +
    " middle varchar(20), last varchar(20), email varchar(100), homepage varchar(255), gender varchar(4)," +
    " favorite_color varchar(20), work_place varchar(100), work_title varchar(100), home_phone varchar(20)," +
    " work_phone varchar(20), PRIMARY KEY (id) )";

    final String tableGroups = "create table groups ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), password varchar(20), representative bigint(21), email varchar(255)," +
    " home_perspective bigint(21), PRIMARY KEY (id) )";

    final String tableTables = "create table auxtables ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21) DEFAULT '0' NOT NULL, field0 int(11), field1 int(11), field2 int(11)," +
    " field3 int(11), field4 int(11), field5 int(11)," +
    " field6 int(11), field7 int(11), field8 int(11), field9 int(11), field10 int(11), field11 int(11), field12 int(11)," +
    " field13 int(11), field14 int(11), field15 int(11), field16 int(11), field17 int(11), field18 int(11), field19 int(11)," +
    " PRIMARY KEY (id), KEY (nid) )";

    final String tableStatements = "create table statements ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), statement text, PRIMARY KEY (id) )";

    final String tableDecisions = "create table decisions ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), decision varchar(100), rationale text, PRIMARY KEY (id) )";

    final String tableBooks = "create table books ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), author varchar(100), year varchar(4), title varchar(100), editor varchar(100), journal varchar(100)," +
    " issue varchar (10), location varchar (100), website varchar (255), description text, abstract text," +
    " keywords varchar (255), PRIMARY KEY (id) )";

    final String tableMultimedia = "create table multimedia ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), medium varchar (255), url varchar (255), PRIMARY KEY (id) )";

    final String tableObjects = "create table objects ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21), class varchar (255), object varchar (255), PRIMARY KEY (id) )";

    final String tableEmail = "create table email ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21) DEFAULT '0' NOT NULL, recipient varchar(255), sender varchar(255), subject varchar(255)," +
    " thread varchar(255), content text, PRIMARY KEY (id) )";

    final String tablePositions = "create table positions ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " nid bigint(21) DEFAULT '0' NOT NULL, perspective bigint(21) DEFAULT '0' NOT NULL, x bigint(21) DEFAULT '0' NOT NULL," +
    " y bigint(21) DEFAULT '0' NOT NULL, PRIMARY KEY (id) )";

    final String tableTransactions = "create table transactions ( id bigint(21) DEFAULT '0' NOT NULL auto_increment," +
    " type bigint(21) DEFAULT '0' NOT NULL, perspective bigint(21)DEFAULT '0' NOT NULL, fromnode bigint(21) DEFAULT '0' NOT NULL," +
    " tonode  bigint(21)DEFAULT '0' NOT NULL, author bigint(21), created timestamp(14), PRIMARY KEY (id), KEY (type)," +
    " KEY (perspective), KEY (fromnode), KEY (tonode) )";

  /**
  drop old tables and create new ones
  */
  public ColabCreateDB(DatabaseMgr dbMgr, ChangeMgr chMgr) {
      try {
          databaseMgr1 = dbMgr;
          changeMgr1 = chMgr;

          tableNames[0] = "spaces";                         //define names of tables to create
          tableNames[1] = "nodes";
          tableNames[2] = "links";
          tableNames[3] = "people";
          tableNames[4] = "groups";
          tableNames[5] = "auxtables";
          tableNames[6] = "statements";
          tableNames[7] = "decisions";
          tableNames[8] = "books";
          tableNames[9] = "multimedia";
          tableNames[10] = "objects";
          tableNames[11] = "email";
          tableNames[12] = "positions";
          tableNames[13] = "transactions";
          //add new tables here and increase numOfTables above

          DBtableName = new Vector();
          int numDropped = dropTables();                    //drop old tables
          for (int i = 0; i < numOfTables; i++) {
              createDtbase(i);                              //create new tables
          }
          populate();                                       //populate new tables
      }
      catch( Exception er ) {
         System.out.println(er.getMessage());
         er.printStackTrace();
      }
  }

  int dropTables() throws Exception {
      Connection con = databaseMgr1.openDatabase();    //open database defined in DatabaseMgr
      int count = 0;
      int dropped = 0;
      DatabaseMetaData dbmd = con.getMetaData();
      ResultSet rs = dbmd.getTables(null, null, "%", null);
      while( rs.next() ) {
          String theName = rs.getString(3);
          if (theName != "null") {
              DBtableName.addElement(theName);
              count++;
          }
      }
      con.close();
      con = databaseMgr1.openDatabase();    //open database defined in DatabaseMgr
      stmt = con.createStatement();
      for (int i=0; i < count; i++) {
          for (int j=0; j < numOfTables; j++) {
              if (tableNames[j].equals(DBtableName.elementAt(i))) {
System.out.println("dropping table: " + tableNames[j]);
                  DeleteTable (tableNames[j]);
                  dropped++;
                  break;
              }
          }
      }
      con.close();
      return dropped;
  }

  private int createDtbase (int tableNumber) throws Exception {
    Connection con = databaseMgr1.openDatabase();    //open database defined in DatabaseMgr
    Statement stmt = con.createStatement();
    String create = "";
    switch (tableNumber) {
        case 0:
             create = tableSpaces;
        break;
        case 1:
             create = tableNodes;
        break;
        case 2:
             create = tableLinks;
        break;
        case 3:
             create = tablePeople;
        break;
        case 4:
             create = tableGroups;
        break;
        case 5:
             create = tableTables;
        break;
        case 6:
             create = tableStatements;
        break;
        case 7:
             create = tableDecisions;
        break;
        case 8:
             create = tableBooks;
        break;
        case 9:
             create = tableMultimedia;
        break;
        case 10:
             create = tableObjects;
        break;
        case 11:
             create = tableEmail;
        break;
        case 12:
             create = tablePositions;
        break;
        case 13:
             create = tableTransactions;
        break;
    }
    int rsSet = stmt.executeUpdate(create);                            // create new table using definition above
System.out.println("creating table: " + tableNames[tableNumber]);
          con.close();
    return 0;
  }

   private void DeleteTable (String table) throws Exception {
        int rsSet = stmt.executeUpdate("drop table " + table);         // drop old copy of table
  }

  /**
  populate() creates initial content of table[i] for a COLAB application.
  This includes defining standard categories, constants and auxiliary tables.
  */
  private void populate() throws Exception {
        Connection con = databaseMgr1.openDatabase();    //open database defined in DatabaseMgr
        Statement stmt = con.createStatement();
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 0, 3, 0, 0, 'COLAB_ROOT', 'root of info space', 13, NULL, 0, NULL)");                           // 1 = COLAB_ROOT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 1, 3, 0, 0, 'COLAB_ROOT_CATEGORY', 'root category', 13, NULL, 0, NULL)");         // 2 = COLAB_ROOT_CATEGORY
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 1, 3, 0, 0, 'COLAB_ROOT_PERSPECTIVE', 'root perspective', 13, NULL, 0, NULL)");   // 3 = COLAB_ROOT_PERSPECTIVE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 1, 3, 0, 0, 'COLAB_ROOT_CONTENT', 'root of content', 13, NULL, 0, NULL)");           // 4 = COLAB_ROOT_CONTENT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 1, 3, 0, 0, 'COLAB_COMPARISON_PERSPECTIVE', 'COMPARISON PERSPECTIVE', 13, NULL, 0, NULL)");   // 5 = COLAB_COMPARISON_PERSPECTIVE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 2, 3, 0, 0, 'LINK_TYPE', 'LINK_TYPE', 13, NULL, 0, NULL)");                             // 6 = LINK_TYPE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 2, 3, 0, 0, 'NODE_KIND', 'NODE_KIND', 13, NULL, 0, NULL)");                             // 7 = NODE_KIND

        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'EDIT', 'EDIT', 13, NULL, 0, NULL)");                                       // 8 = EDIT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'DELETE', 'DELETE', 13, NULL, 0, NULL)");                                   // 9 = DELETE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'DELETELINK', 'DELETELINK', 13, NULL, 0, NULL)");                           // 10 = DELETELINK
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'SYNONYM', 'SYNONYM', 13, NULL, 0, NULL)");                                 // 11= SYNONYM
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'PARENT', 'PARENT', 13, NULL, 0, NULL)");                                   // 12 = PARENT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'ISA', 'ISA', 13, NULL, 0, NULL)");                                         // 13 = ISA
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'VCOPY', 'VCOPY', 13, NULL, 0, NULL)");                                     // 14 = VCOPY
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 6, 3, 0, 0, 'VCHILDREN', 'VCHILDREN', 13, NULL, 0, NULL)");                             // 15 = VCHILDREN

        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'PERSPECTIVE', 'PERSPECTIVE', 13, NULL, 0, NULL)");                         // 16 = PERSPECTIVE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'INFORMATION', 'INFORMATION', 13, NULL, 0, NULL)");                         // 17 = INFORMATION
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'PERSON', 'PERSON', 13, NULL, 0, NULL)");                                   // 18 = PERSON
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'GROUP', 'GROUP', 13, NULL, 0, NULL)");                                     // 19 = GROUP
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'TABLE', 'TABLE', 13, NULL, 0, NULL)");                                     // 20 = TABLE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'CONTENT', 'CONTENT', 13, NULL, 0, NULL)");                                 // 21 = CONTENT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'STATEMENT', 'STATEMENT', 13, NULL, 0, NULL)");                             // 22 = STATEMENT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'DISCUSSION', 'DISCUSSION', 13, NULL, 0, NULL)");                           // 23 = DISCUSSION
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'PROPOSAL', 'PROPOSAL', 13, NULL, 0, NULL)");                               // 24 = PROPOSAL
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'PRIVATE', 'PRIVATE', 13, NULL, 0, NULL)");                                 // 25 = PRIVATE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'DECISION', 'DECISION', 13, NULL, 0, NULL)");                               // 26 = DECISION
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'BOOK', 'BOOK', 13, NULL, 0, NULL)");                                       // 27 = BOOK
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'MULTIMEDIA', 'MULTIMEDIA', 13, NULL, 0, NULL)");                           // 28 = MULTIMEDIA
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'OBJECT', 'INFORMATION', 13, NULL, 0, NULL)");                              // 29 = OBJECT
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'EMAIL', 'EMAIL', 13, NULL, 0, NULL)");                                     // 30 = EMAIL
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 18, 3, 0, 0, 'COLAB_ROOT_AUTHOR', 'ROOT_AUTHOR', 13, NULL, 0, NULL)");            // 31 = COLAB_ROOT_AUTHOR
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 20, 3, 17, 1, 'AUXTABLES_TABLE', 'AUXTABLES_TABLE', 13, NULL, 0, NULL)");               // 32 = AUXTABLES_TABLE
        stmt.executeUpdate("INSERT INTO nodes VALUES (0, 7, 3, 0, 0, 'COLAB_LAST', 'LAST', 13, NULL, 0, NULL)");                           // 33 = COLAB_LAST
        stmt.executeUpdate("INSERT INTO auxtables VALUES (0, 32, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)");

        databaseMgr1.refreshGlobals(stmt);
        int COLAB_ROOT = databaseMgr1.state1.getCOLAB_ROOT();
        int COLAB_ROOT_CATEGORY = databaseMgr1.state1.getCOLAB_ROOT_CATEGORY();
        int COLAB_ROOT_PERSPECTIVE = databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE();
        int COLAB_ROOT_CONTENT = databaseMgr1.state1.getCOLAB_ROOT_CONTENT();
        int COLAB_ROOT_AUTHOR = databaseMgr1.state1.getCOLAB_ROOT_AUTHOR();
        int COLAB_COMPARISON_PERSPECTIVE = databaseMgr1.state1.getCOLAB_COMPARISON_PERSPECTIVE();
        int COLAB_LAST = databaseMgr1.state1.getCOLAB_LAST();
        int LINK_TYPE = databaseMgr1.state1.getLINK_TYPE();
        int NODE_KIND = databaseMgr1.state1.getNODE_KIND();

        int EDIT = databaseMgr1.state1.getEDIT();
        int DELETE = databaseMgr1.state1.getDELETE();
        int DELETELINK = databaseMgr1.state1.getDELETELINK();
        int SYNONYM = databaseMgr1.state1.getSYNONYM();
        int PARENT = databaseMgr1.state1.getPARENT();
        int ISA = databaseMgr1.state1.getISA();
        int VCOPY = databaseMgr1.state1.getVCOPY();
        int VCHILDREN = databaseMgr1.state1.getVCHILDREN();

        int PERSPECTIVE = databaseMgr1.state1.getPERSPECTIVE();
        int INFORMATION = databaseMgr1.state1.getINFORMATION();
        int PERSON = databaseMgr1.state1.getPERSON();
        int GROUP = databaseMgr1.state1.getGROUP();
        int TABLE = databaseMgr1.state1.getTABLE();
        int CONTENT = databaseMgr1.state1.getCONTENT();
        int STATEMENT = databaseMgr1.state1.getSTATEMENT();
        int DISCUSSION = databaseMgr1.state1.getDISCUSSION();
        int PROPOSAL = databaseMgr1.state1.getPROPOSAL();
        int PRIVATE = databaseMgr1.state1.getPRIVATE();
        int DECISION = databaseMgr1.state1.getDECISION();
        int BOOK = databaseMgr1.state1.getBOOK();
        int MULTIMEDIA = databaseMgr1.state1.getMULTIMEDIA();
        int OBJECT = databaseMgr1.state1.getOBJECT();
        int EMAIL = databaseMgr1.state1.getEMAIL();

        int PEOPLE_TABLE = databaseMgr1.state1.getPEOPLE_TABLE();
        int GROUPS_TABLE = databaseMgr1.state1.getGROUPS_TABLE();
        int AUXTABLES_TABLE = databaseMgr1.state1.getAUXTABLES_TABLE();
        int STATEMENTS_TABLE = databaseMgr1.state1.getSTATEMENTS_TABLE();
        int DECISIONS_TABLE = databaseMgr1.state1.getDECISIONS_TABLE();
        int BOOKS_TABLE = databaseMgr1.state1.getBOOKS_TABLE();
        int MULTIMEDIA_TABLE = databaseMgr1.state1.getMULTIMEDIA_TABLE();
        int OBJECTS_TABLE = databaseMgr1.state1.getOBJECTS_TABLE();
        int EMAIL_TABLE = databaseMgr1.state1.getEMAIL_TABLE();
        int TRANSACT_TABLE = databaseMgr1.state1.getTRANSACT_TABLE();

        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_ROOT_CATEGORY + ", " + COLAB_ROOT + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_ROOT + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_ROOT_CONTENT + ", " + COLAB_ROOT + ", 3, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + LINK_TYPE + ", " + COLAB_ROOT_CATEGORY + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + NODE_KIND + ", " + COLAB_ROOT_CATEGORY + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");

        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + EDIT + ", " + LINK_TYPE + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + DELETE + ", " + LINK_TYPE + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + DELETELINK + ", " + LINK_TYPE + ", 3, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + SYNONYM + ", " + LINK_TYPE + ", 4, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + PARENT + ", " + LINK_TYPE + ", 5, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + ISA + ", " + PARENT + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + VCOPY + ", " + LINK_TYPE + ", 6, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + VCHILDREN + ", " + LINK_TYPE + ", 7, " + COLAB_ROOT_AUTHOR + ", NULL)");

        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + PERSPECTIVE + ", " + NODE_KIND + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + INFORMATION + ", " + NODE_KIND + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + PERSON + ", " + INFORMATION + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + GROUP + ", " + INFORMATION + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + TABLE + ", " + INFORMATION + ", 3, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + CONTENT + ", " + NODE_KIND + ", 3, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + STATEMENT + ", " + CONTENT + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + DISCUSSION + ", " + STATEMENT + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + PROPOSAL + ", " + STATEMENT + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + PRIVATE + ", " + STATEMENT + ", 3, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + DECISION + ", " + CONTENT + ", 2, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + BOOK + ", " + CONTENT + ", 3, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + MULTIMEDIA + ", " + CONTENT + ", 4, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + OBJECT + ", " + CONTENT + ", 5, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + EMAIL + ", " + CONTENT + ", 6, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_LAST + ", " + CONTENT + ", 7, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_COMPARISON_PERSPECTIVE + ", " + PERSPECTIVE + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + COLAB_ROOT_AUTHOR + ", " + PERSON + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");
        stmt.executeUpdate("INSERT INTO links VALUES (0, " + PARENT + ", " + COLAB_ROOT_PERSPECTIVE + ", " + AUXTABLES_TABLE + ", " + TABLE + ", 1, " + COLAB_ROOT_AUTHOR + ", NULL)");

        changeMgr1.setAuthor(COLAB_ROOT_AUTHOR);
        //changeMgr1.createCategory("TRANSACT", CONTENT, false, stmt);

        /**
        Formats for AUXTABLES_TABLE entries:
             id, nid, table_name, number of data columns, code for field1 .. filed19
        Codes for AUXTABLES_TABLE:
             0 = field not used
             1 = int (11)
             2 = bigint (21)
             3 = boolean
             4 = timestamp
             5 = text
             6 = blob
             7 = varchar (4)
             8 = varchar (10)
             9 = varchar (20)
             10 = varchar (100)
             11 = varchar (255)
        */
        changeMgr1.createTable("PEOPLE_TABLE", 16, 2, 2, 9, 9, 2, 9, 9, 9, 10, 11, 7, 9, 10, 10, 9, 9, 0, 0, 0, stmt);
        changeMgr1.createTable("STATEMENTS_TABLE", 3, 2, 2, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("DECISIONS_TABLE", 4, 2, 2, 10, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("GROUPS_TABLE", 7, 2, 2, 9, 10, 2, 11, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("BOOKS_TABLE", 13, 2, 2, 10, 7, 10, 10, 10, 8, 10, 11, 5, 5, 11, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("MULTIMEDIA_TABLE", 4, 2, 2, 5, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("OBJECTS_TABLE", 4, 2, 2, 11, 11, 11, 11, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("EMAIL_TABLE", 7, 2, 2, 10, 10, 10, 10, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        changeMgr1.createTable("TRANSACT_TABLE", 6, 2, 2, 2, 2, 2, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, stmt);
        databaseMgr1.refreshGlobals(stmt);

        stmt.executeUpdate("INSERT INTO people VALUES (0, " + databaseMgr1.state1.getPEOPLE_TABLE() +
                            ", 'ROOT_AUTHOR', 'ROOT_AUTHOR', " + databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE() +
                            ", 'ROOT_AUTHOR', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ')");
        stmt.executeUpdate("UPDATE nodes SET suptable = " + databaseMgr1.state1.getPEOPLE_TABLE() + " WHERE nid = " + databaseMgr1.state1.getCOLAB_ROOT_AUTHOR());
        stmt.executeUpdate("UPDATE nodes SET content = 1 WHERE nid = " + databaseMgr1.state1.getCOLAB_ROOT_AUTHOR());

        //define collaboration spaces
        stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'colab', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/colab', 'webguide', 'webguide', 'MySQL',  " +
                           "'Central Colab Spaces Database')");
        stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'artifact', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/artifact', 'webguide', 'webguide', 'MySQL',  " +
                           "'Research on cognitive artifacts')");
        //stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'demo', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/demo', 'webguide', 'webguide', 'MySQL',  " +
        //                   "'Demo of the WebGuide system')");
        stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'design', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/design', 'webguide', 'webguide', 'MySQL',  " +
                           "'Design the WebGuide system')");
        //stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'gerry', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/gerry', 'webguide', 'webguide', 'MySQL',  " +
        //                   "'Gerry-s journal')");
        //stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'interface', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/interface', 'webguide', 'webguide', 'MySQL',  " +
        //                   "'WebGuide Negotiation Interface')");
        //stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'research', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/research', 'webguide', 'webguide', 'MySQL',  " +
        //                   "'Research in Cog Sci')");
        stmt.executeUpdate("INSERT INTO spaces VALUES (0, 'webguide2', 'gwe.sql.gweMysqlDriver', 'jdbc:mysql://webguide.cs.colorado.edu:3306/webguide2', 'webguide', 'webguide', 'MySQL',  " +
                           "'Research in Cog Sci')");
        con.close();

databaseMgr1.db("done populate()");
  }

}