/**
*  Title:           ColabInitializeArtifactDB for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 2000
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     create data for this information space
*/

package colab2000;

import javax.swing.UIManager;
import java.sql.*;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import pow2000.DatabaseMgr;
import pow2000.ChangeMgr;
import colab2000.ColabStandardDB;

public class ColabInitializeArtifactDB {
    boolean packFrame = false;
    static DatabaseMgr databaseMgr1;
    static ChangeMgr changeMgr1;
    static int nidCompare = 0;
    static int nidRoot = 0;
    static public ColabCreateDB dbObject1;
    static public ColabStandardDB dbObject2;

    //Construct the application
    public ColabInitializeArtifactDB() {
    }

    //Main method
    public static void main(String[] args) {
        try  {
            databaseMgr1 = new DatabaseMgr();
            changeMgr1 = new ChangeMgr(databaseMgr1);
            WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {System.exit(0);}
            };
            
            databaseMgr1.checkDBname("artifact");    //switch to this database
            //databaseMgr1.checkDBname("colab");    //switch to this database

            //create database tables and define glogals
            dbObject1 = new ColabCreateDB(databaseMgr1, changeMgr1);
            //add standard initialization data to database
            dbObject2 = new ColabStandardDB(databaseMgr1, changeMgr1);
            //add initialization data that is special to this database
            nidCompare = databaseMgr1.state1.getComparisonPerspectiveNid();
            nidRoot = databaseMgr1.state1.getRootPerspectiveNid();
            runSpecial();
            System.exit(0);
        }
        catch (Exception e) {
        }
    }

    //add data special for this database
    public static void runSpecial() throws Exception {
      try {
        Connection ColabDatabase = databaseMgr1.openDatabase();
        Statement stmt = ColabDatabase.createStatement();
        databaseMgr1.refreshGlobals(stmt);
        changeMgr1.setAuthor(databaseMgr1.state1.getCOLAB_ROOT_AUTHOR());
        int p = databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE();
        int nid = 0;

        //add special statement categories
        changeMgr1.createCategory("definition", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        changeMgr1.createCategory("resource", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        changeMgr1.createCategory("category", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        changeMgr1.createCategory("question", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        changeMgr1.createCategory("claim", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        nid = changeMgr1.createCategory("argument", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        changeMgr1.createCategory("argument for", nid, false, stmt);
        changeMgr1.createCategory("argument against", nid, true, stmt);

        //add more special user perspectives
        dbObject2.addUser("Rogerio", "dePaula", "#FF00FF", stmt);

        dbObject2.addUser("Elizabeth", "Lennel", "#FF0000", stmt);
        dbObject2.addUser("Alena", "Sanusi", "#777700", stmt);
        dbObject2.addUser("Bob", "Craig", "#7700FF", stmt);
        dbObject2.addUser("Curt", "LeBaron", "#770000", stmt);
        dbObject2.addUser("Tammy", "Sumner", "", stmt);
        dbObject2.addUser("Leysia", "Palen", "#00FFFF", stmt);
        dbObject2.addUser("Ben", "Pageler", "#007777", stmt);
        dbObject2.addUser("Stephen", "Biggs", "#000077", stmt);
        dbObject2.addUser("Barb", "Stuart", "#6666CC", stmt);
        dbObject2.addUser("Kelly", "Alanis", "#336600", stmt);
        dbObject2.addUser("Marcelo", "Milroad", "#996699", stmt);
        dbObject2.addUser("Ryan", "Tweney", "", stmt);
        dbObject2.addUser("Marcel", "Hoffmann", "", stmt);
        dbObject2.addUser("Thomas", "Herrmann", "", stmt);
        dbObject2.addUser("Anders", "Morch", "", stmt);
        dbObject2.addUser("Paul", "Feltovitch", "", stmt);
        dbObject2.addUser("Tim", "Koschmann", "", stmt);
        dbObject2.addUser("Leo", "Burd", "#FF6600", stmt);

        System.out.println("***  done runSpecial()");
        ColabDatabase.close();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }

}



