/**
*  Title:           ColabStandardDB for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1999
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     create standard data for all COLAB information spaces
*/

package colab2000;

import javax.swing.UIManager;
import java.sql.*;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import pow2000.DatabaseMgr;
import pow2000.ChangeMgr;

public class ColabStandardDB {
    boolean packFrame = false;
    static DatabaseMgr databaseMgr1;
    static ChangeMgr changeMgr1;
    static int nidCompare = 0;
    static int nidRoot = 0;

    //Construct the application
    public ColabStandardDB(DatabaseMgr dbMgr, ChangeMgr chMgr) throws Exception {
        databaseMgr1 = dbMgr;
        changeMgr1 = chMgr;
        runStandard();
    }

    //save nid of the top root perspective in the content field of COLAB_ROOT_PERSPECTIVE
    private static void makeCOLAB_ROOT_PERSPECTIVE(int nid, Statement stmt) throws Exception {
        int crp = databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE();
        stmt.executeUpdate("UPDATE nodes SET content = " + nid + " WHERE nid = " + crp);
    }

    //save nid of the bottom comparison perspective in the content field of COLAB_COMPARISON_PERSPECTIVE
    private static void makeCOLAB_COMPARISON_PERSPECTIVE(int nid, Statement stmt) throws Exception {
        int ccp = databaseMgr1.state1.getCOLAB_COMPARISON_PERSPECTIVE();
        stmt.executeUpdate("UPDATE nodes SET content = " + nid + " WHERE nid = " + ccp);
    }

     //add people for the perspectives
    public static void convertPerson(int nid, String name, String username, String color, Statement stmt) throws Exception {
        int entryKind = databaseMgr1.state1.getPERSON();
        nid = changeMgr1.createPerson(name, entryKind, username, username, nid, "", "", "", "", "", "", color, "", "", "", "", stmt);
    }

    //add a new user's person and perspective
    public static void addUser(String first, String last, String color, Statement stmt)  throws Exception {
        if ((nidCompare == 0) | (nidRoot == 0)) {System.out.println("error 0 value in addUser"); }
        int nid = changeMgr1.createPerspective(first + "'s personal", nidRoot, false, stmt);
        String newColor = "#FFFFFF";
        if (color.length() > 1) {newColor = color; }
        convertPerson(nid, first + " " + last, first, newColor, stmt);
        changeMgr1.inheritPerspective(nidCompare, nid, stmt);
    }

    //add some more data needed in all databases
    public static void runStandard() throws Exception {
      try {
        Connection ColabDatabase = databaseMgr1.openDatabase();
        Statement stmt = ColabDatabase.createStatement();

        databaseMgr1.refreshGlobals(stmt);
        changeMgr1.setAuthor(databaseMgr1.state1.getCOLAB_ROOT_AUTHOR());
        int p = databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE();
        int nid = 0;
        //add new statement categories
        changeMgr1.createCategory("comment", databaseMgr1.state1.getSTATEMENT(), false, stmt);
        //note: "comment" is used in GulchContentpane.refreshKindComboBox();
        int headingNid = changeMgr1.createCategory("heading", databaseMgr1.state1.getSTATEMENT(), false, stmt);

        //create heading Statements for Proposals and Discussions
        //but use kind = Proposal or Statement so they only show up in the right perspectives
        int prevNid = databaseMgr1.state1.getCOLAB_ROOT_CONTENT();
        int kind = databaseMgr1.state1.getPROPOSAL();
        nid = changeMgr1.addStatement(p, kind, "PROPOSALS", prevNid, "PROPOSALS", 0, null, false, stmt);
        stmt.executeUpdate("UPDATE links SET seq = '-5' WHERE fromnode = '" + nid + "'" );

        kind = databaseMgr1.state1.getDISCUSSION();
        nid = changeMgr1.addStatement(p, kind, "DISCUSSIONS", prevNid, "DISCUSSIONS", 0, null, false, stmt);
        stmt.executeUpdate("UPDATE links SET seq = '-3' WHERE fromnode = '" + nid + "'" );

        //add standard perspectives hierarchy
        nidRoot = changeMgr1.createPerspective("Group", databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), false, stmt);
        convertPerson(nidRoot, "Group", "Group", "#00FF00", stmt);
        makeCOLAB_ROOT_PERSPECTIVE(nidRoot, stmt);
        //add Gerry's personal perspective
        nid = changeMgr1.createPerspective("Gerry's personal", nidRoot, false, stmt);
        convertPerson(nid, "Gerry Stahl", "Gerry", "#770077", stmt);
        //create the bottom comparison perspective
        nidCompare = changeMgr1.createPerspective("Comparison", nid, false, stmt);
        makeCOLAB_COMPARISON_PERSPECTIVE(nidCompare, stmt);
        convertPerson(nidCompare, "Comparison", "Comparison", "#777777", stmt);
        //create guest and admin perspectives
        addUser("guest", "", "#FFCC66", stmt);
        addUser("admin", "", "#996699", stmt);

        System.out.println("***  done runStandard()");
        ColabDatabase.close();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }

}



