
//Title:        GulchEditing.java
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Gerry Stahl
//Company:      L3D
//Description:  component for editing notes

package gulch2000;

import java.util.*;

public class GulchEditing extends GulchPane {
    String blankLine = "                                                                  ";
    String blankMessage = blankLine + blankLine + blankLine + blankLine + blankLine + blankLine + blankLine;

    /**
    Construct the component as a JavaBean
    */
    public GulchEditing() {
        super();
        System.out.println("WARNING: component called without pointer to parent");
    }

    /**
    Construct the component with a pointer to its parent
    */
    public GulchEditing(GulchTabbedPane pane) {
        super(pane);
    }

    /**
    override dummy splitPaneSetup()
    Set up leftPane as GulchContentPane and rightPane as GulchEditingPane
    */
    void splitPaneSetup() {
        leftPane = new GulchContentPane(this);
        rightPane = new GulchEditingPane(this);
    }

    /**
    override dummy refreshContent()
    Initialize GulchContentPane and GulchEditingPane
    */
    void refreshContent() {
        //initialize GulchContentPane with list of perspectives and kinds
        GulchContentPane contentPane = (GulchContentPane) leftPane;
        contentPane.titleTextArea.setText("");
        contentPane.contentTextArea.setText("");
        contentPane.refreshKindComboBox();
        //initialize GulchEditingPane
        GulchEditingPane editingPane = (GulchEditingPane) rightPane;
        editingPane.msgTextArea.setText(blankMessage);
        editingPane.msgTextArea.setText("messages:  ");
        editingPane.firstTextField.setText(parent.getFirstSelection());
        editingPane.firstPerspectiveTextField.setText(parent.getFirstPerspectiveName());
        String second = parent.getSecondSelection();
        if (second.length() > 0) {
            editingPane.secondTextField.setText(second);
            editingPane.secondPerspectiveTextField.setText(parent.getSecondPerspectiveName());
        }
        else {
            editingPane.secondTextField.setText("");
            editingPane.secondPerspectiveTextField.setText("");
        }
        contentPane.kindComboBox.setEnabled(true);
        contentPane.titleTextArea.setEnabled(true);
        contentPane.contentTextArea.setEnabled(true);
        editingPane.belowRadioButton.setSelected(true);
        editingPane.afterRadioButton.setEnabled(true);
        editingPane.belowRadioButton.setEnabled(true);

        //1. process addDiscussionButton
        if (parent.getButton().equals("addDiscussionButton")) {
            contentPane.setKindComboBox("discussion");
            contentPane.setSelectedKind("discussion");
            contentPane.kindComboBox.setEnabled(false);
            editingPane.belowRadioButton.setSelected(true);
            editingPane.afterRadioButton.setEnabled(false);
            editingPane.belowRadioButton.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Enter a title and content. Then press the OK button.");
        }

        //13. process addProposalButton
        if (parent.getButton().equals("addProposalButton")) {
            contentPane.setKindComboBox("proposal");
            contentPane.setSelectedKind("proposal");
            contentPane.kindComboBox.setEnabled(false);
            editingPane.belowRadioButton.setSelected(true);
            editingPane.afterRadioButton.setEnabled(false);
            editingPane.belowRadioButton.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Enter a title and content. Then press the OK button.");
        }

        //14. process addDecisionButton
        if (parent.getButton().equals("addDecisionButton")) {
            contentPane.setKindComboBox("decision");
            contentPane.setSelectedKind("decision");
            contentPane.kindComboBox.setEnabled(false);
            editingPane.belowRadioButton.setSelected(true);
            editingPane.afterRadioButton.setEnabled(false);
            editingPane.belowRadioButton.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Enter a title and content. Then press the OK button.");
        }

        //12. process addPrivateButton
        if (parent.getButton().equals("addPrivateButton")) {
            contentPane.setKindComboBox("private");
            contentPane.setSelectedKind("private");
            contentPane.kindComboBox.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Enter a title and content. Then press the OK button.");
        }

        //2. process addNewNoteButton
        if (parent.getButton().equals("addNewNoteButton")) {
            //contentPane.setSelectedKind("statement");
            editingPane.msgTextArea.setText("messages: Enter a kind, title and content. " +
                                             "Then choose to add after or below the selected item and press the OK button.");
        }

        //3. process editNoteButton
        if (parent.getButton().equals("editNoteButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.setSelectedKind(parent.getKind());
            editingPane.afterRadioButton.setEnabled(false);
            editingPane.belowRadioButton.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Edit the title and content. Then press the OK button.");
        }

        //4. process deleteNoteButton
        if (parent.getButton().equals("deleteNoteButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.setSelectedKind(parent.getKind());
            contentPane.kindComboBox.setEnabled(false);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.afterRadioButton.setEnabled(false);
            editingPane.belowRadioButton.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Are you sure you want to delete this note in the current perspective? " +
                                             "Then press the OK button.");
        }

        //5. process moveNoteButton  -- after selectSecondButton already pressed
        if (parent.getButton().equals("moveNoteButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.setSelectedKind(parent.getKind());
            contentPane.kindComboBox.setEnabled(false);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Select the After or Below radio buttons and check the first and second items. " +
                                            "Then press the OK button to move the first item to after or below the second item.");
        }

        //6. process copyNoteButton  -- after selectSecondButton already pressed
        if (parent.getButton().equals("copyNoteButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.setSelectedKind(parent.getKind());
            contentPane.kindComboBox.setEnabled(false);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Select the After or Below radio buttons and check the first and second items. " +
                                            "Then press the OK button to make a copy of the first item after or below the second item.");
        }

        //7. process linkNoteButton  -- after selectSecondButton already pressed
        if (parent.getButton().equals("linkNoteButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.kindComboBox.setEnabled(false);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Select the After or Below radio buttons. " +
                                            "Then press the OK button to link the first item to its new parent, the second item.");
        }

        //8. process emailNoteButton
        if (parent.getButton().equals("emailNoteButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.msgTextArea.setText("messages: To send email about this note, press the OK button and fill out the form below.");
        }

        //9. process copyNoteButton  -- after selectSecondButton already pressed
        if (parent.getButton().equals("copyToHomeButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
           contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.setSelectedKind(parent.getKind());
            contentPane.kindComboBox.setEnabled(false);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Select the After or Below radio buttons. " +
                                            "Then press the OK button to make a copy of the first item after or below the second item.");
        }

        //10. process linkNoteButton  -- after selectSecondButton already pressed
        if (parent.getButton().equals("linkToHomeButton")) {
            int nid = parent.getNid();
            //String memo = parent.interfaceMgr1.getMemo(nid);
            Vector v = parent.interfaceMgr1.getDescription(nid);
            String name = (String) v.elementAt(1);
            String memo = (String) v.elementAt(4);
            contentPane.titleTextArea.setText(name);
            contentPane.contentTextArea.setText(memo);
            contentPane.kindComboBox.setEnabled(false);
            contentPane.titleTextArea.setEnabled(false);
            contentPane.contentTextArea.setEnabled(false);
            editingPane.msgTextArea.setText("messages: Select the After or Below radio buttons. " +
                                            "Then press the OK button to link the first item to its new parent, the second item.");
        }

        //11. process selectSecondButton  -- returns to the above move, link, copy options
    }

}

