
//Title:        GulchMap.java
//Version:      
//Copyright:    Copyright (c) 1998
//Author:       Gerry Stahl
//Company:      L3D
//Description:  Component for discussion

package gulch2000;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.sql.Timestamp;
import java.net.*;
import pow2000.TreeDataRow;
import pow2000.InterfaceMessageDef;

public class GulchMap extends GulchPane {
    private int lastNidRead = 0;

    /**
    Construct the component as a JavaBean
    */
    public GulchMap() throws Exception {
        super();
        System.out.println("WARNING: component called without pointer to parent");
    }

    /**
    Construct the component with a pointer to its parent
    */
    public GulchMap(GulchTabbedPane pane) {
        super(pane);
    }

    /**
    override dummy splitPaneSetup()
    Set up leftPane as GulchTreePane and rightPane as GulchMapPane
    */
    void splitPaneSetup() {
        leftPane = new GulchTreePane(this);
        rightPane = new GulchMapPane(this);
   }

    /**
    Override dummy processTreeSelection()
    call cgi script to display memo content in external HTML window
    */
    void processTreeSelection(TreeDataRow theRow) {
        try {
            int theNid = theRow.getNid();
            URL cgiScript = new URL("http://" + InterfaceMessageDef.computerServer + "/cgi-bin/showWebGuidePage.cgi?nid=" + theNid +
                                 "&db=" + InterfaceMessageDef.DBurl);
            if (parent.isApplet()) {
                parent.getTheApp().getAppletContext().showDocument(cgiScript, "target");
            }
            //log that current author read this node
            if (theNid != lastNidRead) {
                lastNidRead = theNid;
                parent.interfaceMgr1.logChange(InterfaceMessageDef.LOG_READ_NOTE, parent.getCurrentPerspective(), theNid, 0);
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    Override dummy refreshContent()
    */
    void refreshContent()  {
        //initialize GulchTreePane with discussion content in tree
        GulchTreePane treePane = (GulchTreePane) leftPane;
        treePane.refreshPerspectiveComboBox();          //fill with list of perspectives and set to current perspective
        treePane.refreshTree();                         //fill with notes visible in current perspective
        //initialize GulchMapPane with author's name
        if (parent.getAuthor() == 0) {return;}
        GulchMapPane discussPane = (GulchMapPane) rightPane;
        discussPane.setAuthorTextField(parent.getAuthorName());
        discussPane.setMessageTextArea("Message: Select a note in the tree. Its content will appear in an HTML window.");
        discussPane.checkPermissions();                  //enable and disable buttons depending on author and perspective
    }

    /**
    Override dummy getRootName()
    set name in root to perspective name
    */
    String getRootName() {
        //String name = parent.interfaceMgr1.getPerspectiveName(parent.getCurrentPerspective());
        //parent.setCurrentPerspectiveName(name);
        return parent.state1.getCurrentPerspectiveName();
    }

    /**
    Override dummy getRootNid()
    set nid in root to COLAB_ROOT_CONTENT
    */
    int getRootNid() {
        return parent.state1.getCOLAB_ROOT_CONTENT();
    }

    /**
    Override dummy getList()
    get theList of visible nodes in p
    */
    Vector getList() {
        GulchMapPane discussPane = (GulchMapPane) rightPane;
        boolean titles = discussPane.titleRadioButton.isSelected();
        int p = parent.state1.getCurrentPerspectiveNid();
        int startNid = parent.state1.getCOLAB_ROOT_CONTENT();
        int mode = parent.state1.getCurrentDisplayMode();
        int viewer = p;
        parent.interfaceMgr1.getTreeOfNodes(p, startNid, mode);      //this resets tree in state1
        Vector v = parent.state1.getTreeOfNodes();
        for (int i = 0; i < v.size(); i++) {
            TreeDataRow row = (TreeDataRow) v.elementAt(i);
            String description = "";
            //compute recency stars
            long recency = 99;
            String dateString = "";
            Timestamp createDate = row.getCreated();
            Timestamp modifyDate = row.getModified();
            if (modifyDate != null) {                    //then use modified date
                dateString = modifyDate.toString().substring(5, 7) + "/" + createDate.toString().substring(8, 10) + "/" + createDate.toString().substring(0, 4);
                recency = System.currentTimeMillis() - modifyDate.getTime();
            }
            else {
                if (createDate != null) {                 //then use created date
                    dateString = createDate.toString().substring(5, 7) + "/" + createDate.toString().substring(8, 10) + "/" + createDate.toString().substring(0, 4);
                    recency = System.currentTimeMillis() - createDate.getTime();
                }                                         //else use defaults for no date
            }
            long recencyDays = recency / 86400000;
            if (recencyDays < 14) { description = description + "*"; }
            if (recencyDays < 7)  { description = description + "*"; }
            if (recencyDays < 2)  { description = description + "*"; }
            //set up description for display
            if (titles) {   //display title, etc.
                if (row.getKindName().length() > 0) {description = description + row.getKindName() + ":  ";}
                description = description + row.getName();
                description = description + "  (";
                if (row.getAuthorName().length() > 0) {description = description + "by: " + row.getAuthorName();}
                if (row.getPerspectiveName().length() > 0) {description = description + " in: " + row.getPerspectiveName();}
                if (dateString.length() > 1) {description = description + " on: " + dateString;}
                description = description + ")";
            }
            else {          //display content
                String content = row.getDescription();
                if (content.length() > 100) {content = content.substring(0, 100);}
                if (content.length() > 0) {description = description + content;}
            }
            row.setDescription(description);
        }
        return v;
    }

}
