/**
*  Title:           GulchTabbedPane for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     tabbed pane interface for GulchApplet and GulchApplication
*/

package gulch2000;

import java.awt.*;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.tree.*;
import pow2000.InterfaceMgrClient;
import pow2000.InterfaceState;
import pow2000.InterfaceMessageDef;

public class GulchTabbedPane extends JPanel {
    //variables for this applet
    private boolean isApplet = false;
    private GulchApp theApp;
    private JFrame theApplication;
    GulchLogin login1;
    GulchDiscuss discuss1;
    GulchEditing editing1;
    GulchPerspectives perspectives1;
    GulchMap map1;
    //GulchCategories categories1;
    //GulchWebsites websites1;
    //GulchPeople people1;
    //GulchGroups groups1;
    InterfaceMgrClient interfaceMgr1;
    public static InterfaceState state1;
    JTabbedPane tabbedPane = new JTabbedPane();

    //store parameters for editing actions between DiscussionPane and EditingPane
    private int kind = 0;
    private Component component = null;
    private String firstSelection = "";
    private String secondSelection = "";
    private int firstPerspectiveNid = 0;
    private String firstPerspectiveName = "";
    private int secondPerspectiveNid = 0;
    private String secondPerspectiveName = "";
    private String button = "";
    private int nid = 0;
    private int secondNid = 0;
    private DefaultMutableTreeNode node = null;
    private DefaultMutableTreeNode secondNode = null;
    private int loginNid = 0;
    private int parentNid = 0;
    private int secondParentNid = 0;
    private String currentServer;
    

    /**
    Construct the Panel as JavaBean
    */
    public GulchTabbedPane() {
        System.out.println("WARNING: component called without pointer to parent");
        try {
            jbInit(InterfaceMessageDef.computerServer, InterfaceMessageDef.DBportValue, InterfaceMessageDef.DBcentralSpace);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        try  {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    /**
    Construct the Panel in a GulchApp
    */
    public GulchTabbedPane(GulchApp anApp, int aLoginNid, String server, int port, String db) {
        try {
            setIsApplet(true);
            setTheApp(anApp);
            setLoginNid(aLoginNid);
            jbInit(server, port, db);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    Construct the Panel in an Application
    */
    public GulchTabbedPane(JFrame anApplication, int aLoginNid, String server, int port, String db) {
        try {
            setIsApplet(false);
            setTheApplication(anApplication);
            setLoginNid(aLoginNid);
            jbInit(server, port, db);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    Initialize the JavaBean
    */
    private void jbInit(String server, int port, String db) {
        //set currentServer
        setCurrentServer(server);
        //initialize client state
        state1 = new InterfaceState();
        //initialize interfaceMgr for client -- with pointer to state1
        interfaceMgr1 = new InterfaceMgrClient(server, port, db, state1);
        //get list of Spaces and update it in state1
        interfaceMgr1.getListOfSpaces();
        //initialize Login tab component
        login1 = new GulchLogin(this);

        //process login if not done by calling webpage
        if (getLoginNid() == 0) {
            tabbedPane.addTab("Login", null, login1, "log in as a user");
            tabbedPane.setSelectedComponent(login1);
        }
        //process auto login if done by calling webpage  -- needs to be revised to do this
        else {
            tabbedPane.addTab("Login", null, login1, "log in as a user");
            tabbedPane.setSelectedComponent(login1);
        }
        this.add(tabbedPane);
    }

    public void authorize(int user) {
        //add discuss component
        discuss1 = new GulchDiscuss(this);
        tabbedPane.addTab("Discussions", null, discuss1, "display discussions in a perspective");
        tabbedPane.setSelectedComponent(discuss1);    //content component appears as default

        //do not add editing component until needed -- using addEdit()
        editing1 = new GulchEditing(this);

        //add perspectives component
//        perspectives1 = new GulchPerspectives(this);
//        tabbedPane.addTab("Perspectives", null, perspectives1, "display information on all perspectives");

        //add InfoMap component graphic display of discussion
        map1 = new GulchMap(this);
        tabbedPane.addTab("Display", null, map1, "display sets of discussion notes");

        //add categories component
//        categories1 = new GulchCategories(this);
//        tabbedPane.addTab("Categories", null, categories1, "display information on all categories");

        //add websites component
//        websites1 = new GulchWebsites(this);
//        tabbedPane.addTab("Websites", null, websites1, "display information on all websites");

        //add people component
//        people1 = new GulchPeople(this);
//        tabbedPane.addTab("People", null, people1, "display information on all people");

        //add groups component
//        groups1 = new GulchGroups(this);
//        tabbedPane.addTab("Groups", null, groups1, "display information on all groups");
    }

    public void unauthorize(int user) {
        state1.setCurrentAuthorNid(0);
        int count = tabbedPane.getTabCount();
        for (int i = count - 1; i > 0; i--) {
            tabbedPane.removeTabAt(i);
        }
        tabbedPane.setSelectedIndex(0);
    }

    void setHourglassCursor() {
        if (isApplet()) {
            getTheApp().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }
        else {
            getTheApplication().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }
    }

    void setDefaultCursor() {
        if (isApplet()) {
            getTheApp().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
        else {
            getTheApplication().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }

    void addEdit() {
        if (tabbedPane.indexOfComponent(editing1) < 0) {
            tabbedPane.addTab("Editing", null, editing1, "edit new or selected note");
        }
        editing1.refreshContent();
        tabbedPane.setSelectedComponent(editing1);
    }

    void removeEdit() {
        int index = tabbedPane.indexOfComponent(editing1);
        if (index > -1) {
            tabbedPane.removeTabAt(index);
        }
    }

    // JavaBean property getter and setter methods

    public void setHomePerspective(int newHomePerspective) {
        state1.setHomePerspectiveNid(newHomePerspective);
    }

    public int getHomePerspective() {
        return state1.getHomePerspectiveNid();
    }

    public void setCurrentPerspective(int newCurrentPerspective) {
        state1.setCurrentPerspectiveNid(newCurrentPerspective);
    }

    public int getCurrentPerspective() {
        return state1.getCurrentPerspectiveNid();
    }

    public void setHomePerspectiveName(String newHomePerspectiveName) {
        state1.setHomePerspectiveName(newHomePerspectiveName);
    }

    public String getHomePerspectiveName() {
        return state1.getHomePerspectiveName();
    }

    public void setCurrentPerspectiveName(String newCurrentPerspectiveName) {
        state1.setCurrentPerspectiveName(newCurrentPerspectiveName);
    }

    public String getCurrentPerspectiveName() {
        return state1.getCurrentPerspectiveName();
    }

    public void setKind(int newKind) {
        kind = newKind;
    }

    public int getKind() {
        return kind;
    }

    public void setComponent(Component newComponent) {
        component = newComponent;
    }

    public Component getComponent() {
        return component;
    }

    public void setButton(String newButton) {
        button = newButton;
    }

    public String getButton() {
        return button;
    }

    public void setNode(DefaultMutableTreeNode newNode) {
        node = newNode;
    }

    public DefaultMutableTreeNode getNode() {
        return node;
    }

    public void setSecondNode(DefaultMutableTreeNode newSecondNode) {
        secondNode = newSecondNode;
    }

    public DefaultMutableTreeNode getSecondNode() {
        return secondNode;
    }

    public void setSecondNid(int newSecondNid) {
        secondNid = newSecondNid;
    }

    public int getSecondNid() {
        return secondNid;
    }

    public void setNid(int newNid) {
        nid = newNid;
    }

    public int getNid() {
        return nid;
    }

    public void setLoginNid(int newNid) {
        loginNid = newNid;
    }

    public int getLoginNid() {
        return loginNid;
    }

    public void setIsApplet(boolean newIsApplet) {
        isApplet = newIsApplet;
    }

    public boolean isApplet() {
        return isApplet;
    }

    public void setTheApplication(JFrame newTheApplication) {
        theApplication = newTheApplication;
    }

    public JFrame getTheApplication() {
        return theApplication;
    }

    public void setAuthor(int newAuthor) {
        state1.setCurrentAuthorNid(newAuthor);
    }

    public int getAuthor() {
        return state1.getCurrentAuthorNid();
    }

    public void setAuthorName(String newAuthorName) {
        state1.setCurrentAuthorName(newAuthorName);
    }

    public String getAuthorName() {
        return state1.getCurrentAuthorName();
    }

    public void setTheApp(GulchApp newTheApp) {
        theApp = newTheApp;
    }

    public GulchApp getTheApp() {
        return theApp;
    }

    public void setFirstSelection(String newFirstSelection) {
        firstSelection = newFirstSelection;
    }

    public String getFirstSelection() {
        return firstSelection;
    }

    public void setSecondSelection(String newSecondSelection) {
        secondSelection = newSecondSelection;
    }

    public String getSecondSelection() {
        return secondSelection;
    }

    public void setFirstPerspectiveNid(int newFirstPerspectiveNid) {
        firstPerspectiveNid = newFirstPerspectiveNid;
    }

    public int getFirstPerspectiveNid() {
        return firstPerspectiveNid;
    }

    public void setSecondPerspectiveNid(int newSecondPerspectiveNid) {
        secondPerspectiveNid = newSecondPerspectiveNid;
    }

    public int getSecondPerspectiveNid() {
        return secondPerspectiveNid;
    }

    public void setFirstPerspectiveName(String newFirstPerspectiveName) {
        firstPerspectiveName = newFirstPerspectiveName;
    }

    public String getFirstPerspectiveName() {
        return firstPerspectiveName;
    }

    public void setSecondPerspectiveName(String newSecondPerspectiveName) {
        secondPerspectiveName = newSecondPerspectiveName;
    }

    public String getSecondPerspectiveName() {
        return secondPerspectiveName;
    }

    public boolean isIsApplet() {
        return isApplet;
    }

    public void setParentNid(int newParentNid) {
        parentNid = newParentNid;
    }

    public int getParentNid() {
        return parentNid;
    }

    public void setSecondParentNid(int newSecondParentNid) {
        secondParentNid = newSecondParentNid;
    }

    public int getSecondParentNid() {
        return secondParentNid;
    }

    private void jbInit() throws Exception {
        this.setBackground(Color.magenta);
    }

    public void setCurrentServer(String newCurrentServer) {
        currentServer = newCurrentServer;
    }

    public String getCurrentServer() {
        return currentServer;
    }

}

