/**
*  Title:           ChangeMgr for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     manage changes to data
*/

package pow2000;

import java.sql.*;
import java.util.*;

public class ChangeMgr {
    DatabaseMgr databaseMgr1;
    private boolean sibling = true;
    private boolean child = false;

    public ChangeMgr() throws Exception {
        databaseMgr1 = new DatabaseMgr();
        jbInit();
    }

    public ChangeMgr(DatabaseMgr newDatabaseMgr) throws Exception {
        databaseMgr1 = newDatabaseMgr;
        jbInit();
    }

    private void jbInit() {
    }

    public void setAuthor(int newAuthor) {
        databaseMgr1.state1.setCurrentAuthorNid(newAuthor);   //current author stored in InterfaceState
    }

    public int getAuthor() {
        return databaseMgr1.state1.getCurrentAuthorNid();
    }

    /**
    Surround a String with single quotes for concatenation in an SQL query.
    First escape code single quotes already in the String
    */
    public String squote(String s) {
        return "'" + databaseMgr1.encode(s) + "'";
    }

    /**
    Pack up data in a NodeDataRow
    */
    public NodeDataRow packARow(int kind, int p, String name, String description, int author, Timestamp created) {
        NodeDataRow newRow = new NodeDataRow();
        int editor = getAuthor();
        newRow.setEditor(editor);
        newRow.setAuthor(author);
        newRow.setCreated(created);
        newRow.setModified(null);
        newRow.setKind(kind);
        newRow.setPerspective(p);
        newRow.setName(name);
        newRow.setDescription(description);
        return newRow;
    }

    /**
    Set up data for addItem()
    */
    public int addAnItem(int kind, int p, String name, String description, int prevNid, int author, Timestamp created, Statement stmt) throws Exception {
        NodeDataRow newRow = packARow(kind, p, name, description, author, created);
        return addItem(prevNid, newRow, p, stmt);
    }

    /**
    addItem() modifies the nodes and links tables of the database
     to reflect that item newRow was added immediately after (and under the same parent) previousRow in perspective p.
    */
    public int addItem(int previousNid, NodeDataRow newRow, int p, boolean after, Statement stmt) throws Exception {
        int newNid = 0;
        if (after) {
            newNid = addItem(previousNid, newRow, p, stmt);
        }
        else {
            newNid = addSubItem(previousNid, newRow, p, stmt);
        }
        return newNid;
    }

    /**
    addItem() modifies the nodes and links tables of the database
     to reflect that item newRow was added immediately after (and under the same parent) previousRow in perspective p.
    */
    public int addItem(int previousNid, NodeDataRow newRow, int p, Statement stmt) throws Exception {
        int newNid = 0;
        //add newRow to nodes table
        newNid = databaseMgr1.addNode(newRow, stmt);
        //find parents and seq of previousRow and link newRow to parent at newseq = seq + 1
        Vector theParents = databaseMgr1.getParentNodes(previousNid, p, stmt);     //find this node's parents and sequence among its children
        for (int i = 0; i < theParents.size(); i++) {
            ParentSeqRow psRow = (ParentSeqRow) theParents.elementAt(i);
            int parentNid = psRow.getParent();
            if (parentNid == 0) {databaseMgr1.db("no parent found for " + previousNid + "persp: " + p);}
            int previousSeq = psRow.getSequence();
            LinkDataRow linkRow = new LinkDataRow(0, databaseMgr1.state1.getPARENT(), p, newNid, parentNid, previousSeq + 1, getAuthor(), null);
            //for all children of parent with seq > previousSeq, seq++
            String query = "UPDATE links SET seq = seq + 1 " +
                               "WHERE type = " + databaseMgr1.state1.getPARENT() + " AND tonode = " + parentNid +
                               " AND perspective = " + p + " AND seq > " + previousSeq;
            databaseMgr1.updateLinks(query, stmt);
            databaseMgr1.addLink(linkRow, stmt);
        }
        return newNid;
    }

    /**
    Set up data for addSubitem
    Pass author=0 and created=null for new node
    public int addASubItem(int kind, int p, String name, String description, int parentNid, int author, Timestamp created, Statement stmt) throws Exception {
        NodeDataRow newRow = new NodeDataRow();
        int editor = getAuthor();
        if (author == 0) { setAuthor(editor); }
        newRow.setEditor(editor);
        newRow.setKind(kind);
        newRow.setPerspective(p);
        newRow.setName(name);
        newRow.setDescription(description);
        return addSubItem(parentNid, newRow, p, stmt);
    }
    */

    /**
    addSubItem() modifies the nodes and links tables of the database
    to reflect that a sub-item newRow was added as the first child of parentRow in perspective p.
    */
    public int addSubItem(int parentNid, NodeDataRow newRow, int p, Statement stmt) throws Exception {
        int newNid = 0;
        //add newRow to nodes table
        newNid = databaseMgr1.addNode(newRow, stmt);
        //link newRow to parentRow at newseq = greatest seq + 1
        int PARENT = databaseMgr1.state1.getPARENT();
        int author = getAuthor();
        Timestamp create = null;       //dummy timestamp
        //for all children of parent set seq++
        String query = "UPDATE links SET seq = seq + 1 WHERE type = " + PARENT + " AND perspective = " + p + " AND tonode = " + parentNid;
        databaseMgr1.updateLinks(query, stmt);
        //link newNid as first child of parentNid
        LinkDataRow linkRow = new LinkDataRow(0, PARENT, p, newNid, parentNid, 1, author, create);
        int newLid = databaseMgr1.addLink(linkRow, stmt);
        return newNid;
    }

    /**
    Set up data for editItem
    */
    public int editAnItem(int kind, int p, String name, String description, int oldNid, Statement stmt) throws Exception {
        NodeDataRow newRow = new NodeDataRow();
        int author = getAuthor();
        newRow.setAuthor(author);
        newRow.setKind(kind);
        newRow.setPerspective(p);
        newRow.setName(name);
        newRow.setDescription(description);
        return editItem(oldNid, newRow, p, stmt);
    }

    /**
    editItem() modifies the nodes and links tables of the database
    to reflect that a given item was edited in perspective p.
    Check for VCOPYs in other perspectives and reflect the edit there too
    */
    public int editItem(int oldNid, NodeDataRow newRow, int p, Statement stmt) throws Exception {
        int newNid = 0;
        //add newRow to nodes table
        newNid = databaseMgr1.addNode(newRow, stmt);
        //link editing from oldRow to newRow
        LinkDataRow linkRow = new LinkDataRow();
        int author = getAuthor();
        linkRow.setAuthor(author);
        linkRow.setType(databaseMgr1.state1.getEDIT());         //type = "edit"
        linkRow.setPerspective(p);                       //perspective = p
        linkRow.setFrom(oldNid);                         //from = oldRow Nid
        linkRow.setTo(newNid);                           //to = newRow Nid
        int newLid = databaseMgr1.addLink(linkRow, stmt);      //lid set to maxLid
        //find all parent links to oldNid, then add new parent links to newNid in p
        //only add links that existed in p or in its ancestors
        int thePARENT = databaseMgr1.state1.getPARENT();

        Vector v = databaseMgr1.perspectiveInheritance(p, stmt);
        String list = "(";
        if (v.size() > 0) {
            linkRow = (LinkDataRow) v.elementAt(0);
            list = list + "" + linkRow.getLid() + "";
        }
        for (int i = 1; i < v.size(); i++) {
            linkRow = (LinkDataRow) v.elementAt(i);
            list = list + ", " + linkRow.getLid() + "";
        }
        list = list + ")";

        String theQuery = "SELECT * FROM links WHERE type = " + thePARENT + " AND tonode = " + oldNid +
                          " AND perspective IN " + list;
        ResultSet rs = stmt.executeQuery(theQuery);
        v = new Vector();
        while (rs.next() ) {
            linkRow = new LinkDataRow(rs);
            v.addElement(linkRow);                           //store linkRows from query
        }
        for (int i = 0; i < v.size(); i++) {                 //now go thru all the linkRows
            linkRow = (LinkDataRow) v.elementAt(i);
            if (databaseMgr1.linkNotDeleted(linkRow.getFrom(), linkRow.getTo(), linkRow.getPerspective(), p, stmt)) {
                linkRow.setAuthor(author);                       //set author to current editor
                linkRow.setPerspective(p);                       //set perspective to current p
                linkRow.setTo(newNid);                           //set tonode to newNid
                databaseMgr1.addLink(linkRow, stmt);             //save new link in addition to old one
            }
        }
        //find all children links from oldNid, then add new children links from newNid in p
        //only add links that existed in p or in its ancestors
        theQuery = "SELECT * FROM links WHERE type = " + thePARENT + " and fromnode = " + oldNid +
                          " AND perspective IN " + list;
        rs = stmt.executeQuery(theQuery);
        v = new Vector();
        while (rs.next() ) {
            linkRow = new LinkDataRow(rs);
            v.addElement(linkRow);                               //store linkRows from query
        }
        for (int i = 0; i < v.size(); i++) {                     //now go thru all the linkRows
            linkRow = (LinkDataRow) v.elementAt(i);              //check for deleteLink
            if (databaseMgr1.linkNotDeleted(linkRow.getFrom(), linkRow.getTo(), linkRow.getPerspective(), p, stmt)) {
                linkRow.setAuthor(author);                       //set author to current editor
                linkRow.setPerspective(p);                       //set perspective to current p
                linkRow.setFrom(newNid);                         //set tonode to newNid
                databaseMgr1.addLink(linkRow, stmt);             //save new link in addition to old one
            }
        }

        //search for VCOPY of oldNid and process each one
        theQuery = "SELECT * FROM links WHERE type = " + databaseMgr1.state1.getVCOPY() + " and fromnode = " + oldNid;
        rs = stmt.executeQuery(theQuery);
        v = new Vector();
        while (rs.next() ) {
            linkRow = new LinkDataRow(rs);
            v.addElement(linkRow);                               //store linkRows from query
        }
        for (int i = 0; i < v.size(); i++) {                     //now go thru all the linkRows
            linkRow = (LinkDataRow) v.elementAt(i);              //check for VCOPY
            VCopyEdit(oldNid, newNid, linkRow.getPerspective(), stmt);
        }
        return newNid;
    }

    /**
    VCopyEdit() modifies the nodes and links tables of the database
    for a VCOPY of an item that was edited.
    */
    public void VCopyEdit(int oldNid, int newNid, int p, Statement stmt) throws Exception {
        //do not process if newNid already is VCPYed or EDITed or DELETEd in p
        ResultSet rs;
        rs = stmt.executeQuery("SELECT * FROM links WHERE type = " + databaseMgr1.state1.getVCOPY() + " AND fromnode = " + newNid +
                                  " AND perspective = " + p);
        if (rs.next() ) {return; }
        rs = stmt.executeQuery("SELECT * FROM links WHERE type = " + databaseMgr1.state1.getEDIT() + " AND tonode = " + newNid +
                                  " AND perspective = " + p);
        if (rs.next() ) {return; }
        rs = stmt.executeQuery("SELECT * FROM links WHERE type = " + databaseMgr1.state1.getDELETE() + " AND fromnode = " + newNid +
                                  " AND perspective = " + p);
        if (rs.next() ) {return; }


        //link EDITing from oldRow to newRow in p
        LinkDataRow linkRow = new LinkDataRow();
        int author = getAuthor();
        linkRow.setAuthor(author);
        linkRow.setType(databaseMgr1.state1.getEDIT());         //type = "EDIT"
        linkRow.setPerspective(p);                       //perspective = p
        linkRow.setFrom(oldNid);                         //from = oldRow Nid
        linkRow.setTo(newNid);                           //to = newRow Nid
        databaseMgr1.addLink(linkRow, stmt);
        //link VCOPY of newRow in p
        linkRow = new LinkDataRow();
        linkRow.setAuthor(author);
        linkRow.setType(databaseMgr1.state1.getVCOPY());         //type = "VCOPY"
        linkRow.setPerspective(p);
        linkRow.setFrom(newNid);
        linkRow.setTo(0);
        databaseMgr1.addLink(linkRow, stmt);
        //find all parent links to oldNid, then add new parent links to newNid in p
        //only add links that existed in p or in its ancestors
        int thePARENT = databaseMgr1.state1.getPARENT();
        Vector v = databaseMgr1.perspectiveInheritance(p, stmt);
        String list = "(";
        if (v.size() > 0) {
            linkRow = (LinkDataRow) v.elementAt(0);
            list = list + "" + linkRow.getLid() + "";
        }
        for (int i = 1; i < v.size(); i++) {
            linkRow = (LinkDataRow) v.elementAt(i);
            list = list + ", " + linkRow.getLid() + "";
        }
        list = list + ")";
        String theQuery = "SELECT * FROM links WHERE type = " + thePARENT + " AND tonode = " + oldNid +
                          " AND perspective IN " + list;
        rs = stmt.executeQuery(theQuery);
        v = new Vector();
        while (rs.next() ) {
            linkRow = new LinkDataRow(rs);
            v.addElement(linkRow);                           //store linkRows from query
        }
        for (int i = 0; i < v.size(); i++) {                 //now go thru all the linkRows
            linkRow = (LinkDataRow) v.elementAt(i);
            if (databaseMgr1.linkNotDeleted(linkRow.getFrom(), linkRow.getTo(), linkRow.getPerspective(), p, stmt)) {
                linkRow.setAuthor(author);                       //set author to current editor
                linkRow.setPerspective(p);                       //set perspective to current p
                linkRow.setTo(newNid);                           //set tonode to newNid
                databaseMgr1.addLink(linkRow, stmt);             //save new link in addition to old one
            }
        }
        //find all children links from oldNid, then add new children links from newNid in p
        //only add links that existed in p or in its ancestors
        theQuery = "SELECT * FROM links WHERE type = " + thePARENT + " and fromnode = " + oldNid +
                          " AND perspective IN " + list;
        rs = stmt.executeQuery(theQuery);
        v = new Vector();
        while (rs.next() ) {
            linkRow = new LinkDataRow(rs);
            v.addElement(linkRow);                               //store linkRows from query
        }
        for (int i = 0; i < v.size(); i++) {                     //now go thru all the linkRows
            linkRow = (LinkDataRow) v.elementAt(i);              //check for deleteLink
            if (databaseMgr1.linkNotDeleted(linkRow.getFrom(), linkRow.getTo(), linkRow.getPerspective(), p, stmt)) {
                linkRow.setAuthor(author);                       //set author to current editor
                linkRow.setPerspective(p);                       //set perspective to current p
                linkRow.setFrom(newNid);                         //set tonode to newNid
                databaseMgr1.addLink(linkRow, stmt);             //save new link in addition to old one
            }
        }
    }

    /**
    deleteItem() modifies the links table of the database
     to reflect that a given item was deleted in perspective p.
    */
    public int deleteItem(int oldNid, int p, Statement stmt) throws Exception {
        int newLid = 0;
        //link deleting from oldRow
        LinkDataRow linkRow = new LinkDataRow();
        int author = getAuthor();
        linkRow.setAuthor(author);
        linkRow.setType(databaseMgr1.state1.getDELETE());       //type = "delete"
        linkRow.setPerspective(p);                       //perspective = p
        linkRow.setFrom(oldNid);                         //from = oldRow Nid
        newLid = databaseMgr1.addLink(linkRow, stmt);    //lid set to maxLid
        return newLid;
    }

    /**
    moveItem() modifies the nodes and links tables of the database
    to reflect that theNid in p1 is moved after/below thePrev in perspective p2.
    Normal move is within p1; promote deletes in p1 and adds in p2.
    */
    public int moveItem(int theNid, int oldParent, int thePrev, int thePrevParent, int p1, int p2, boolean after, Statement stmt) throws Exception {
        //first signal that the old parent link is deleted in p1
        //do not delete links in other perspectives because
        //they may be needed -- the node is only being moved away in p1!
        int newNid = theNid;
        String theQuery = "";
        LinkDataRow linkRow = new LinkDataRow();
        linkRow.setType(databaseMgr1.state1.getDELETELINK());
        linkRow.setFrom(theNid);
        linkRow.setTo(oldParent);
        linkRow.setPerspective(p1);
        linkRow.setAuthor(getAuthor());
        databaseMgr1.addLink(linkRow, stmt);
        if (p1 == p2) {       //moving within one perspective
            //add new parent link
            linkRow = new LinkDataRow();
            linkRow.setType(databaseMgr1.state1.getPARENT());
            linkRow.setFrom(theNid);
            if (after) {
                linkRow.setTo(thePrevParent);
                int previousSeq = 1;
                Vector theParents = databaseMgr1.getParentNodes(thePrev, p1, stmt);     //find this node's parents and sequence among its children
                for (int i = 0; i < theParents.size(); i++) {
                    ParentSeqRow psRow = (ParentSeqRow) theParents.elementAt(i);
                    if (thePrevParent == psRow.getParent()) {
                        previousSeq = psRow.getSequence();
                    }
                }                                         
                linkRow.setSequence(previousSeq + 1);
                //for all children of parent with seq > previousSeq, seq++
                theQuery = "UPDATE links SET seq = seq + 1 " +
                               "WHERE type = " + databaseMgr1.state1.getPARENT() + " AND tonode = " + thePrevParent +
                               " AND perspective = " + p1 + " AND seq > " + previousSeq;
                databaseMgr1.updateLinks(theQuery, stmt);
            }
            else {
                linkRow.setTo(thePrev);
                linkRow.setSequence(1);
                //for all children of parent seq++
                theQuery = "UPDATE links SET seq = seq + 1 " +
                               "WHERE type = " + databaseMgr1.state1.getPARENT() + " AND tonode = " + thePrevParent +
                               " AND perspective = " + p1;
                databaseMgr1.updateLinks(theQuery, stmt);
            }
            linkRow.setPerspective(p1);
            linkRow.setAuthor(getAuthor());
            databaseMgr1.addLink(linkRow, stmt);
        }
        else {                //promoting from p1 to p2
            //delete theNid in p1
            deleteItem(theNid, p1, stmt);
            //add another copy of theNid in p2 in the nodes table and link this new copy to its new parent in p2
            NodeDataRow nodeRow = databaseMgr1.getNodeRow(theNid, stmt);
            nodeRow.setPerspective(p2);
            newNid = addItem(thePrev, nodeRow, p2, after, stmt);
            //duplicate all children links of theNid, to the new copy -- in same p's
            theQuery = "SELECT * FROM links WHERE type = " + databaseMgr1.state1.getPARENT() + " AND tonode = " + theNid;
            ResultSet rs = stmt.executeQuery(theQuery);
            while (rs.next() ) {
                linkRow = new LinkDataRow(rs);
                linkRow.setTo(newNid);                           //set tonode to newNid
                databaseMgr1.addLink(linkRow, stmt);             //save new link in addition to old one
            }
            //duplicate all DELETE links of theNid, to the new copy -- in same p's
            theQuery = "SELECT * FROM links WHERE type = " + databaseMgr1.state1.getDELETE() + " AND fromnode = " + theNid;
            rs = stmt.executeQuery(theQuery);
            while (rs.next() ) {
                linkRow = new LinkDataRow(rs);
                linkRow.setFrom(newNid);                         //set fromnode to newNid
                databaseMgr1.addLink(linkRow, stmt);             //save new link in addition to old one
            }
        }

        return newNid;
    }

    /**
    copyStatement() modifies the nodes, links, and statements tables of the database
    to reflect that theNid is copied after/below thePrev in perspective p.
    */
    public int copyStatement(int theNid, int thePrev, int p, boolean after, Statement stmt) throws Exception {
        int copyNid = 0;
        //add newRow to nodes table and statements
        NodeDataRow newRow = databaseMgr1.getNodeRow(theNid, stmt);
        int kind = newRow.getKind();
        String name = newRow.getName();
        String memo = databaseMgr1.getMemo(theNid, stmt);
        copyNid = addStatement(p, kind, name, thePrev, memo, newRow.getAuthor(), newRow.getCreated(), after, stmt);
        return copyNid;
    }

    /**
    Link two items -- child in p1 and parent in p2 -- link them in p2
    VCOPY child into p2 if p1 != p2
    */
    public int linkItem(int childNid, int prevNid, int p1, int p2, boolean after, Statement stmt) throws Exception {
        LinkDataRow linkRow;
        int returnInt = 0;
        if (p1 != p2) {
            linkRow = new LinkDataRow(0, databaseMgr1.state1.getVCOPY(), p2, childNid, 0, 1, getAuthor(), null);
            databaseMgr1.addLink(linkRow, stmt);
        }
        if (childNid != prevNid) {       //do not link a node to itself!
            if (after) {
                //find parents and seq of previousRow and link newRow to parent at newseq = seq + 1
                Vector theParents = databaseMgr1.getParentNodes(prevNid, p2, stmt);     //find this node's parents and sequence among its children
                for (int i = 0; i < theParents.size(); i++) {
                    ParentSeqRow psRow = (ParentSeqRow) theParents.elementAt(i);
                    int parentNid = psRow.getParent();
                    if (parentNid == 0) {databaseMgr1.db("no parent found for " + prevNid + "persp: " + p2);}
                    int previousSeq = psRow.getSequence();
                    linkRow = new LinkDataRow(0, databaseMgr1.state1.getPARENT(), p2, childNid, parentNid, previousSeq + 1, getAuthor(), null);
                    //for all children of parent with seq > previousSeq, seq++
                    String query = "UPDATE links SET seq = seq + 1 " +
                                       "WHERE type = " + databaseMgr1.state1.getPARENT() + " AND tonode = " + parentNid +
                                       " AND perspective = " + p2 + " AND seq > " + previousSeq;
                    databaseMgr1.updateLinks(query, stmt);
                    returnInt = databaseMgr1.addLink(linkRow, stmt);
                }
            }
            else {
                linkRow = new LinkDataRow(0, databaseMgr1.state1.getPARENT(), p2, childNid, prevNid, 1, getAuthor(), null);
                returnInt = databaseMgr1.addLink(linkRow, stmt);
            }
        }
        return returnInt;
    }

/** ============================== methods for contents =======================
    STATEMENTS, DISCUSSIONS, DECISIONS, BOOKS, MULTIMEDIA, OBJECTS, WEBSITES
    Entries in these tables need to have corresponding entries in NODES and LINKS.
    Entries in these tables should be created and deleted in the currently active perspective.
    Entries in these tables should be linked to other content entries.
*/

    /**
    createContent()
    First create the new row in the auxiliary table and then
    call createContent(), passing it the table name (in lower case), the id of the new row, the nodes table row,
    the preceeding/parent node's Nid, the current perspective, and true for sibling or false for child after/below.
    */
    public int createContent(String tableName, int tableRowId, NodeDataRow nodeRow, int prevNid, int p, boolean after, Statement stmt) throws Exception {
        int tableId = 0;
        int TABLEkind = databaseMgr1.state1.getTABLE();
        String tableTitle = tableName.toUpperCase() + "_TABLE";
        ResultSet rs = stmt.executeQuery("SELECT nid FROM nodes WHERE kind = " + TABLEkind + " AND name = '" + tableTitle + "'");
        if (rs.next()) {tableId = rs.getInt(1);}
        //create row in nodes tables
        nodeRow.setSuptable(tableId);
        nodeRow.setContent(tableRowId);
        if (nodeRow.getAuthor() == 0) { nodeRow.setAuthor(getAuthor()); }
        nodeRow.setEditor(getAuthor());
        //store row in nodes and links tables
        int newNid = 0;
        if (after) {
            newNid = addItem(prevNid, nodeRow, p, stmt);
        }
        else {
            newNid = addSubItem(prevNid, nodeRow, p, stmt);
        }
        //put pointer back to nodes row in content table
        databaseMgr1.updateTable("UPDATE " + tableName + " SET nid = " + newNid + " WHERE id = " + tableRowId, tableName, stmt);
        return newNid;
    }

    /**
    editContent()
    Call edititem() passing it a NodeDataRow.
    editContent() should be called after saving the content in its auxilliary table.
    Revise pointer.
    */
    public int editContent(int kind, int p, String tableName, int contentId, String name, String description, int oldNid, Statement stmt) throws Exception {
        //compute tableId
        int tableId = 0;
        int TABLEkind = databaseMgr1.state1.getTABLE();
        String tableTitle = tableName.toUpperCase() + "_TABLE";
        ResultSet rs = stmt.executeQuery("SELECT nid FROM nodes WHERE kind = " + TABLEkind + " AND name = '" + tableTitle + "'");
        if (rs.next()) {tableId = rs.getInt(1);}
        NodeDataRow oldRow = databaseMgr1.getNodeRow(oldNid, stmt);
        //create row in nodes tables and link up in links table
        NodeDataRow newRow = new NodeDataRow();
        newRow.setKind(kind);
        newRow.setPerspective(p);
        newRow.setSuptable(tableId);
        newRow.setContent(contentId);
        newRow.setName(name);
        newRow.setDescription(description);
        newRow.setEditor(getAuthor());
        newRow.setAuthor(oldRow.getAuthor());
        newRow.setCreated(oldRow.getCreated());
        newRow.setModified(null);
        int newNid = editItem(oldNid, newRow, p, stmt);

        //put pointer back to nodes row in content table
        databaseMgr1.updateTable("UPDATE " + tableName + " SET nid = " + newNid + " WHERE id = " + contentId, tableName, stmt);
        return newNid;
    }

    /**
    deleteContent()
    Call deleteContent(), passing it the id of the row in the nodes table.
    Deleted information should only be flagged as deleted (in a perspective) in the links table.
    */
    public int deleteContent(int oldNid, int p, Statement stmt) throws Exception {
        return deleteItem(oldNid, p, stmt);
    }

    /**
    clipStatement() --  Compute a 255 character abstract.
    */
    public String clipStatement(String memo) {
        String description = memo;
        if (description.length() > 255) {
            int index = description.substring(0, 249).lastIndexOf(" ");
            description = description.substring(0, index) + " . . .";
        }
        return memo;
    }

    /**
    addStatement() --  add an entry for a textual statement. First compute a 255 character abstract.
    Then call createStatement () to create row in STATEMENTS table with long text and abstract in nodes table.
    */
    public int addStatement(int p, int kind, String name, int prevNid, String memo, int author, Timestamp created, boolean after, Statement stmt) throws Exception {
        if (kind == 0) { kind =  databaseMgr1.state1.getSTATEMENT(); }
        if (name.length() == 0) {
            name = "no name";
            if (kind == databaseMgr1.state1.getSTATEMENT())  {name = "Statement"; }
            if (kind == databaseMgr1.state1.getDISCUSSION()) {name = "Discussion"; }
            if (kind == databaseMgr1.state1.getPROPOSAL())   {name = "Proposal"; }
            if (kind == databaseMgr1.state1.getDECISION())   {name = "Decision"; }
            if (kind == databaseMgr1.state1.getPRIVATE())    {name = "Private comment"; }
        }
        String description = clipStatement(memo);
        return createStatement(kind, p, name, description, prevNid, memo, author, created, after, stmt);
    }

    /**
    createStatement() -- create row in STATEMENTS table with long text, then call createContent() with 250 character abstract
    */
    public int createStatement(int kind, int p, String name, String description, int prevNid, String memo, int author, Timestamp created, boolean after, Statement stmt) throws Exception {
        //first create row in STATEMENTS table
        int tableRow = 0;
        tableRow = databaseMgr1.updateTable("INSERT INTO statements VALUES (0, 0, " + squote(memo) + ")", "statements", stmt);
        NodeDataRow nodeRow = packARow(kind, p, name, description, author, created);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createContent("statements", tableRow, nodeRow, prevNid, p, after, stmt);
    }

    /**
    editStatement()
    Call editContent(), passing it the id of the row in the auxilliary table.
    */
    public int editStatement(int kind, int p, String name, String memo, int oldNid, Statement stmt) throws Exception {
        //first create row in STATEMENTS table
        int tableRow = 0;
        String description = clipStatement(memo);
        if (name.length() == 0) {name = "Text"; }
        tableRow = databaseMgr1.updateTable("INSERT INTO statements VALUES (0, 0, " + squote(memo) + ")", "statements", stmt);
        return editContent(kind, p, "statements", tableRow, name, description, oldNid, stmt);
    }

    /**
    addDiscussion()
    GroupNid points to an anchor visible in inherited Group perspective
    prevNid points to note this is a reponse to
    Copy prevNid in Group with kind=Discussion if not in Group and link newNid to it
    */
    public int addDiscussion(int p, String name, int prevNid, int groupNid, String memo, boolean after, Statement stmt) throws Exception {
        int kind =  databaseMgr1.state1.getDISCUSSION();
        //find Discussions statement
        int discussNid = 0;
        ResultSet rs = stmt.executeQuery("SELECT nid FROM nodes WHERE name = 'DISCUSSIONS' AND nid < " + 100);
        if (rs.next()) {
            discussNid = rs.getInt(1);
        }
        if (discussNid == 0) {
            System.out.println("Warning: Discussion note not found in ChangeMgr.");
            return 0;
        }
        //check for discussion of root -- illegal
        if (prevNid == databaseMgr1.state1.getCOLAB_ROOT_CONTENT() ) {
            System.out.println("Warning: cannot add discussion to the root note.");
            return(0);
        }
        //link to "DISCUSSIONS" as default
        if (prevNid  == 0) { prevNid  = discussNid; }
        if (groupNid == 0) { groupNid = discussNid; }
        //if prevNid in Group, just add new Statement
        if (prevNid == groupNid) {
            return addStatement(p, kind, name, prevNid, memo, 0, null, after, stmt);
        }
        else {            //else copy prevNid node into Group with kind=Discussion linked to groupNid
            NodeDataRow prevRow = databaseMgr1.getNodeRow(prevNid, stmt);
            String prevMemo = databaseMgr1.getMemo(prevNid, stmt);
            int newNid = addStatement(p, kind, prevRow.getName(), groupNid, prevMemo, prevRow.getAuthor(), prevRow.getCreated(), after, stmt);
//databaseMgr1.logChange(16, p, newNid, 5, 5, stmt);
            return addStatement(p, kind, name, newNid, memo, 0, null, after, stmt);
        }
    }

    /**
    addDiscussion()

    Add an entry for a Discussion statement. First compute a 255 character abstract.
    Then call createStatement () to create row in STATEMENTS table with long text and abstract in nodes table.
    GroupNid points to an anchor visible in inherited Group perspective
    prevNid points to note this is a reponse to -- Copy prevNid with kind=Discussion if not in Group and link newNid to it
    */
/**
    public int addDiscussion(int p, String name, int prevNid, int groupNid, String memo, boolean after, Statement stmt) throws Exception {
        int kind =  databaseMgr1.state1.getDISCUSSION();
        //find Discussions statement
        int discussNid = 0;
        ResultSet rs = stmt.executeQuery("SELECT nid FROM nodes WHERE name = 'Discussions' AND nid < " + 100);
        if (rs.next()) {
            discussNid = rs.getInt(1);
        }
        if (discussNid == 0) {
            System.out.println("Warning: Discussion note not found in ChangeMgr.");
            return 0;
        }
        //check for discussion of root -- illegal
        if (prevNid == databaseMgr1.state1.getCOLAB_ROOT_CONTENT() ) {
            System.out.println("Warning: cannot add discussion to the root note.");
            return(0);
        }
        if (prevNid == 0) { prevNid = discussNid; }
        else if (groupNid == 0) { groupNid = discussNid; }
        //see if prevNid exists in Group (or in Collab_Root) as node or VCopy
        NodeDataRow row = databaseMgr1.getNodeRow(prevNid, stmt);
        int prevPerspective = row.getPerspective();
        int groupPerspective = databaseMgr1.state1.getRootPerspectiveNid();
        //int authorPerspective = databaseMgr1.state1.getHomePerspectiveNid();
        int colabPerspective = databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE();
        if (prevPerspective != groupPerspective) {     //prevNid not in Group perspective
          if (prevPerspective != colabPerspective) {     //prevNid not in Colab_Root_Perspective
            rs = stmt.executeQuery("SELECT * FROM links WHERE type = " + databaseMgr1.state1.getVCOPY() +
                                            " AND perspective = " + groupPerspective + " AND fromnode = " + prevNid);
            if ( !(rs.next()) ) {      //prevNid not already vCopied in Group perspective
                //vCopy prevNid in groupPerspective
                LinkDataRow linkRow = new LinkDataRow();
                linkRow.setAuthor(databaseMgr1.state1.getCurrentAuthorNid());
                linkRow.setType(databaseMgr1.state1.getVCOPY());
                linkRow.setPerspective(groupPerspective);
                linkRow.setFrom(prevNid);
                linkRow.setTo(0);
                databaseMgr1.addLink(linkRow, stmt);
                //link prevNid below groupNid
                linkRow = new LinkDataRow();
                linkRow.setAuthor(databaseMgr1.state1.getCurrentAuthorNid());
                linkRow.setType(databaseMgr1.state1.getPARENT());
                linkRow.setPerspective(groupPerspective);
                linkRow.setFrom(prevNid);
                linkRow.setTo(groupNid);
                databaseMgr1.addLink(linkRow, stmt);
            }
          }
        }
        return addStatement(groupPerspective, kind, name, prevNid, memo, after, stmt);
    }
*/

    /**
    createWebsite() -- create row in WEBSITES table, then call createContent()
    */
    public int createWebsite(int kind, int p, String name, String description, int prevNid, String url, String bookmark, boolean after, Statement stmt) throws Exception {
        //first create row in Websites table
        int tableRow = 0;
        tableRow = databaseMgr1.updateTable("INSERT INTO Websites VALUES (0, 0, " + squote(url) + ", " + squote(bookmark) +")", "websites", stmt);
        NodeDataRow nodeRow = packARow(kind, p, name, description, 0, null);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createContent("websites", tableRow, nodeRow, prevNid, p, after, stmt);
    }

    /**
    deleteWebsite()
    Call deleteContent(), passing it the id of the row in the nodes table.
    */
    public int deleteWebsite(int oldNid, int p, Statement stmt) throws Exception {
        return deleteItem(oldNid, p, stmt);
    }

    /**
    createDecisions() -- create row in DECISIONS table, then call createContent()
    */
    public int createDecisions(int kind, int p, String name, String description, int prevNid, String decision, String rationale, boolean after, Statement stmt) throws Exception {
        //first create row in Websites table
        int tableRow = 0;
        tableRow = databaseMgr1.updateTable("INSERT INTO Decisions VALUES (0, 0, " + squote(decision) + ", " + squote(rationale) +")", "decisions", stmt);
        NodeDataRow nodeRow = packARow(kind, p, name, description, 0, null);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createContent("decisions", tableRow, nodeRow, prevNid, p, after, stmt);
    }

    /**
    deleteDecisions()
    Call deleteContent(), passing it the id of the row in the nodes table.
    */
    public int deleteDecisions(int oldNid, int p, Statement stmt) throws Exception {
        return deleteItem(oldNid, p, stmt);
    }

    /**
    createBooks() -- create row in BOOKS table, then call createContent()
    */
    public int createBooks(int kind, int p, int prevNid, String author, String year, String title, boolean after, Statement stmt) throws Exception {
        //first create row in Books table
        int tableRow = 0;
        tableRow = databaseMgr1.updateTable("INSERT INTO Books VALUES (0, 0, " + squote(author) + ", " + squote(year) + ", " + squote(title) + ")", "books", stmt);
        NodeDataRow nodeRow = packARow(kind, p, author + ", " + year, title, 0, null);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createContent("books", tableRow, nodeRow, prevNid, p, after, stmt);
    }

    /**
    deleteBooks()
    Call deleteContent(), passing it the id of the row in the nodes table.
    */
    public int deleteBooks(int oldNid, int p, Statement stmt) throws Exception {
        return deleteItem(oldNid, p, stmt);
    }

    /**
    createMultimedia() -- create row in MULTIMEDIA table, then call createContent()
    */
    public int createMultimedia(int kind, int p, String name, String description, int prevNid, String medium, Object theObject, boolean after, Statement stmt) throws Exception {
        //first create row in Multimedia table
        int tableRow = 0;
        tableRow = databaseMgr1.updateTable("INSERT INTO Multimedia VALUES (0, 0, " + squote(medium) + ", " + theObject +")", "multimedia", stmt);
        NodeDataRow nodeRow = packARow(kind, p, name, description, 0, null);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createContent("multimedia", tableRow, nodeRow, prevNid, p, after, stmt);
    }

    /**
    deleteMultimedia()
    Call deleteContent(), passing it the id of the row in the nodes table.
    */
    public int deleteMultimedia(int oldNid, int p, Statement stmt) throws Exception {
        return deleteItem(oldNid, p, stmt);
    }

    /**
    createObjects() -- create row in OBJECTS table, then call createContent()
    */
    public int createObjects(int kind, int p, String name, String description, int prevNid, String theClass, Object theObject, boolean after, Statement stmt) throws Exception {
        //first create row in Objects table
        int tableRow = 0;
        tableRow = databaseMgr1.updateTable("INSERT INTO Objects VALUES (0, 0, " + squote(theClass) + ", " + theObject +")", "objects", stmt);
        NodeDataRow nodeRow = packARow(kind, p, name, description, 0, null);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createContent("objects", tableRow, nodeRow, prevNid, p, after, stmt);
    }

    /**
    deleteObjects()
    Call deleteContent(), passing it the id of the row in the nodes table.
    */
    public int deleteObjects(int oldNid, int p, Statement stmt) throws Exception {
        return deleteItem(oldNid, p, stmt);
    }

/** ============================== methods for information =======================
    PERSON, GROUP, TABLE
    Entries in these tables need to have corresponding entries in NODES and LINKS.
    Entries in these tables should be created and deleted in the colab_root_perspective.
    Entries in these tables should be linked to their node_kind, not to other entries.
*/

    /**
    createInformation()
    First create the new row in the auxiliary table and then
    call the createInformation(), passing it the id of the new row, the table name (in lower case), an entry name and the Kind.
    */
    public int createInformation(int tableRow, String tableName, String entryName, String entryDescription, int theKind, Statement stmt) throws Exception {
        int TABLEkind = databaseMgr1.state1.getTABLE();
        int entryPerspective = databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE();
        int entryKind = 0;
        int tableId = 0;
        String tableTitle = tableName.toUpperCase() + "_TABLE";
        ResultSet rs = stmt.executeQuery("SELECT nid FROM nodes WHERE kind = " + TABLEkind + " AND name = '" + tableTitle + "'");
        if (rs.next()) {tableId = rs.getInt(1);}
        //create rows in nodes tables
        NodeDataRow newRow = new NodeDataRow();
        if (tableName == "people") {entryKind = databaseMgr1.state1.getPERSON();}       //defaults
        if (tableName == "groups") {entryKind = databaseMgr1.state1.getGROUP();}
        if (tableName == "auxtables") {entryKind = databaseMgr1.state1.getTABLE();}
        if (theKind > 0) {entryKind = theKind;}                                  //pass in the Kind
        newRow.setKind(entryKind);
        newRow.setPerspective(entryPerspective);
        newRow.setSuptable(tableId);
        newRow.setContent(tableRow);
        newRow.setName(entryName);
        newRow.setDescription(entryDescription);
        newRow.setAuthor(getAuthor());
        newRow.setEditor(getAuthor());
        //store row in nodes and links tables
        int newNid = addSubItem(entryKind, newRow, entryPerspective, stmt);
        //put pointer back to nodes row in content table
        databaseMgr1.updateTable("UPDATE " + tableName + " SET nid = " + newNid + " WHERE id = " + tableRow, tableName, stmt);
        return newNid;
    }

    /**
    deleteInformation()
    Call deleteInformation(), passing it the id of the row in the nodes table.
    Deleted information should disappear from all tables so searches will not retrieve deleted items.
    */
    public int deleteInformation(int oldNid, Statement stmt) throws Exception{
        // get name of aux table and its entry
        NodeDataRow entryRow = databaseMgr1.getNodeRow(oldNid, stmt);
        int tableNid = entryRow.getSuptable();
        int tableContent = entryRow.getContent();
        NodeDataRow tableRow = databaseMgr1.getNodeRow(tableNid, stmt);
        String tableName = tableRow.getName();
        // delete entry in aux table
        databaseMgr1.updateTable("DELETE FROM " + tableName + " WHERE id = " + tableContent, tableName, stmt);
        // delete links to and from node entry
        databaseMgr1.updateLinks("DELETE FROM links WHERE tonode = " + oldNid, stmt);
        databaseMgr1.updateLinks("DELETE FROM links WHERE fromnode = " + oldNid, stmt);
        // delete node entry
        databaseMgr1.updateNodes("DELETE FROM nodes WHERE nid = " + oldNid, stmt);
        return 0;
    }

    /**
    createPerson() -- create row in PEOPLE table, then call createInformation()
    */
    public int createPerson(String fullName, int entryKind, String userName, String password, int home_perspective_id,
                             String first, String middle, String last, String email,
                             String homepage, String sex, String color, String workplace,
                             String workTitle, String homePhone, String workPhone, Statement stmt) throws Exception {
        //first create row in people table
        int tableRow = 0;
        String theQuery = "INSERT INTO people VALUES (0, 0, " + squote(userName) + ", " + squote(password) + ", " +
                               home_perspective_id + ", " + squote(fullName) + ", " + squote(first) + ", " + squote(middle) + ", " + squote(last) + ", " +
                               squote(email) + ", " + squote(homepage) + ", " + squote(sex) + ", " + squote(color) + ", " +
                               squote(workplace) + ", " + squote(workTitle) + ", " + squote(homePhone) + ", " + squote(workPhone) + ")";
        tableRow = databaseMgr1.updateTable(theQuery, "people", stmt);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createInformation(tableRow, "people", userName, fullName, entryKind, stmt);
    }

    /**
    deletePerson()
    Call deleteInformation(), passing it the id of the row in the nodes table.
    */
    public int deletePerson(int oldNid, Statement stmt) throws Exception {
        return deleteInformation(oldNid, stmt);
    }

    /**
    createGroup() -- create row in GROUPS table, then call createInformation()
    */
    public int createGroup(String password, String name, String description, int representative, String email, int home_perspective_id, Statement stmt) throws Exception {
        //first create row in groups table
        int tableRow = 0;
        String theQuery = "INSERT INTO groups VALUES (0, 0, " + squote(password) + ", " + squote(name) + ", " +
                               representative + ", " + squote(email) + ", " + home_perspective_id + ")";
        tableRow = databaseMgr1.updateTable(theQuery, "groups", stmt);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createInformation(tableRow, "groups", name, description, 0, stmt);
    }

    /**
    deleteGroup()
    Call deleteInformation(), passing it the id of the row in the nodes table.
    */
    public int deleteGroup(int oldNid, Statement stmt) throws Exception {
        return deleteInformation(oldNid, stmt);
    }

    /**
    createTable() -- create row in TABLES table, then call createInformation()
    */
    public int createTable(String name, int field0, int field1, int field2, int field3, int field4,
                           int field5, int field6, int field7, int field8, int field9, int field10,
                           int field11, int field12, int field13, int field14, int field15, int field16,
                           int field17, int field18, int field19, Statement stmt) throws Exception {
        //first create row in TABLES table
        int tableRow = 0;
        String theQuery = "INSERT INTO auxtables VALUES (0, 0, " + field0 + ", " + field1 + ", " + field2 + ", " +
                            field3 + ", " + field4 + ", " + field5 + ", " + field6 + ", " + field7 + ", " +
                            field8 + ", " + field9 + ", " + field10 + ", " + field11 + ", " + field12 + ", " +
                            field13 + ", " + field14 + ", " + field15 + ", " + field16 + ", " + field17 + ", " +
                            field18 + ", " + field19 + ")";
        tableRow = databaseMgr1.updateTable(theQuery, "auxtables", stmt);
        //then create rows in nodes and links tables, and revise back pointer in people table
        return createInformation(tableRow, "auxtables", name, name + " auxiliary table", 0, stmt);
    }

    /**
    deleteTable()
    Call deleteInformation(), passing it the id of the row in the nodes table.
    */
    public int deleteTable(int oldNid, Statement stmt) throws Exception {
        return deleteInformation(oldNid, stmt);
    }

/** ============================== methods for categories ======================= */

    /**
    createCategory() -- given its name and the category it follows in the category hierarchy
    */
    public int createCategory(String name, int prevNid, boolean after, Statement stmt) throws Exception {
        NodeDataRow newRow = new NodeDataRow();
        newRow.setPerspective(databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE());
        newRow.setName(name);
        newRow.setDescription(name);
        newRow.setAuthor(getAuthor());
        newRow.setEditor(getAuthor());
        newRow.setKind(databaseMgr1.state1.getNODE_KIND());         //set kind to NODE_KIND
        if (after) {
            return addItem(prevNid, newRow, databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), stmt);
        }
        else {
            return addSubItem(prevNid, newRow, databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), stmt);
        }
    }

    /**
    deleteCategory() -- delete selected category.
    only if no sub-category exists, and not used as Kind or Type
    */
    public int deleteCategory(int oldNid, Statement stmt) throws Exception{
        //check for sub-category
        int PARENTtype = databaseMgr1.state1.getPARENT();
        ResultSet rs = stmt.executeQuery("SELECT lid FROM links WHERE type = " + PARENTtype + " AND tonode = " + oldNid);
        if (rs.next()) {
            return 0;
        }
        //check for category used as node kind
        rs = stmt.executeQuery("SELECT nid FROM nodes WHERE kind = " + oldNid);
        if (rs.next()) {
            return 0;
        }
        //check for category used as link type
        rs = stmt.executeQuery("SELECT lid FROM links WHERE type = " + oldNid);
        if (rs.next()) {
            return 0;
        }
        //delete links
        rs = stmt.executeQuery("SELECT lid FROM links WHERE fromnode = " + oldNid);
        while (rs.next()) {
            databaseMgr1.deleteLink(rs.getInt(1), stmt);
        }
        //delete node
        int nid = databaseMgr1.deleteNode(oldNid, stmt);
        return nid;
    }

/** ============================== methods for perspectives ======================= */

    /**
    createAPerspective() -- given its name and the perspective it follows in the perspective hierarchy
    */
    public int createPerspective(String name, int prevNid, boolean after, Statement stmt) throws Exception {
        NodeDataRow newRow = new NodeDataRow();
        newRow.setKind(databaseMgr1.state1.getPERSPECTIVE());
        newRow.setPerspective(databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE());
        newRow.setName(name);
        newRow.setDescription(name);
        newRow.setAuthor(getAuthor());
        newRow.setEditor(getAuthor());
        if (after) { return addItem(prevNid, newRow, databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), stmt); }
        else { return addSubItem(prevNid, newRow, databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), stmt); }
    }

    /**
    editAPerspective() -- edit the name/description of a perspective
    */
    public int editPerspective(String name, int oldNid, Statement stmt) throws Exception {
        NodeDataRow newRow = new NodeDataRow();
        newRow.setKind(databaseMgr1.state1.getPERSPECTIVE());
        newRow.setPerspective(databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE());
        newRow.setName(name);
        newRow.setDescription(name);
        return editItem(oldNid, newRow, databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), stmt);
    }

    /**
    inheritAPerspective() -- the given perspective will inherit content from another perspective
    */
    public int inheritPerspective(int theChild, int theParent, Statement stmt) throws Exception {
        int newLid = 0;
        LinkDataRow linkRow = new LinkDataRow();
        linkRow.setType(databaseMgr1.state1.getPARENT());         //type = "parent"
        linkRow.setPerspective(databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE());
        linkRow.setFrom(theChild);                         //from = child
        linkRow.setTo(theParent);                          //to = parent
        linkRow.setAuthor(getAuthor());
        //link newNid as first child of parentNid
        linkRow.setSequence(1);                          //to = parent
        newLid = databaseMgr1.addLink(linkRow, stmt);            //lid set to maxLid
        //for all children of parent set seq++
        String query = "UPDATE links SET seq = seq + 1 WHERE type = " + databaseMgr1.state1.getPARENT()
                         + " AND perspective = " + databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE() + " AND tonode = " + theParent;
        databaseMgr1.updateLinks(query, stmt);
        return newLid;
    }

    /**
    deletePerspective() -- delete selected perspective.
    only if no sub-perspective exists, no content in this perspective and no links in this perspective
    */
    public int deletePerspective(int oldNid, Statement stmt) throws Exception{
        //check for sub-perspective
        int PARENTtype = databaseMgr1.state1.getPARENT();
        ResultSet rs = stmt.executeQuery("SELECT lid FROM links WHERE type = " + PARENTtype + " AND tonode = " + oldNid);
        if (rs.next()) {
            return 0;
        }
        //check for content in perspective
        rs = stmt.executeQuery("SELECT nid FROM nodes WHERE perspective = " + oldNid);
        if (rs.next()) {
            return 0;
        }
        //check for links in perspective
        rs = stmt.executeQuery("SELECT lid FROM links WHERE perspective = " + oldNid);
        if (rs.next()) {
            return 0;
        }
        //delete links and change other seq numbers
        Vector links = new Vector();
        Vector seqs = new Vector();
        Vector parents = new Vector();
        rs = stmt.executeQuery("SELECT lid, seq, tonode FROM links WHERE fromnode = " + oldNid);
        while (rs.next()) {
            int theLid = rs.getInt(1);
            int theSeq = rs.getInt(2);
            int theParent = rs.getInt(3);
            links.addElement(new Integer(theLid));          //store the links in a Vector until the database is closed
            seqs.addElement(new Integer(theSeq));           //store the seqs in a Vector until the database is closed
            parents.addElement(new Integer(theParent));     //store the parents in a Vector until the database is closed
        }
        if (seqs.size() > 0) {
            for (int i = 0; i < seqs.size(); i++) {
                Integer anSeq = (Integer) seqs.elementAt(i);
                Integer aParent = (Integer) parents.elementAt(i);
                int theSeq = anSeq.intValue();
                int theParent = aParent.intValue();
                String query = "UPDATE links SET seq = seq - 1 WHERE tonode = " + theParent + " AND seq > " + theSeq;
                databaseMgr1.updateLinks(query, stmt);
            }
        }
        //delete links found above
        Integer anLid;
        if (links.size() > 0) {
            for (int i = 0; i < links.size(); i++) {
                anLid = (Integer) links.elementAt(i);
                databaseMgr1.deleteLink(anLid.intValue(), stmt);
            }
        }
        //delete node
        int nid = databaseMgr1.deleteNode(oldNid, stmt);
        return nid;
    }

/** ======================= methods for user-defined supplementary data tables =================== */

    /**
    createNewTable() -- define a new table in the database
    */
    public int createNewTable(String table, int[] dataTypes, Statement stmt) {
        int nid = 0;
        //put entry for new table in nodes table and return its nid
        //create new table with N fields, where
        //dataTypes[0] = N  and dataTypes[1] ... dataTypes[N-1] encode the fields with:
        //1=int, 2=String, 3=float, 4=memo, 5=date, 6=time, 7=boolean, ....

        //put entry in Tables table with values of dataTypes[]

        return nid;
    }

    /**
    createEntryInTable() -- define a new row in a supplementary table in the database
    */
    public int createEntryInTable(int table, int p, Statement stmt) {
        int nid = 0;
        //put entry in nodes tabel and return its nid

        //put entry in table with default values
        //(real values will be inserted with editStringInTable, etc.
        return nid;
    }

    /**
    deleteEntryInTable() -- define a new row in a supplementary table in the database
    */
    public int deleteEntryInTable(int table, int p, Statement stmt) {
        int nid = 0;

        return nid;
    }

    /**
    vcopyEntryInTable() -- define a new row in a supplementary table in the database
    */
    public int vcopyEntryInTable(int table, int p, Statement stmt) {
        int nid = 0;

        return nid;
    }

    /**
    editStringInTable() -- define a new row in a supplementary table in the database
    */
    public int editStringInTable(int table, int p, Statement stmt) {
        int nid = 0;

        return nid;
    }

    /**
    editIntInTable() -- define a new row in a supplementary table in the database
    */
    public int editIntInTable(int table, int p, Statement stmt) {
        int nid = 0;

        return nid;
    }


 
    /*  methods in DatabaseMgr:

    //need to have a tables table that has N fields to register dataTypes for each table

    public int createTable (String table, int[] dataTypes) throws Exception {
    public int addToTable (String table) throws Exception {
    public int updateTable (String table, int nid, int field, int type, String content) throws Exception {
    */

}
