/**
*  Title:           DatabaseMgr for COLAB
*  Version:         1.0                                                 
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl                                           
*  Organization:    L3D
*  Description:     DataModule to manage database retrieval
*/

package pow2000;

import java.sql.*;
import java.util.*;
import com.objectspace.jgl.OrderedSet;

public class DatabaseMgr {
    public static InterfaceState state1;
    // return items from links table
    private static final int TABLE_FIELD_COUNT = 19;
    private NodeOrderedSet peopleCache = new NodeOrderedSet();
    private NodeOrderedSet perspectiveCache = new NodeOrderedSet();
    private NodeOrderedSet kindCache = new NodeOrderedSet();
    private PositionOrderedSet positionCache = new PositionOrderedSet();
    private Vector perspectiveInheritanceCache = new Vector();
    private int perspectiveInheritanceCachePerspectiveNid = 0;

    private int DBsid = 2;
    private String DBname = InterfaceMessageDef.DBname;
    private String DBdriver = InterfaceMessageDef.DBdriver;
    private String DBurl = InterfaceMessageDef.DBurl;
    private String DBcentral = InterfaceMessageDef.DBcentral;
    private String DBusername = InterfaceMessageDef.DBusername;
    private String DBpasswd = InterfaceMessageDef.DBpasswd;
    public boolean isMySQL = true;

    //constant flags for method queryNodesOrLinks
    public final static boolean GET_ROW = true;       // return whole DataRows
    public final static boolean GET_ID = false;       // just return nid/lid
    public final static boolean GET_FIRST = true;     // just return first item found
    public final static boolean GET_LIST = false;     // return list of all items found
    public final static boolean GET_NODES = true;     // return items from nodes table
    public final static boolean GET_LINKS = false;

    public DatabaseMgr() {
        jbInit();
    }

    public DatabaseMgr(String newDBdriver, String newDBurl, String newDBusername, String newDBpasswd) {
            DBdriver = newDBdriver;
            DBurl = newDBurl;
            DBusername = newDBusername;
            DBpasswd = newDBpasswd;
            jbInit();
    }

    /**jbInit initializes the globals from the database*/
    private void jbInit() {
        try {
            state1 = new InterfaceState();
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            if (state1.getCOLAB_ROOT() < 0) {
                refreshGlobals(stmt);
                }
            refreshPeopleCache(stmt);
            refreshPerspectiveCache(stmt);
            refreshKindCache(stmt);
            refreshPositionCache(stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    checkDBname() changes parameters for DB if necessary.
    Have to openDatabase in this method and then close and open new one.
    Always use CentralDatabase as database for finding other databases.
    */
    public int checkDBname(String newDBname) {
        int returnInt = 0;
        if (newDBname.equalsIgnoreCase(DBname)) { }
        else {
            try {
                Connection CentralDatabase = openCentralDatabase();
                Statement stmt = CentralDatabase.createStatement();
                ResultSet rs = stmt.executeQuery("SELECT sid, driver, url, username, password, dbms FROM spaces WHERE name = '" + newDBname + "'");
                if (rs.next()) {
                    DBname = newDBname;
                    DBsid = rs.getInt(1);
                    DBdriver = rs.getString(2);
                    DBurl = rs.getString(3);
                    DBusername = rs.getString(4);
                    DBpasswd = rs.getString(5);
                    isMySQL = (rs.getString(6).equalsIgnoreCase("MySQL"));
                }
                CentralDatabase.close();

                //refresh global values from new database
                Connection ColabDatabase = openDatabase();
                Statement newStmt = ColabDatabase.createStatement();
                refreshGlobals(newStmt);
                state1.setCurrentSpaceNid(DBsid);
                state1.setCurrentSpaceName(DBname);
                refreshPeopleCache(newStmt);
                refreshPerspectiveCache(newStmt);
                refreshKindCache(newStmt);
                refreshPositionCache(newStmt);
                ColabDatabase.close();
                returnInt = DBsid;
            }
            catch (Exception e) {
                    e.printStackTrace();
            }
        }
        return returnInt;
    }

    /**
    openDatabase() opens the COLAB database.
    */
    public Connection openDatabase() {
        for (int i = 0 ; i < 5; i++) {       //allow 3 tries in case of intermittent "bad handshake"
            try {
                Class.forName(DBdriver);
                Connection ColabDatabase = DriverManager.getConnection(DBurl, DBusername, DBpasswd);
                return ColabDatabase;
            }
            catch (Exception e) {
                if (i == 4) {
                    e.printStackTrace();
                    db("DatabaseMgr.OpenDatabase() failed 5 times." + DBurl);
                }
                else {
                    //db("DatabaseMgr.OpenDatabase() encountered bad handshake or other Exception.");
                }
            }
        }
        return null;
    }

    /**
    openCentralDatabase() opens the central COLAB database.
    */
    public Connection openCentralDatabase() {
        for (int i = 0 ; i < 5; i++) {       //allow 3 tries in case of intermittent "bad handshake"
            try {
                Class.forName("gwe.sql.gweMysqlDriver");
                Connection ColabDatabase = DriverManager.getConnection(DBcentral, DBusername, DBpasswd);
                return ColabDatabase;
            }
            catch (Exception e) {
                if (i == 4) {
                    e.printStackTrace();
                    db("DatabaseMgr.openCentralDatabase() failed 5 times. " + DBcentral);
                }
                else {
                    //db("DatabaseMgr.OpenDatabase() encountered bad handshake or other Exception.");
                }
            }
        }
        return null;
    }

    /**
    getMaxId() retrieves the highest id from a table.
    */
    private int getMaxId(String table_name, Statement stmt) throws SQLException {
        ResultSet rs = null;
        int maxId = 0;
        if (isMySQL) {
            rs = stmt.executeQuery("SELECT LAST_INSERT_ID()");
        }
        else {
            rs = stmt.executeQuery("SELECT MAX(id) FROM " + table_name);
        }
        if (rs.next()) {
            maxId = rs.getInt(1);
        }
        return maxId;
    }

    /**
    getMaxNodeNid() retrieves the highest Nid from the nodes table.
    */
    private int getMaxNodeNid(Statement stmt) throws SQLException {
        ResultSet rs = null;
        int maxNid = 0;
        if (isMySQL) {
            rs = stmt.executeQuery("SELECT LAST_INSERT_ID()");
        }
        else {
            rs = stmt.executeQuery("SELECT MAX(nid) FROM nodes");
        }
        if (rs.next()) {
            maxNid = rs.getInt(1);
        }
        return maxNid;
    }

    /**
    getMaxLinkLid() retrieves the highest Lid from the links table.
    */
    private int getMaxLinkLid(Statement stmt) throws ClassNotFoundException, SQLException {
        ResultSet rs = null;
        int maxLid = 0;
        if (isMySQL) {
            rs = stmt.executeQuery("SELECT LAST_INSERT_ID()");
        }
        else {
            rs = stmt.executeQuery("SELECT MAX(lid) FROM links");
        }
        if (rs.next()) {
            maxLid = rs.getInt(1);
        }
        return maxLid;
    }

    /**
    updateTable() executes a synchronized update query on a table -- takes a table name and returns the new row id
    */
    public synchronized int updateTable(String query, String table, Statement stmt) throws ClassNotFoundException, SQLException {
        stmt.executeUpdate(query);
        return getMaxId(table, stmt);
        }

    /**
    updateNodes() executes a synchronized update query on the Nodes table -- takes a table name and returns the new row Nid
    */
    public synchronized int updateNodes(String query, Statement stmt) throws ClassNotFoundException, SQLException {
        stmt.executeUpdate(query);
        return getMaxNodeNid(stmt);
        }

    /**
    updateLinks() executes a synchronized update query on the Links table -- takes a table name and returns the new row Lid
    */
    public synchronized int updateLinks(String query, Statement stmt) throws ClassNotFoundException, SQLException {
        stmt.executeUpdate(query);
        return getMaxLinkLid(stmt);
        }

    /**
    addNode() adds one row to the nodes table.
    replace single quotes in name and description with "\'"
    Returns Nid of added row.

    Cannot save old Created date -- crashes with old value instead of null ???
    */
    public int addNode(NodeDataRow newRow, Statement stmt) throws ClassNotFoundException, SQLException {
        String query = "INSERT INTO nodes VALUES (" + 0 + ", " + newRow.getKind() + ", " +
                               newRow.getPerspective() + ", " + newRow.getSuptable() + ", " + newRow.getContent() + ", '" +
                               encode(newRow.getName()) + "', '" + encode(newRow.getDescription()) + "', " +
                               newRow.getAuthor() + ", " + null + ", "  + newRow.getEditor() + ", "  + null + " )";
        int nid = updateNodes(query, stmt);
        return nid;
    }

    /**
    addLink() adds one row to the links table.
    Returns Lid of added row.
    */
    public  int addLink(LinkDataRow newRow, Statement stmt) throws ClassNotFoundException, SQLException {
        String query = "INSERT INTO links VALUES (" + 0 + ", " + newRow.getType() + ", " +
                               newRow.getPerspective() + ", " + newRow.getFrom() + ", " + newRow.getTo() + ", " +
                               newRow.getSequence() + ", " + newRow.getAuthor() + ", " + null + " )";
        return updateLinks(query, stmt);
        }

    /**
    deleteNode() deletes one row from the nodes table.
    Returns Nid of deleted row.
    */
    public  int deleteNode(int oldNid, Statement stmt) throws ClassNotFoundException, SQLException {
        updateNodes("DELETE FROM nodes WHERE nid = " + oldNid, stmt);
        return oldNid;
    }

    /**
    deleteLink() deletes one row from the links table.
    Returns Lid of deleted row.
    */
    public int deleteLink(int oldLid, Statement stmt) throws ClassNotFoundException, SQLException {
        updateLinks("DELETE FROM links WHERE lid = " + oldLid, stmt);
        return oldLid;
    }

    /**
    setNewPassword() change author's password in people table
    */
    public void setNewPassword(int author, String password, Statement stmt) throws ClassNotFoundException, SQLException {
        String theQuery = "UPDATE people SET password = '" + password + "' WHERE nid = " + author;
        updateTable(theQuery, "people", stmt);
    }

    /**
    logChange()
    */
    public void logChange(int logType, int p, int fromnode, int tonode, int author, Statement stmt) throws ClassNotFoundException, SQLException {
        String theQuery = "INSERT INTO transactions VALUES (0, " + logType + ", " + p + ", " + fromnode + ", "
                            + tonode + ", " + author + ", " + null + ")";
        updateTable(theQuery, "transactions", stmt);
    }

    /**
    createTable() creates a new table in the database.
    dataTypes[11] encodes number of fields used and their types
    max number of fields = TABLE_FIELD_COUNT = 19
    */
    public int createTable (String table, int[] dataTypes, Statement stmt) throws Exception {
        //add entry in nodes table for a new table in this database with dataTypes[0] encoded fields of type dataType[i]
        int tid = 0;

        //create new table in this database with dataTypes[0] encoded fields of type dataType[i]

        //create row in tables table for this new table
        String theQuery = "INSERT INTO tables VALUES (" + tid + ", " + dataTypes[0] + ", " + dataTypes[1] + ", " + dataTypes[2] + ", " + dataTypes[3] + ", "
                                                        + dataTypes[4] + ", " + dataTypes[5] + ", " + dataTypes[6] + ", " + dataTypes[7] + ", "
                                                        + dataTypes[8] + ", " + dataTypes[9] + ", " + dataTypes[10] + ", " + dataTypes[11] + ", "
                                                        + dataTypes[12] + ", " + dataTypes[13] + ", " + dataTypes[14] + ", " + dataTypes[15] + ", "
                                                        + dataTypes[16] + ", " + dataTypes[17] + ", " + dataTypes[18] + ", "+ dataTypes[19] + " )";
        tid = updateTable(theQuery, "tables", stmt);
        return tid;
    }

    /**
    addToTable() creates a new row to a supplementary table in the database.
    fields will have default values until updateTable is called
    */
    public int addToTable (String table, Statement stmt) throws Exception {
        //look up tid for this table

        //look up field datatypes in tables table

        //insert row with default values in fields

        return 0;
    }

    /**
    updateTable() changes the content of a row in a supplementary table in the database.
    */
    public int updateTable (String table, int nid, int field, int type, String content, Statement stmt) throws Exception {
        return 0;
    }

    /**
    cacheNodes() reads in the nodes table for perspective p and its inherited ancestors.
    */
    public NodeOrderedSet cacheNodes(int p, Statement stmt) throws Exception {
        NodeNidPredicate thePredicate = new NodeNidPredicate();
        NodeOrderedSet nodeDB = new NodeOrderedSet(thePredicate, false);    //create OrderedSet ordered on nid, no dups
        int maxNid = 0;
        LinkDataRow row = new LinkDataRow();
        ResultSet rs;
        Vector v = perspectiveInheritanceCache;      //reset before cacheNodes called
        String list = "(";
        if (v.size() > 0) {
            row = (LinkDataRow) v.elementAt(0);
            list = list + "" + row.getLid() + "";
        }
        for (int i = 1; i < v.size(); i++) {
            row = (LinkDataRow) v.elementAt(i);
            list = list + ", " + row.getLid() + "";
        }
        list = list + ")";
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind > " + state1.getCONTENT() + " AND perspective IN " + list);
        NodeDataRow nodeRow;
        while (rs.next() ) {
            nodeRow = new NodeDataRow(rs);
            nodeDB.add(nodeRow);
        }
        //add nodes VCOPYed into these perspectives
        v = new Vector();
        rs = stmt.executeQuery("SELECT fromnode FROM links WHERE type = " + state1.getVCOPY() + " AND perspective IN " + list);
        while (rs.next() ) {
            int nid = rs.getInt(1);
            v.addElement(new Integer(nid));
        }
        for (int i = 0; i < v.size(); i++) {
            Integer theNid = (Integer) v.elementAt(i);
            nodeRow = getNodeRow(theNid.intValue(), stmt);
            nodeDB.add(nodeRow);
        }
        return nodeDB;
    }

    /**
    cacheLinks() reads in the links table for perspective p and its inherited ancestors.
    */
    public LinkOrderedSet cacheLinks(int p, Statement stmt) throws Exception {
        LinkLidPredicate thePredicate = new LinkLidPredicate();
        LinkOrderedSet linkDB = new LinkOrderedSet(thePredicate, false);    //create OrderedSet ordered on lid, no dups
        Vector linkVector = new Vector();         //temp Vector of LinkDataRows
        String list = "(";
        int maxLid = 0;
        LinkDataRow row = new LinkDataRow();
        ResultSet rs;
        Vector v = perspectiveInheritanceCache;
        if (v.size() > 0) {
            row = (LinkDataRow) v.elementAt(0);
            list = list + row.getLid();
        }
        for (int i = 1; i < v.size(); i++) {
            row = (LinkDataRow) v.elementAt(i);
            list = list + ", " + row.getLid();
        }
        list = list + ")";
        rs = stmt.executeQuery("SELECT * FROM links WHERE perspective IN " + list);
        LinkDataRow linkRow;
        while (rs.next() ) {
            linkRow = new LinkDataRow(rs);
            linkVector.addElement(linkRow);
        }
        //remove duplicate parent links with same fromnode and tonode in different perspectives
        for (int i = 0; i < linkVector.size() - 1; i++) {
            linkRow = (LinkDataRow) linkVector.elementAt(i);
            for (int j = i + 1; j < linkVector.size(); j++) {
                LinkDataRow linkRow2 = (LinkDataRow) linkVector.elementAt(j);
                if (linkRow.getFrom() == linkRow2.getFrom() && linkRow.getTo() == linkRow2.getTo()) {
                    linkVector.removeElementAt(j);
                }
            }
         }
        //copy Vector into LinkOrderedSet
        for (int i = 0; i < linkVector.size(); i++) {
            linkRow = (LinkDataRow) linkVector.elementAt(i);
            linkDB.add(linkRow);
        }
        return linkDB;
    }

    /**
    refreshPeopleCache() stores NodeDataRows of all people in global peopleCache
    should be refreshed when person info changes
    */
    private void refreshPeopleCache(Statement stmt) throws SQLException {
        NodeNidPredicate thePredicate = new NodeNidPredicate();
        peopleCache = new NodeOrderedSet(thePredicate, false);    //create OrderedSet ordered on nid, no dups
        ResultSet rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + state1.getPERSON());
        while (rs.next() ) {
            NodeDataRow nodeRow = new NodeDataRow(rs);
            peopleCache.add(nodeRow);
        }
    }

    /**
    refreshPerspectiveCache() stores NodeDataRows of all perspectives in global perspectiveCache
    should be refreshed when perspective info changes
    */
    private void refreshPerspectiveCache(Statement stmt) throws SQLException {
        NodeNidPredicate thePredicate = new NodeNidPredicate();
        perspectiveCache = new NodeOrderedSet(thePredicate, false);    //create OrderedSet ordered on nid, no dups
        ResultSet rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + state1.getPERSPECTIVE());
        while (rs.next() ) {
            NodeDataRow nodeRow = new NodeDataRow(rs);
            perspectiveCache.add(nodeRow);
        }
    }

    /**
    refreshKindCache() stores NodeDataRows of all node_kinds in global kindCache in alphabetical order
    should be refreshed when Node_Kind info changes
    */
    private void refreshKindCache(Statement stmt) throws SQLException {
        NodeNidPredicate thePredicate = new NodeNidPredicate();
        kindCache = new NodeOrderedSet(thePredicate, false);    //create OrderedSet ordered on nid, no dups
        ResultSet rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + state1.getNODE_KIND() + " AND nid > " + state1.getCOLAB_LAST());
        while (rs.next() ) {
            NodeDataRow nodeRow = new NodeDataRow(rs);
            kindCache.add(nodeRow);
        }
    }

    /**
    refreshPositionCache() stores PositionDataRows of positions in global positionCache
    should be refreshed when a graphical view is saved
    */
    private void refreshPositionCache(Statement stmt) throws SQLException {
        PositionNidPredicate thePredicate = new PositionNidPredicate();
        positionCache = new PositionOrderedSet(thePredicate, false);    //create OrderedSet ordered on nid, no dups
        LinkDataRow row = new LinkDataRow();
        ResultSet rs;
        Vector v = perspectiveInheritanceCache;      //reset before cacheNodes called
        String list = "(";
        if (v.size() > 0) {
            row = (LinkDataRow) v.elementAt(0);
            list = list + "" + row.getLid() + "";
        }
        for (int i = 1; i < v.size(); i++) {
            row = (LinkDataRow) v.elementAt(i);
            list = list + ", " + row.getLid() + "";
        }
        list = list + ")";
        rs = stmt.executeQuery("SELECT * FROM positions");
        //rs = stmt.executeQuery("SELECT * FROM positions WHERE perspective IN " + list);
        PositionDataRow pRow;
        while (rs.next() ) {
            pRow = new PositionDataRow(rs);
            positionCache.add(pRow);
        }
    }


    /**
    refreshGlobals() recomputes the values of the standard constants
    in case the user changed something or when initializing system
    */
    public void refreshGlobals(Statement stmt) throws SQLException {
        NodeDataRow row;
        ResultSet rs = stmt.executeQuery("SELECT * FROM nodes WHERE name = 'COLAB_ROOT' ");
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("COLAB_ROOT")) {state1.setCOLAB_ROOT(row.getNid()); }
        }
        String ROOTString = "" + state1.getCOLAB_ROOT();
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + ROOTString);
        //rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + ROOTString + " AND name IN ('COLAB_ROOT_CATEGORY', 'COLAB_ROOT_PERSPECTIVE', 'COLAB_ROOT_CONTENT', 'COLAB_COMPARISON_PERSPECTIVE')");
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("COLAB_ROOT_CATEGORY")) {
                state1.setCOLAB_ROOT_CATEGORY(row.getNid());
            }
            if (row.getName().equals("COLAB_ROOT_PERSPECTIVE")) {
                state1.setCOLAB_ROOT_PERSPECTIVE(row.getNid());
                if (row.getContent() > 0) {
                    state1.setRootPerspectiveNid(row.getContent());                  //save its nid in state
                }
            }
            if (row.getName().equals("COLAB_ROOT_CONTENT")) {
                state1.setCOLAB_ROOT_CONTENT(row.getNid());
            }
            if (row.getName().equals("COLAB_COMPARISON_PERSPECTIVE")) {
                state1.setCOLAB_COMPARISON_PERSPECTIVE(row.getNid());              //nid pointer saved in content field
                if (row.getContent() > 0) {
                    state1.setComparisonPerspectiveNid(row.getContent());                  //save its nid in state
                }
            }
        }
        state1.setRootPerspectiveName(getName(state1.getRootPerspectiveNid(), stmt));  //save its name in state too
        state1.setComparisonPerspectiveName(getName(state1.getComparisonPerspectiveNid(), stmt));  //save its name in state too

        String CATEGORYString = "" + state1.getCOLAB_ROOT_CATEGORY();
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + CATEGORYString + " AND name IN ('NODE_KIND', 'LINK_TYPE')");
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("NODE_KIND")) {state1.setNODE_KIND(row.getNid()); }
            if (row.getName().equals("LINK_TYPE")) {state1.setLINK_TYPE(row.getNid()); }
        }
        String KINDString = "" + state1.getNODE_KIND();
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + KINDString + " AND name IN ('PERSPECTIVE', 'CONTENT', " +
                               "'INFORMATION', 'DISCUSSION', 'STATEMENT', 'DECISION', 'PROPOSAL', 'PRIVATE', 'BOOK', 'MULTIMEDIA', " +
                               "'OBJECT', 'EMAIL', 'PERSON', 'GROUP', 'TABLE', 'COLAB_LAST')" );
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("PERSPECTIVE")) {state1.setPERSPECTIVE(row.getNid()); }
            if (row.getName().equals("INFORMATION")) {state1.setINFORMATION(row.getNid()); }
            if (row.getName().equals("PERSON")) {state1.setPERSON(row.getNid()); }
            if (row.getName().equals("GROUP")) {state1.setGROUP(row.getNid()); }
            if (row.getName().equals("TABLE")) {state1.setTABLE(row.getNid()); }
            if (row.getName().equals("CONTENT")) {state1.setCONTENT(row.getNid()); }
            if (row.getName().equals("DISCUSSION")) {state1.setDISCUSSION(row.getNid()); }
            if (row.getName().equals("PROPOSAL")) {state1.setPROPOSAL(row.getNid()); }
            if (row.getName().equals("PRIVATE")) {state1.setPRIVATE(row.getNid()); }
            if (row.getName().equals("STATEMENT")) {state1.setSTATEMENT(row.getNid()); }
            if (row.getName().equals("DECISION")) {state1.setDECISION(row.getNid()); }
            if (row.getName().equals("BOOK")) {state1.setBOOK(row.getNid()); }
            if (row.getName().equals("MULTIMEDIA")) {state1.setMULTIMEDIA(row.getNid()); }
            if (row.getName().equals("OBJECT")) {state1.setOBJECT(row.getNid()); }
            if (row.getName().equals("EMAIL")) {state1.setEMAIL(row.getNid()); }
            if (row.getName().equals("COLAB_LAST")) {state1.setCOLAB_LAST(row.getNid()); }
        }
        String TYPEString = "" + state1.getLINK_TYPE();
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + TYPEString + " AND name IN ('EDIT', 'DELETE', 'DELETELINK', 'SYNONYM', 'PARENT', 'ISA', 'VCOPY', 'VCHILDREN')" );
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("EDIT")) {state1.setEDIT(row.getNid()); }
            if (row.getName().equals("DELETE")) {state1.setDELETE(row.getNid()); }
            if (row.getName().equals("DELETELINK")) {state1.setDELETELINK(row.getNid()); }
            if (row.getName().equals("SYNONYM")) {state1.setSYNONYM(row.getNid()); }
            if (row.getName().equals("PARENT")) {state1.setPARENT(row.getNid()); }
            if (row.getName().equals("ISA")) {state1.setISA(row.getNid()); }
            if (row.getName().equals("VCOPY")) {state1.setVCOPY(row.getNid()); }
            if (row.getName().equals("VCHILDREN")) {state1.setVCHILDREN(row.getNid()); }
        }
        String PERSONString = "" + state1.getPERSON();
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + PERSONString + " AND name IN ('COLAB_ROOT_AUTHOR')" );
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("COLAB_ROOT_AUTHOR")) {state1.setCOLAB_ROOT_AUTHOR(row.getNid()); }
        }
        String TABLEString = "" + state1.getTABLE();
        rs = stmt.executeQuery("SELECT * FROM nodes WHERE kind = " + TABLEString + " AND name IN ('PEOPLE_TABLE', 'STATEMENTS_TABLE', " +
        "'DECISIONS_TABLE', 'GROUPS_TABLE', 'WEBSITES_TABLE', 'BOOKS_TABLE', 'MULTIMEDIA_TABLE', 'OBJECTS_TABLE', " +
        "'EMAIL_TABLE', 'TRANSACT_TABLE', 'AUXTABLES_TABLE')" );
        while (rs.next()) {
            row = new NodeDataRow(rs);
            if (row.getName().equals("PEOPLE_TABLE")) {state1.setPEOPLE_TABLE(row.getNid()); }
            if (row.getName().equals("STATEMENTS_TABLE")) {state1.setSTATEMENTS_TABLE(row.getNid()); }
            if (row.getName().equals("DECISIONS_TABLE")) {state1.setDECISIONS_TABLE(row.getNid()); }
            if (row.getName().equals("GROUPS_TABLE")) {state1.setGROUPS_TABLE(row.getNid()); }
            if (row.getName().equals("BOOKS_TABLE")) {state1.setBOOKS_TABLE(row.getNid()); }
            if (row.getName().equals("MULTIMEDIA_TABLE")) {state1.setMULTIMEDIA_TABLE(row.getNid()); }
            if (row.getName().equals("OBJECTS_TABLE")) {state1.setOBJECTS_TABLE(row.getNid()); }
            if (row.getName().equals("EMAIL_TABLE")) {state1.setEMAIL_TABLE(row.getNid()); }
            if (row.getName().equals("TRANSACT_TABLE")) {state1.setTRANSACT_TABLE(row.getNid()); }
            if (row.getName().equals("AUXTABLES_TABLE")) {state1.setAUXTABLES_TABLE(row.getNid()); }
        }
    }

    /**
    getKind() returns nid of a Node_Kind.
    */
    public int getKind(String theKind, Statement stmt) throws SQLException {
        ResultSet rs = stmt.executeQuery("SELECT nid FROM nodes WHERE name = '" + theKind +
                                         "' AND perspective = " + state1.getCOLAB_ROOT_PERSPECTIVE());
        if (rs.next()) {return rs.getInt(1);}
        else {return 0;}
    }

    /**
    getNodesOfKind() returns a Vector of all nodes of a given kind
    */
    public Vector getNodesOfKind(int theKind, Statement stmt) throws SQLException {
        String KINDString = "" + theKind;
        Vector v = queryNodesOrLinks(GET_ID, GET_LIST, GET_NODES, "kind", KINDString, "", "", stmt);
        return v;
    }

    /**
    getLinksOfType() returns a Vector of all links of a given type
    */
    public Vector getLinksOfType(int theType, Statement stmt) throws SQLException {
      String TYPEString = "" + theType;
      return queryNodesOrLinks(GET_ID, GET_LIST, GET_LINKS, "kind", TYPEString, "", "", stmt);
    }

    /**
    getOrderedSetOfNodes returns an OrderedSet containing the nodes whose nid is in theList
    */
    public OrderedSet getOrderedSetOfNodes(Vector theList, Statement stmt) throws Exception {
        //NodeSeqPredicate thePredicate = new NodeSeqPredicate();
        //OrderedSet theNodes = new OrderedSet(thePredicate, true);
        OrderedSet theNodes = new OrderedSet(true);
        NodeDataRow theRow = new NodeDataRow();
        Integer theInteger;
        for (int i = 0; i < theList.size(); i++) {
            theInteger = (Integer) theList.elementAt(i);
            theRow = getNodeRow(theInteger.intValue(), stmt);
            theNodes.add(theRow);
        }
        return theNodes;
    }

    /**
    getName() returns name for a given nid.
    */
    public String getName(int theNid, Statement stmt) throws SQLException {
        String name = "";
        if (theNid > 0) {
            ResultSet rs = stmt.executeQuery("SELECT name FROM nodes WHERE nid = '" + theNid + "'");
            if (rs.next()) {name = rs.getString(1);}
        }
        return name;
    }

    /**
    getNodeRow() returns a row of nodeDataSet, given its nid
    */
    public NodeDataRow getNodeRow(int theNid, Statement stmt) throws Exception {
        String theNidString = "" + theNid;
        Vector v = queryNodesOrLinks(GET_ROW, GET_FIRST, GET_NODES, "nid", theNidString, "", "", stmt);
        return (NodeDataRow) v.elementAt(0);
    }

    /**
    getLinkRow returns a row of linkDataSet, given its lid
    */
    public LinkDataRow getLinkRow(int theLid, Statement stmt) throws Exception {
        String theLidString = "" + theLid;
        Vector v = queryNodesOrLinks(GET_ROW, GET_FIRST, GET_LINKS, "lid", theLidString, "", "", stmt);
        return (LinkDataRow) v.elementAt(0);
    }

    /**
    queryNodesOrLinks() queries the nodes or links table.
    It returns NodeDataRows/LinkDataRows or nid/lid(s) of first node/link or list of nodes/links
    matching one or two column values;
    use constants GET_ROW or GET_ID for isRow
    use constants GET_FIRST or GET_LIST for isFirst
    use constants GET_NODES or GET_LINKS for isNodes
    use "" for columnName2 and columnValue2 to specify just one value
    */
    public Vector queryNodesOrLinks(boolean isRow, boolean isFirst, boolean isNodes, String columnName1,
                                  String columnValue1, String columnName2, String columnValue2, Statement stmt) throws SQLException {
        Vector theList = new Vector();
        String theField = "links";
        String theQuery = "";
        if (columnName1 == "name" | columnName1 == "description") {columnValue1 = "'" + columnValue1 + "'";}
        if (columnName2 == "name" | columnName2 == "description") {columnValue2 = "'" + columnValue2 + "'";}
        if (isNodes) {theField = "nodes";}
        if (columnName2 == "") {theQuery = "SELECT * FROM " + theField + " WHERE " + columnName1 + " = " + columnValue1; }
        else {theQuery = "SELECT * FROM " + theField + " WHERE " + columnName1 + " = " + columnValue1 + " AND " + columnName2 + " = " + columnValue2; }
        ResultSet rs = stmt.executeQuery(theQuery);
        while (rs.next()) {
            if (isRow) {
                if (isNodes) {theList.addElement(new NodeDataRow(rs));}
                else {theList.addElement(new LinkDataRow(rs));}
            }
            else {theList.addElement(new Integer(rs.getInt(1)));}
            if (isFirst) {break;}
        }
        return theList;
    }

    /**
    perspectiveHierarchy returns a Vector of LinkDataRows of descendants of perspective p
    in sequence order with level (root level = 0)
    retrieve the parent nid with: myRow.getInt("to")
    */
    public Vector perspectiveHierarchy (int p, Statement stmt) throws Exception {
        LinkDataRow addRow = new LinkDataRow();
        addRow.setLid(p);
        addRow.setLevel(0);
        Vector outList = new Vector();
        outList.addElement(addRow);                             //add row for lid = p at level = 0
        LinkLidPredicate thePredicate = new LinkLidPredicate();       //maintained in order by sequence
        OrderedSet inList = new OrderedSet(thePredicate, true);
        inList = perspectiveChildren(p, stmt);
        Enumeration iterator = inList.elements();
        while (iterator.hasMoreElements()) {
            addRow = (LinkDataRow) iterator.nextElement();
            Vector v = perspectiveHierarchy(addRow.getFrom(), stmt);
            for (int i = 0; i < v.size(); i++) {
                addRow = (LinkDataRow) v.elementAt(i);
                addRow.incrLevel();
                outList.addElement(addRow);
            }
        }
        return outList;
    }

    /**
    perspectiveTree returns a Vector of TreeDataRows of descendants of perspective p
    in sequence order
    */
    public Vector perspectiveTree (int p, Statement stmt) throws Exception {
        Vector outList = new Vector();
        Vector tempList = new Vector();     //temporary Vector of child nids
        //insert a row for p -- leave parent to be filled in by caller
        ResultSet rs2 = stmt.executeQuery("SELECT description FROM nodes WHERE nid = " + p);
        rs2.next();
        String theDescription = rs2.getString(1).replace('\\', '\'');       //description
        TreeDataRow treeRow = new TreeDataRow();
        treeRow.setNid(p);
        treeRow.setDescription(theDescription);
        outList.addElement(treeRow);                     //add row for this p
        //call recursively for each child of this p
        ResultSet rs = stmt.executeQuery("SELECT fromnode, seq FROM links WHERE tonode = " + p + " ORDER BY seq");
        while (rs.next()) {
            int theNid = rs.getInt(1);                    //fromnode
            tempList.addElement(new Integer(theNid));     //store list of nids
        }
        if (tempList.size() > 0) {
            for (int j = 0; j < tempList.size(); j++) {    //for each child nid
                Integer childInteger = (Integer) tempList.elementAt(j);
                int childNid = childInteger.intValue();
                Vector v = perspectiveTree(childNid, stmt);
                if (v.size() > 0) {
                    treeRow = (TreeDataRow) v.elementAt(0);
                    treeRow.setParent(p);                //fill in parent
                    outList.addElement(treeRow);         //add row to output list
                    for (int i = 1; i < v.size(); i++) {
                        treeRow = (TreeDataRow) v.elementAt(i);
                        outList.addElement(treeRow);         //add row to output list
                    }
                }
            }
        }
        return outList;
    }

    /**
    perspectiveInheritance returns a Vector of LinkDataRows of ancestors of perspective p
    in sequence order with level (root level = 0)
    retrieve the parent nid with: myRow.getInt("to")
    */
    public Vector perspectiveInheritance (int p, Statement stmt) throws Exception {
        LinkDataRow addRow = new LinkDataRow();
        addRow.setLid(p);
        addRow.setLevel(0);
        Vector outList = new Vector();
        outList.addElement(addRow);                                   //add row for p at level = 0
        LinkSeqPredicate thePredicate = new LinkSeqPredicate();       //maintained in order by sequence
        OrderedSet inList = new OrderedSet(thePredicate, true);
        inList = perspectiveParents(p, stmt);
        Enumeration iterator = inList.elements();
        while (iterator.hasMoreElements()) {
            addRow = (LinkDataRow) iterator.nextElement();
            Vector v = perspectiveInheritance(addRow.getTo(), stmt);
            for (int i = 0; i < v.size(); i++) {
                addRow = (LinkDataRow) v.elementAt(i);
                addRow.incrLevel();
                outList.addElement(addRow);
            }
        }
        return outList;
    }

    /**
    perspectiveParent returns a Vector of nids of parents of perspective p sorted on seq
   */
    public Vector perspectiveParent (int p, Statement stmt) throws Exception {
        Vector v = new Vector();
        ResultSet rs = stmt.executeQuery("SELECT tonode FROM links WHERE type = " + state1.getPARENT() + " AND fromnode = " + p + " ORDER BY seq");
        while (rs.next()) {
            v.addElement(new Integer(rs.getInt(1)));
        }
        return v;
    }

    /**
    perspectiveParents returns an OrderedSet of LinkDataRows of parents of perspective p
    sorted on sequence
    retrieve the parent nid with: myRow.getInt("to")
   */
    public OrderedSet perspectiveParents (int p, Statement stmt) throws SQLException {
        String PARENTString = "" + state1.getPARENT();
        String thePString = "" + p;
        Vector v = queryNodesOrLinks(GET_ROW, GET_LIST, GET_LINKS, "type", PARENTString, "fromnode", thePString, stmt);
        LinkSeqPredicate thePredicate = new LinkSeqPredicate();       //maintain in order by sequence  -- should use "order by seq"
        OrderedSet theList = new OrderedSet(thePredicate, true);
        for (int i = 0; i < v.size(); i++) {
            LinkDataRow addRow = (LinkDataRow) v.elementAt(i);
            theList.add(addRow);
        }
        return theList;
    }

    /**
    perspectiveChildren returns an OrderedSet of LinkDataRows of children of perspective p
    */
    public OrderedSet perspectiveChildren (int p, Statement stmt) throws SQLException {
        String PARENTString = "" + state1.getPARENT();
        String thePString = "" + p;
        Vector v = queryNodesOrLinks(GET_ROW, GET_LIST, GET_LINKS, "type", PARENTString, "tonode", thePString, stmt);
        LinkSeqPredicate thePredicate = new LinkSeqPredicate();       //maintain in order by sequence  -- should use "order by seq"
        OrderedSet theList = new OrderedSet(thePredicate, true);
        for (int i = 0; i < v.size(); i++) {
            LinkDataRow addRow = (LinkDataRow) v.elementAt(i);
            theList.add(addRow);
        }
        return theList;
    }

    //for a given nid in perspective p, return its multiple parents and its sequence among each parent's children.
    public Vector getParentNodes(int nid, int p, Statement stmt) throws Exception {
        LinkDataRow linkRow;
        Vector parents = new Vector();
        if (getPerspectiveInheritanceCachePerspectiveNid() != p) {    //make sure cache is for p
            setPerspectiveInheritanceCache(perspectiveInheritance(p, stmt));
        }
        Vector v = getPerspectiveInheritanceCache();
        String list = "(";
        if (v.size() > 0) {
            LinkDataRow row = (LinkDataRow) v.elementAt(0);
            list = list + "" + row.getLid() + "";
        }
        for (int i = 1; i < v.size(); i++) {
            LinkDataRow row = (LinkDataRow) v.elementAt(i);
            list = list + ", " + row.getLid() + "";
        }
        list = list + ")";
        ResultSet rs = stmt.executeQuery("SELECT * FROM links WHERE type = " + state1.getPARENT() + " AND fromnode = " +
                                          nid + " AND perspective IN " + list);
        v = new Vector();
        while (rs.next()) {
            linkRow = new LinkDataRow(rs);
            v.addElement(linkRow);
        }
        for (int i = 0; i < v.size(); i++) {
            linkRow = (LinkDataRow) v.elementAt(i);

            int persp = linkRow.getPerspective();
            int par = linkRow.getTo();
            int seq = linkRow.getSequence();
            if (linkNotDeleted(nid, persp, par, list, p, stmt)) {
                ParentSeqRow psRow = new ParentSeqRow(par, seq);
                parents.addElement(psRow);
            }
        }
        return parents;
    }

    //return false if parent link from fromnode to tonode in perspective p is a deletedLink
    public boolean linkNotDeleted(int fromnode, int tonode, int linkPersp, int p, Statement stmt) throws Exception {
        if (getPerspectiveInheritanceCachePerspectiveNid() != p) {    //make sure cache is for p
            setPerspectiveInheritanceCache(perspectiveInheritance(p, stmt));
        }
        Vector v = getPerspectiveInheritanceCache();
        String list = "(";
        if (v.size() > 0) {
            LinkDataRow row = (LinkDataRow) v.elementAt(0);
            list = list + "" + row.getLid() + "";
        }
        for (int i = 1; i < v.size(); i++) {
            LinkDataRow row = (LinkDataRow) v.elementAt(i);
            list = list + ", " + row.getLid() + "";
        }
        list = list + ")";
        return linkNotDeleted(fromnode, linkPersp, tonode, list, p, stmt);
    }

    /**
    check for linkNotDeleted()
    */
    public boolean linkNotDeleted(int nid, int linkPersp, int parentNid, String list, int p, Statement stmt) throws Exception {
        ResultSet rs = stmt.executeQuery("SELECT perspective FROM links WHERE type = " + state1.getDELETELINK() + " AND fromnode = " +
                                          nid + " AND tonode = " + parentNid + " AND perspective IN " + list);
        int deletePersp = 0;
        while (rs.next()) {
            deletePersp = rs.getInt(1);
            if (linkPersp == deletePersp) {return false;}  //if deleted in linkPersp, then deleted
            if (p == deletePersp) {return false;}          //if deleted in current p, then deleted
        }
        //if any path from linkPersp to p does not go thru a deletePersp, then link not deleted
        if (deletePersp > 0) {   //the link from nid to parent was deleted in some deletePersp
            return undeletedPath(nid, linkPersp, parentNid, p, stmt);
        }
        return true;
    }

    /**
    check for undeletedPath()
    */
    public boolean undeletedPath(int nid, int linkPersp, int parentNid, int p, Statement stmt) throws Exception {
        OrderedSet parents = perspectiveParents(p, stmt);
        Enumeration iterator = parents.elements();
        while (iterator.hasMoreElements()) {
            LinkDataRow pRow = (LinkDataRow) iterator.nextElement();
            if (linkPersp == pRow.getTo()) {return true; }     //got to end of an undeleted path
            else {
                ResultSet rs = stmt.executeQuery("SELECT lid FROM links WHERE type = " + state1.getDELETELINK() + " AND fromnode = " +
                                          nid + " AND tonode = " + parentNid + " AND perspective = " + pRow.getTo());
                if (rs.next()) {
                    return false;  //this path is deleted
                }
                else {
                    return undeletedPath(nid, linkPersp, parentNid, pRow.getTo(), stmt);
                }
            }
        }
        return false;
    }

    /**
    *  validateLogin() -- if valid login then return nid and set new values in state1
    */
    public int validateLogin(String username, String password, String space, Statement stmt) throws Exception {
        int nid = 0;
        int home = 0;
        String name = "";
        String query = "SELECT nid, perspective, fullname FROM people WHERE LOWER(TRIM(username)) = '" + username.trim().toLowerCase()
                        + "' AND LOWER(TRIM(password)) = '" + password.trim().toLowerCase() + "'";
        ResultSet rs = stmt.executeQuery(query);
        if (rs.next()) {
            nid = rs.getInt(1);
            home = rs.getInt(2);
            name = rs.getString(3);
        }
        String pName = getNodeRow(home, stmt).getName();
        state1.setCurrentAuthorNid(nid);
        state1.setCurrentAuthorName(name);
        state1.setCurrentPerspectiveNid(home);
        state1.setCurrentPerspectiveName(pName);
        state1.setHomePerspectiveNid(home);
        state1.setHomePerspectiveName(pName);
        state1.setCurrentSpaceName(space);
        return nid;
    }

    /**
    getAuthorName()
    */
    public Vector getAuthorName(int theNid, Statement stmt) throws Exception {
        int home = 0;
        String name = "";
        String query = "SELECT perspective, fullname FROM people WHERE nid = " + theNid;
        ResultSet rs = stmt.executeQuery(query);
        if (rs.next()) {
            home = rs.getInt(1);
            name = rs.getString(2);
        }
        Vector userVector = new Vector();
        userVector.addElement(new Integer(home));
        userVector.addElement(name);
        return userVector;
    }

    /**
    getCurrentPerspectiveOwner()
    */
    public int getCurrentPerspectiveOwner(int thePerspective, Statement stmt) throws Exception {
        int nid = 0;
        String query = "SELECT nid FROM people WHERE perspective = " + thePerspective;
        ResultSet rs = stmt.executeQuery(query);
        if (rs.next()) {
            nid = rs.getInt(1);
        }
        return nid;
    }

    /**
    getDescription() returns a Vector with the nid's kind, name, description, author
    */
    public Vector getDescription(int theNid, Statement stmt) throws Exception {
        int theKind = 0;
        String nameText = "";
        String descText = "";
        int theAuthor = 0;
        String query = "SELECT kind, name, description, author FROM nodes WHERE nid = " + theNid;
        ResultSet rs = stmt.executeQuery(query);
        if (rs.next()) {
            theKind = rs.getInt(1);
            nameText = rs.getString(2);
            descText = rs.getString(3);
            theAuthor = rs.getInt(4);    //original author
        }
        Vector descVector = new Vector();
        descVector.addElement(new Integer(theKind));
        descVector.addElement(nameText);
        descVector.addElement(descText);
        descVector.addElement(new Integer(theAuthor));
        return descVector;
    }

    /**
    getMemo() retrieves memoText from statements table for theNid.
    */
    public String getMemo(int theNid, Statement stmt) throws Exception {
        String memoText = "";
        String query = "SELECT statement FROM statements WHERE nid = " + theNid;
        ResultSet rs = stmt.executeQuery(query);
        if (rs.next()) {memoText = rs.getString(1);}
        return memoText;
    }

     /**
    encode(msg) encodes a ' as \' for SQL.
    */
    public String encode(String msg) {
        String str = msg;
        int index = str.indexOf("'", 0);
        while (index > -1) {
            str = str.substring(0, index) + "\\" + str.substring(index);
            index = str.indexOf("'", index + 2);
        }
        return str;
    }
    
    /**
    decode(msg) decodes a \' as ' from SQL.
    found in NodeDataRow.java
    */

    /**
    db(msg) is a debugging function. It prints msg to the System output display.
    */
    public void db(String msg) {
        System.out.println("***   " + msg);
        }

    public void db(int msg) {
        System.out.println("******   " + msg);
        }

    public void db(long msg) {
        System.out.println("*********   " + msg);
        }

    public void setDBdriver(String newDBdriver) {
        DBdriver = newDBdriver;
    }

    public String getDBdriver() {
        return DBdriver;
    }

    public void setDBurl(String newDBurl) {
        DBurl = newDBurl;
    }

    public String getDBurl() {
        return DBurl;
    }

    public void setDBusername(String newDBusername) {
        DBusername = newDBusername;
    }

    public String getDBusername() {
        return DBusername;
    }

    public void setDBpasswd(String newDBpasswd) {
        DBpasswd = newDBpasswd;
    }

    public String getDBpasswd() {
        return DBpasswd;
    }

    public NodeOrderedSet getPeopleCache() {
        return peopleCache;
    }

    public void setPeopleCache(NodeOrderedSet newCache) {
        peopleCache = newCache;
    }

    public NodeOrderedSet getPerspectiveCache() {
        return perspectiveCache;
    }

    public void setPerspectiveCache(NodeOrderedSet newCache) {
        perspectiveCache = newCache;
    }

    public NodeOrderedSet getKindCache() {
        return kindCache;
    }

    public void setKindCache(NodeOrderedSet newCache) {
        kindCache = newCache;
    }

    public PositionOrderedSet getPositionCache() {
        return positionCache;
    }

    public void setPositionCache(PositionOrderedSet newCache) {
        positionCache = newCache;
    }

    public Vector getPerspectiveInheritanceCache() {
        return perspectiveInheritanceCache;
    }

    public void setPerspectiveInheritanceCache(Vector newCache) {
        perspectiveInheritanceCache = newCache;
    }

    public int getPerspectiveInheritanceCachePerspectiveNid() {
        return perspectiveInheritanceCachePerspectiveNid;
    }

    public void setPerspectiveInheritanceCachePerspectiveNid( int newNid) {
        perspectiveInheritanceCachePerspectiveNid = newNid;
    }

}




