/**
*  Title:           InterfaceMgrClient for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     Methods for use in interface to COLAB server
*/

package pow2000;

import java.util.*;
import java.sql.*;

/**
InterfaceMgr() is a class with methods for manipulating data stored in a relational database
that preserves perspectives.
*/
public class InterfaceMgrClient {
    private InterfaceNetClient netClient1;
    private InterfaceDataSerialized dataSerial1;
    private InterfaceMessageDef msg;
    private InterfaceState state1;

    /**
    InterfaceMgr() constucts an InterfaceMgr object.
    */
    public InterfaceMgrClient(String host, int port, String db, InterfaceState theState) {
        try {
          state1 = theState;
          netClient1 = new InterfaceNetClient(host, port);
          dataSerial1 = new InterfaceDataSerialized();
          dataSerial1.setStatus(state1.getCurrentAuthorNid());     //store authorNid in returnStatus
//System.out.println("create InterfaceMgrClient and set status to: " + state1.getCurrentAuthorNid());

          dataSerial1.clearObjects();
          msg = new InterfaceMessageDef();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    initializeMethod() start methods below with clearObjects() and setStatus()
    */
    private void initializeMethod() {
        dataSerial1.setStatus(state1.getCurrentAuthorNid());     //store authorNid in returnStatus
//System.out.println("set status to: " + state1.getCurrentAuthorNid());
        dataSerial1.clearObjects();
    }

/**
    ============================== methods =======================
*/

    /**
    getListOfSpaces() get Vector with alpha list of defined information spaces and put it in state1.
    */
    public void getListOfSpaces() {
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.GETSPACESLIST);
          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        Vector oldVector = dataSerial1.getVector();
        Vector newVector = (Vector) oldVector.clone();
        state1.setListOfSpaces(newVector);
    }

    /**
    validateLogin() validates username and password in the database -- returns 0 if login fails.
    */
    public int validateLogin(String username, String password, String newDBname) {
        int returnInt = 0;
        try {
            initializeMethod();
            //dataSerial1.clearObjects();
            dataSerial1.setCommand(msg.VALIDATELOGIN);
            dataSerial1.setObject(username);
            dataSerial1.setObject(password);
            dataSerial1.setObject(newDBname);
                                                  //set new Database during login processing
                                                  //do checkDBName and logChange on Server side
            netClient1.write(dataSerial1);
            dataSerial1 = netClient1.read();
            if (dataSerial1.getStatus() > 0) {
                Object returnObject = dataSerial1.getObject();
                InterfaceState returnState = (InterfaceState) returnObject;
                state1.copy(returnState);         //change contents of InterfaceState here!
                //state1 = (InterfaceState) dataSerial1.getObject();;         //change contents of InterfaceState here!
                returnInt = state1.getCurrentAuthorNid();
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    getTreeOfNodes() get Vector of TreeDataNodes.
    */
    public void getTreeOfNodes(int p, int startNid, int mode) {
        Vector getVector;
        try {
            initializeMethod();
            //dataSerial1.clearObjects();
            dataSerial1.setCommand(msg.GETNODESFORDISPLAY);
            dataSerial1.setObject((String) new Integer(p).toString());
            dataSerial1.setObject((String) new Integer(startNid).toString());
            dataSerial1.setObject((String) new Integer(mode).toString());
            netClient1.write(dataSerial1);
            dataSerial1 = netClient1.read();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        getVector = dataSerial1.getVector();
        Vector returnVector = (Vector) getVector.clone();
        state1.setTreeOfNodes(returnVector);
        state1.setCurrentPerspectiveNid(p);
    }

    /**
    setNewPassword() change author's password in people table
    */
    public void setNewPassword(int author, String password) {
        int result;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.SETNEWPASSWORD);
          dataSerial1.setObject((String) new Integer(author).toString());
          dataSerial1.setObject(password);

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          result = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    logChange()
    */
    public void logChange(int logType, int p, int from_node, int to_node) {
        int result;
        try {
          initializeMethod();
          dataSerial1.setCommand(msg.LOGCHANGE);
          dataSerial1.setObject((String) new Integer(logType).toString());
          dataSerial1.setObject((String) new Integer(p).toString());
          dataSerial1.setObject((String) new Integer(from_node).toString());
          dataSerial1.setObject((String) new Integer(to_node).toString());
          netClient1.write(dataSerial1);                                    //do not wait to return
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    getStatement() retrieve memo text for a statement.
    */
    public String getStatement(int nid) {
        String memoText = "";
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.GETSTATEMENT);
          dataSerial1.setObject((String) new Integer(nid).toString());
          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          memoText = (String) dataSerial1.firstElement();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return memoText;
    }

    /**
    getDescription() returns a Vector with the nid's kind, name, description, author, and memo string
    Note that the memo is added from a related table in InterfaceMgrServer
    */
    public Vector getDescription(int nid) {
        Vector descVector;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.GETDESCRIPTION);
          dataSerial1.setObject((String) new Integer(nid).toString());
          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        descVector = dataSerial1.getVector();
        Vector returnVector = (Vector) descVector.clone();
        return returnVector;
    }

/**
    ============================== methods for changeMgr =======================
*/

     /**
    addItem()
    */
    public int addItem(int previousNid, NodeDataRow newRow, int p, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.ADDITEM);
          dataSerial1.setObject((String) new Integer(previousNid).toString());
          dataSerial1.setObject((Object) newRow);
          dataSerial1.setObject((String) new Integer(p).toString());
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    moveItem() move a node after/below a second node
    */
    public int moveItem(int theNid, int theOldParent, int thePrev, int thePrevParent, int oldP, int newP, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.MOVEITEM);
          dataSerial1.setObject((String) new Integer(theNid).toString());
          dataSerial1.setObject((String) new Integer(theOldParent).toString());
          dataSerial1.setObject((String) new Integer(thePrev).toString());
          dataSerial1.setObject((String) new Integer(thePrevParent).toString());
          dataSerial1.setObject((String) new Integer(oldP).toString());
          dataSerial1.setObject((String) new Integer(newP).toString());
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    copyItem() -- create a new node copy after/below a second node
    */
    public int copyStatement(int theNid, int thePrev, int oldP, int newP, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.COPYITEM);
          dataSerial1.setObject((String) new Integer(theNid).toString());
          dataSerial1.setObject((String) new Integer(thePrev).toString());
          dataSerial1.setObject((String) new Integer(oldP).toString());
          dataSerial1.setObject((String) new Integer(newP).toString());
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    linkItem() -- link a child node in p1 to a parent in p2
    create a VCOPY of child in p2 if p1 != p2
    */
    public int linkItem(int childNid, int parentNid, int p1, int p2, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.LINKITEM);
          dataSerial1.setObject((String) new Integer(childNid).toString());
          dataSerial1.setObject((String) new Integer(parentNid).toString());
          dataSerial1.setObject((String) new Integer(p1).toString());
          dataSerial1.setObject((String) new Integer(p2).toString());
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    addStatement()
    */
    public int addStatement(int p, int kind, String name, int prevNid, String memo, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.ADDSTATEMENT);
          dataSerial1.setObject((String) new Integer(p).toString());
          dataSerial1.setObject((String) new Integer(kind).toString());
          dataSerial1.setObject(name);
          dataSerial1.setObject((String) new Integer(prevNid).toString());
          dataSerial1.setObject(memo);
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    editStatement()
    */
    public int editStatement(int kind, int p, String name, String memo, int oldNid) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.EDITSTATEMENT);
          dataSerial1.setObject((String) new Integer(kind).toString());
          dataSerial1.setObject((String) new Integer(p).toString());
          dataSerial1.setObject(name);
          dataSerial1.setObject(memo);
          dataSerial1.setObject((String) new Integer(oldNid).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    deleteStatement()
    */
    public int deleteStatement(int oldNid, int p) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.DELETESTATEMENT);
          dataSerial1.setObject((String) new Integer(oldNid).toString());
          dataSerial1.setObject((String) new Integer(p).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    addDiscussion()
    */
    public int addDiscussion(int p, String name, int prevNid, int groupNid, String memo, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.ADDDISCUSSION);
          dataSerial1.setObject((String) new Integer(p).toString());
          dataSerial1.setObject(name);
          dataSerial1.setObject((String) new Integer(prevNid).toString());
          dataSerial1.setObject((String) new Integer(groupNid).toString());
          dataSerial1.setObject(memo);
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createPerspective()
    */
    public int createPerspective(String name, int prevNid, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.CREATEPERSPECTIVE);
          dataSerial1.setObject(name);
          dataSerial1.setObject((String) new Integer(prevNid).toString());
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    editPerspective()
    */
    public int editPerspective(String name, int oldNid) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.EDITPERSPECTIVE);
          dataSerial1.setObject(name);
          dataSerial1.setObject((String) new Integer(oldNid).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    inheritPerspective()
    */
    public int inheritPerspective(int theChild, int theParent) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.INHERITPERSPECTIVE);
          dataSerial1.setObject((String) new Integer(theChild).toString());
          dataSerial1.setObject((String) new Integer(theParent).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deletePerspective()
    */
    public int deletePerspective(int oldNid) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.DELETEPERSPECTIVE);
          dataSerial1.setObject((String) new Integer(oldNid).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createCategory()
    */
    public int deleteCategory(String name, int prevNid, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.CREATECATEGORY);
          dataSerial1.setObject(name);
          dataSerial1.setObject((String) new Integer(prevNid).toString());
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deleteCategory()
    */
    public int deleteCategory(int oldNid) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.DELETECATEGORY);
          dataSerial1.setObject((String) new Integer(oldNid).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createPerson()
    */
    public int createPerson(String fullName, int entryKind, String userName,
                             String password, int home_perspective_id,
                             String first, String middle, String last, String email,
                             String homepage, String sex, String color, String workplace,
                             String workTitle, String homePhone, String workPhone) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.CREATEPERSON);
          dataSerial1.setObject(fullName);
          dataSerial1.setObject((String) new Integer(entryKind).toString());
          dataSerial1.setObject(userName);
          dataSerial1.setObject(password);
          dataSerial1.setObject((String) new Integer(home_perspective_id).toString());
          dataSerial1.setObject(first);
          dataSerial1.setObject(middle);
          dataSerial1.setObject(last);
          dataSerial1.setObject(email);
          dataSerial1.setObject(homepage);
          dataSerial1.setObject(sex);
          dataSerial1.setObject(color);
          dataSerial1.setObject(workplace);
          dataSerial1.setObject(workTitle);
          dataSerial1.setObject(homePhone);
          dataSerial1.setObject(workPhone);

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deletePerson()
    */
    public int deletePerson(int oldNid) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.DELETEPERSON);
          dataSerial1.setObject((String) new Integer(oldNid).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createGroup()
    */
    public int createGroup(String password, String name, String description,
                           int representative, String email, int home_perspective_id) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.CREATEGROUP);
          dataSerial1.setObject(password);
          dataSerial1.setObject(name);
          dataSerial1.setObject(description);
          dataSerial1.setObject((String) new Integer(representative).toString());
          dataSerial1.setObject(email);
          dataSerial1.setObject((String) new Integer(home_perspective_id).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deleteGroup()
    */
    public int deleteGroup(int oldNid) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.DELETEGROUP);
          dataSerial1.setObject((String) new Integer(oldNid).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createWebsite()
    */
    public int createWebsite(int kind, int p, String name, String description, int prevNid,
                             String url, String bookmark, boolean after) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.CREATEWEBSITE);
          dataSerial1.setObject((String) new Integer(kind).toString());
          dataSerial1.setObject((String) new Integer(p).toString());
          dataSerial1.setObject(name);
          dataSerial1.setObject(description);
          dataSerial1.setObject((String) new Integer(prevNid).toString());
          dataSerial1.setObject(url);
          dataSerial1.setObject(bookmark);
          dataSerial1.setObject((String) new Boolean(after).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deleteWebsite()
    */
    public int deleteWebsite(int oldNid, int p) {
        int returnInt = 0;
        try {
          initializeMethod();
          //dataSerial1.clearObjects();
          dataSerial1.setCommand(msg.DELETEWEBSITE);
          dataSerial1.setObject((String) new Integer(oldNid).toString());
          dataSerial1.setObject((String) new Integer(p).toString());

          netClient1.write(dataSerial1);
          dataSerial1 = netClient1.read();
          returnInt = dataSerial1.getStatus();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    multiNodeDisplay()
    */
    public void multiNodeDisplay(String s) {
        try {
          initializeMethod();
          dataSerial1.setCommand(msg.MULTINODEDISPLAY);
          dataSerial1.setObject(s);
          netClient1.write(dataSerial1);       //do not wait for return
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

}

