/**
*  Title:           InterfaceMgrServer for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     Methods for interfaces to COLAB InterfaceMgrClient
*                   Stubs that call methods in components define the interface
*                   Most Exception handling done in this component
*                   Most database opening and closing done in this component
*/

package pow2000;

import java.util.*;
import java.sql.*;

/**
InterfaceMgr() is a class with methods for manipulating data stored in a relational database
that preserves perspectives.
*/
public class InterfaceMgrServer {
    private InterfaceDataSerialized msg;
    //private InterfaceMessageDef msgDef;
    private DatabaseMgr databaseMgr1;
    private ListMgr listMgr1;
    private ChangeMgr changeMgr1;

    /**
    InterfaceMgr() constucts an InterfaceMgr object.
    */
    public InterfaceMgrServer(InterfaceDataSerialized msgPersp) {
        try {
            msg = msgPersp;
            jbInit();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    InterfaceMgr() constucts an InterfaceMgr object.
    */
    public InterfaceMgrServer() {
        try {
          jbInit();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
    jbInit() constucts a DatabaseMgr, ListMgr and ChangeMgr
    */
    void jbInit() throws Exception {
            databaseMgr1 = new DatabaseMgr();
            listMgr1 = new ListMgr(databaseMgr1);
            changeMgr1 = new ChangeMgr(databaseMgr1);
            //msgDef = new InterfaceMessageDef();
    }

    /**
    setMessage() sets msg
    */
    public void setMessage(InterfaceDataSerialized msgPersp) {
	          msg = msgPersp;
    }

/**
    ============================== General Methods =======================
*/
    /**
     * msgManager - Controls all operations in a perspective
     * the message comes from a client, which is serialized and deserialized in the server
     * the server creates perspMechanism, which in turn initializes the message class
     * the msgManager then call the operation that corresponds to the requested operation
     * Those operations are private, so they cannot be accessed from outside this class
     * they have to go through the msgManager()
     */
    public void msgManager () {
    Boolean b;
    Integer nid;
    Integer p;
    Integer kind;
    Integer from;
    Integer to;

    // save current author Nid
    databaseMgr1.state1.setCurrentAuthorNid(msg.getStatus());

    switch (msg.getCommand()) {
        case InterfaceMessageDef.GETSPACESLIST:
          msg.clearObjects();
          msg.setObject(getSpacesList());
        break;

        case InterfaceMessageDef.VALIDATELOGIN:
          String user = new String ((String) msg.firstElement());
          String pass = new String ((String) msg.nextElement());
          String space = new String ((String) msg.nextElement());
          checkDBname(space);                               //call checkDBName here
          int theNid = validateLogin(user, pass, space);
          msg.clearObjects();
          msg.setStatus(theNid);
          msg.setObject(databaseMgr1.state1);
        break;

        case InterfaceMessageDef.AUTOMATELOGIN:
          user = new String ((String) msg.firstElement());
          pass = new String ((String) msg.nextElement());
          space = new String ((String) msg.nextElement());
          checkDBname(space);                               //call checkDBName here
          theNid = validateLogin(user, pass, space);
          msg.clearObjects();
          msg.setStatus(theNid);
          msg.setObject(databaseMgr1.state1);
        break;

        case InterfaceMessageDef.GETNODESFORDISPLAY:
          nid = new Integer((String) msg.firstElement());
          Integer startNid = new Integer((String) msg.nextElement());
          Integer mode = new Integer((String) msg.nextElement());
          msg.clearObjects();
          msg.setObject(getNodesForDisplay(nid.intValue(), startNid.intValue(), mode.intValue()));
        break;

        case InterfaceMessageDef.LOGCHANGE:
          Integer logType = new Integer ((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          from = new Integer ((String) msg.nextElement());
          to = new Integer ((String) msg.nextElement());
          logChange(logType.intValue(), p.intValue(), from.intValue(), to.intValue());
          msg.clearObjects();
          msg.setStatus(InterfaceMessageDef.NORETURN);
        break;

        case InterfaceMessageDef.SETNEWPASSWORD:
          nid = new Integer ((String) msg.firstElement());
          pass = new String ((String) msg.nextElement());
          msg.clearObjects();
          setNewPassword(nid.intValue(),pass);
          msg.setStatus(0);
        break;

        case InterfaceMessageDef.GETSTATEMENT:
          nid = new Integer((String) msg.firstElement());
          msg.clearObjects();
          msg.setObject(getStatement(nid.intValue()));
        break;

        case InterfaceMessageDef.GETDESCRIPTION:
          nid = new Integer((String) msg.firstElement());
          msg.clearObjects();
          msg.setObject(getDescription(nid.intValue()));
        break;

        case InterfaceMessageDef.ADDITEM:
          nid = new Integer((String) msg.firstElement());
          NodeDataRow row = (NodeDataRow) msg.nextElement();
          p = new Integer((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(addItem(nid.intValue(),row,p.intValue(),b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.MOVEITEM:
          nid = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          Integer prev = new Integer ((String) msg.nextElement());
          Integer prevP = new Integer ((String) msg.nextElement());
          from = new Integer((String) msg.nextElement());
          to = new Integer ((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(moveItem(nid.intValue(),p.intValue(),prev.intValue(),prevP.intValue(),from.intValue(),to.intValue(),b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.COPYITEM:
          nid = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          from = new Integer((String) msg.nextElement());
          to = new Integer ((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(copyStatement(nid.intValue(),p.intValue(),from.intValue(),to.intValue(),b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.LINKITEM:
          nid = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          from = new Integer((String) msg.nextElement());
          to = new Integer ((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(linkItem(nid.intValue(),p.intValue(),from.intValue(),to.intValue(),b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.ADDSTATEMENT:
          p = new Integer((String) msg.firstElement());
          kind = new Integer ((String) msg.nextElement());
          user = new String ((String) msg.nextElement());
          nid = new Integer((String) msg.nextElement());
          pass = new String ((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(addStatement(p.intValue(),kind.intValue(),user,nid.intValue(),pass,b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.EDITSTATEMENT:
          kind = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          user = new String ((String) msg.nextElement());
          pass = new String ((String) msg.nextElement());
          nid = new Integer((String) msg.nextElement());
          msg.setStatus(editStatement(kind.intValue(),p.intValue(),user,pass,nid.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.DELETESTATEMENT:
          nid = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          msg.setStatus(deleteStatement(nid.intValue(),p.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.ADDDISCUSSION:
          p = new Integer((String) msg.firstElement());
          user = new String ((String) msg.nextElement());
          Integer prevNid = new Integer((String) msg.nextElement());
          Integer groupNid = new Integer((String) msg.nextElement());
          pass = new String ((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(addDiscussion(p.intValue(),user,prevNid.intValue(),groupNid.intValue(),pass,b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.CREATEPERSPECTIVE:
          user = new String ((String) msg.firstElement());
          nid = new Integer((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(createPerspective(user,nid.intValue(),b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.EDITPERSPECTIVE:
          user = new String ((String) msg.firstElement());
          nid = new Integer((String) msg.nextElement());
          msg.setStatus(editPerspective(user,nid.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.DELETEPERSPECTIVE:
          nid = new Integer((String) msg.firstElement());
          msg.setStatus(deletePerspective(nid.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.INHERITPERSPECTIVE:
          nid = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          msg.setStatus(inheritPerspective(nid.intValue(),p.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.CREATECATEGORY:
          user = new String ((String) msg.firstElement());
          nid = new Integer((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(createCategory(user,nid.intValue(),b.booleanValue()));
        break;

        case InterfaceMessageDef.DELETECATEGORY:
          nid = new Integer((String) msg.firstElement());
          msg.setStatus(deleteCategory(nid.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.CREATEPERSON:
          String fullName = new String ((String) msg.firstElement());
          Integer entryKind = new Integer ((String) msg.nextElement());
          String userName = new String ((String) msg.nextElement());
          String password = new String ((String) msg.nextElement());
          Integer home_perspective_id = new Integer ((String) msg.nextElement());
          String first = new String ((String) msg.nextElement());
          String middle = new String ((String) msg.nextElement());
          String last = new String ((String) msg.nextElement());
          String email = new String ((String) msg.nextElement());
          String homepage = new String ((String) msg.nextElement());
          String sex = new String ((String) msg.nextElement());
          String color = new String ((String) msg.nextElement());
          String workplace = new String ((String) msg.nextElement());
          String workTitle = new String ((String) msg.nextElement());
          String homePhone = new String ((String) msg.nextElement());
          String workPhone = new String ((String) msg.nextElement());
          msg.setStatus(createPerson(fullName, entryKind.intValue(), userName,
                             password, home_perspective_id.intValue(),
                             first, middle, last, email,
                             homepage, sex, color, workplace,
                             workTitle, homePhone, workPhone));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.DELETEPERSON:
          nid = new Integer((String) msg.firstElement());
          msg.setStatus(deletePerson(nid.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.CREATEGROUP:
          pass = new String ((String) msg.firstElement());
          user = new String ((String) msg.nextElement());
          String description = new String ((String) msg.nextElement());
          nid = new Integer ((String) msg.nextElement());
          email = new String ((String) msg.nextElement());
          p = new Integer ((String) msg.nextElement());
          msg.setStatus(createGroup(pass,user,description,nid.intValue(),email,p.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.DELETEGROUP:
          nid = new Integer((String) msg.firstElement());
          msg.setStatus(deleteGroup(nid.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.CREATEWEBSITE:
          kind = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          user = new String ((String) msg.nextElement());
          description = new String ((String) msg.nextElement());
          nid = new Integer((String) msg.nextElement());
          String url = new String ((String) msg.nextElement());
          String bookmark = new String ((String) msg.nextElement());
          b = new Boolean((String) msg.nextElement());
          msg.setStatus(createWebsite(kind.intValue(),p.intValue(),user,description,nid.intValue(),url,bookmark,b.booleanValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.DELETEWEBSITE:
          nid = new Integer((String) msg.firstElement());
          p = new Integer ((String) msg.nextElement());
          msg.setStatus(deleteWebsite(nid.intValue(),p.intValue()));
          msg.clearObjects();
        break;

        case InterfaceMessageDef.MULTINODEDISPLAY:
          String s = new String ((String) msg.firstElement());
          msg.setStatus(multiNodeDisplay(s));
          msg.clearObjects();
          msg.setStatus(InterfaceMessageDef.NORETURN);
        break;
      }  //switch
    } // msgManager


    /**
    * Return the result of the operation
    */
    public InterfaceDataSerialized getResult() {
      return msg;
    }

/**
    ============================== definition of methods =======================
*/

    /**
    openDatabase() open the database for queries.
    */
    public Connection openDatabase() {
        return databaseMgr1.openDatabase();
    }

    /**
    checkDBname() changes DB if necessary
    open database and Exception handling done in method code
    */
    public int checkDBname(String newDBname) {
        int DBsid = databaseMgr1.checkDBname(newDBname);
        if (DBsid > 0) {
            logChange(InterfaceMessageDef.LOG_CHANGE_DB, 0, 0, DBsid);
        }
        return 1;
    }

    /**
    getSpacesList() get Vector with alpha list of defined information spaces.
    */
    public Vector getSpacesList() {
        Vector returnVector = new Vector();
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnVector = listMgr1.getSpacesList(stmt);
            Vector newList = (Vector) returnVector.clone();
            databaseMgr1.state1.setListOfSpaces(newList);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnVector;
    }

    /**
    validateLogin() validates username and password in the database.
    */
    public int validateLogin(String username, String password, String space) {
       int nid = 0;

       try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            nid = databaseMgr1.validateLogin(username, password, space, stmt);
            int p = databaseMgr1.state1.getCurrentPerspectiveNid();
            databaseMgr1.state1.setListOfPerspectives(listMgr1.getPerspectiveAlphaList(stmt));
            databaseMgr1.state1.setTreeOfPerspectives(listMgr1.getPerspectiveTree(stmt));
            databaseMgr1.state1.setListOfKinds(listMgr1.getKindAlphaList(stmt));
            //open default tree of nodes
            int startNid = databaseMgr1.state1.getCOLAB_ROOT_CONTENT();
            int mode = InterfaceMessageDef.GET_NODES_PERSONAL;
            int viewer = p;
            databaseMgr1.state1.setTreeOfNodes(listMgr1.getNodesForDisplay(p, startNid, mode, stmt));
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return nid;
    }

    /**
    automateLogin() validates username and password in the database.
    */
    public int automateLogin(String username, String password, String space) {
       int nid = 0;
       try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            nid = databaseMgr1.validateLogin(username, password, space, stmt);
            int p = databaseMgr1.state1.getCurrentPerspectiveNid();
            databaseMgr1.state1.setListOfPerspectives(listMgr1.getPerspectiveAlphaList(stmt));
            databaseMgr1.state1.setTreeOfPerspectives(listMgr1.getPerspectiveTree(stmt));
            databaseMgr1.state1.setListOfKinds(listMgr1.getKindAlphaList(stmt));
            //open default tree of nodes
            int startNid = databaseMgr1.state1.getCOLAB_ROOT_CONTENT();
            int mode = InterfaceMessageDef.GET_NODES_PERSONAL;
            int viewer = p;
            databaseMgr1.state1.setTreeOfNodes(listMgr1.getNodesForDisplay(p, startNid, mode, stmt));
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return nid;
    }

    /**
    getNodesForDisplay() get Vector of TreeDataNodes.
    */
    public Vector getNodesForDisplay(int p, int startNid, int mode) {
        Vector returnVector = new Vector();
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnVector = listMgr1.getNodesForDisplay(p, startNid, mode, stmt);
            Vector newList = (Vector) returnVector.clone();
            databaseMgr1.state1.setTreeOfNodes(newList);
            databaseMgr1.state1.setCurrentPerspectiveNid(p);
            NodeDataRow theRow = databaseMgr1.getNodeRow(p, stmt);
            databaseMgr1.state1.setCurrentPerspectiveName(theRow.getName());
            databaseMgr1.state1.setTreeOfNodes(newList);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnVector;
    }

    /**
    logChange()
    */
    public void logChange(int logType, int p, int from_node, int to_node) {
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            int authorNid = databaseMgr1.state1.getCurrentAuthorNid();
            databaseMgr1.logChange(logType, p, from_node, to_node, authorNid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    setNewPassword() change author's password in people table
    */
    public void setNewPassword(int author, String password) {
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            databaseMgr1.setNewPassword(author, password, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_CHANGE_PASSWORD, 0, 0, 0);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    getStatement() retrieve memo text for a statement.
    */
    public String getStatement(int nid) {
        String memoText = "";
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            memoText = databaseMgr1.getMemo(nid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return memoText;
    }

    /**
    getDescription() returns a Vector with the nid's kind, name, description, author, memo
    */
    public Vector getDescription(int nid) {
        Vector descVector = new Vector();
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            descVector = databaseMgr1.getDescription(nid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        String memo = getMemo(nid);
        descVector.addElement(memo);
        return descVector;
    }

    /**
    getMemo() returns a Vector with the nid's kind, name, description, author

    */
    public String getMemo(int nid) {
        String theMemo = "";
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            theMemo = databaseMgr1.getMemo(nid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return theMemo;
    }


/**
    ============================== methods for changeMgr =======================
*/

     /**
    addItem()
    */
    public int addItem(int previousNid, NodeDataRow newRow, int p, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.addItem(previousNid, newRow, p, after, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    moveItem() move a node after/below a second node
    */
    public int moveItem(int theNid, int theOldParent, int thePrev, int thePrevParent, int oldP, int newP, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.moveItem(theNid, theOldParent, thePrev, thePrevParent, oldP, newP, after, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_MOVE_NOTE, oldP, theNid, returnInt);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    linkItem() -- link a child node in p1 to a parent in p2
    create a VCOPY of child in p2 if p1 != p2
    */
    public int linkItem(int childNid, int parentNid, int p1, int p2, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.linkItem(childNid, parentNid, p1, p2, after, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_LINK_NOTE, p2, childNid, parentNid);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    addStatement()
    */
    public int addStatement(int p, int kind, String name, int prevNid, String memo, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.addStatement(p, kind, name, prevNid, memo, 0, null, after, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_ADD_NOTE, p, prevNid, returnInt);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    editStatement()
    */
    public int editStatement(int kind, int p, String name, String memo, int oldNid) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.editStatement(kind, p, name, memo, oldNid, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_EDIT_NOTE, p, oldNid, returnInt);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

    /**
    deleteStatement()
    */
    public int deleteStatement(int oldNid, int p) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.deleteItem(oldNid, p, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_DELETE_NOTE, p, oldNid, 0);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }
    /**
    copyStatement() -- create a new node copy after/below a second node
    */
    public int copyStatement(int theNid, int thePrev, int oldP, int newP, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.copyStatement(theNid, thePrev, newP, after, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_COPY_NOTE, oldP, theNid, returnInt);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    addDiscussion()
    */
    public int addDiscussion(int p, String name, int prevNid, int groupNid, String memo, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.addDiscussion(p, name, prevNid, groupNid, memo, after, stmt);
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_ADD_DISCUSSION, p, prevNid, returnInt);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createPerspective()
    */
    public int createPerspective(String name, int prevNid, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.createPerspective(name, prevNid, after, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    editPerspective()
    */
    public int editPerspective(String name, int oldNid) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.editPerspective(name, oldNid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    inheritPerspective()
    */
    public int inheritPerspective(int theChild, int theParent) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.inheritPerspective(theChild, theParent, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deletePerspective()
    */
    public int deletePerspective(int oldNid) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.deletePerspective(oldNid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createCategory()
    */
    public int createCategory(String name, int prevNid, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.createCategory(name, prevNid, after, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deleteCategory()
    */
    public int deleteCategory(int oldNid) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.deleteCategory(oldNid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createPerson()
    */
    public int createPerson(String fullName, int entryKind, String userName,
                             String password, int home_perspective_id,
                             String first, String middle, String last, String email,
                             String homepage, String sex, String color, String workplace,
                             String workTitle, String homePhone, String workPhone) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.createPerson(fullName, entryKind, userName,
                             password, home_perspective_id,
                             first, middle, last, email,
                             homepage, sex, color, workplace,
                             workTitle, homePhone, workPhone, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deletePerson()
    */
    public int deletePerson(int oldNid) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.deletePerson(oldNid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createGroup()
    */
    public int createGroup(String password, String name, String description,
                           int representative, String email, int home_perspective_id) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.createGroup(password, name, description,
                                               representative, email, home_perspective_id, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deleteGroup()
    */
    public int deleteGroup(int oldNid) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.deleteGroup(oldNid, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    createWebsite()
    */
    public int createWebsite(int kind, int p, String name, String description, int prevNid,
                             String url, String bookmark, boolean after) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.createWebsite(kind, p, name, description,
                                                 prevNid, url, bookmark, after, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    deleteWebsite()
    */
    public int deleteWebsite(int oldNid, int p) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            returnInt = changeMgr1.deleteWebsite(oldNid, p, stmt);
            ColabDatabase.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

     /**
    multiNodeDisplay()
    */
    public int multiNodeDisplay(String s) {
        int returnInt = 0;
        try {
            Connection ColabDatabase = openDatabase();
            Statement stmt = ColabDatabase.createStatement();
            //output s as a file and call CGI
            ColabDatabase.close();
            logChange(InterfaceMessageDef.LOG_MULTINODEDISPLAY, 0, 0, 0);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return returnInt;
    }

}
