/**
*  Title:           InterfaceState
*  Version:         1.0
*  Copyright:       Copyright (c) 1999
*  Author:          Gerry
*  Organization:    L3D
*  Description:     Defines values of state variables from selected Space and current values in client application
*/

package pow2000;

import java.util.Vector;
import java.io.*;

/**
*  InterfaceState encapsulates state of the client application.
*  It is available to the Pow Server and to the client application, including database creation applications.
*  When a new author logs in or when a network connection is lost, state is copied between server and client.
*  InterfaceState maintains the values of global constants stored in the database of the selected Space,
*  values of the current Author and the current Perspective, and lists of Spaces, Perspectives, etc.
*/
public class InterfaceState implements Serializable {
    private  int COLAB_ROOT = -1;
    private  int COLAB_ROOT_CATEGORY = 0;
    private  int COLAB_ROOT_PERSPECTIVE = 0;
    private  int COLAB_ROOT_CONTENT = 0;
    private  int COLAB_ROOT_AUTHOR = 0;
    private  int COLAB_COMPARISON_PERSPECTIVE = 0;
    private  int COLAB_LAST = 0;
    private  int LINK_TYPE = 0;
    private  int NODE_KIND = 0;

    private  int EDIT = 0;
    private  int DELETE = 0;
    private  int DELETELINK = 0;
    private  int SYNONYM = 0;
    private  int PARENT = 0;
    private  int ISA = 0;
    private  int VCOPY = 0;
    private  int VCHILDREN = 0;

    private  int PERSPECTIVE = 0;
    private  int INFORMATION = 0;
    private  int PERSON = 0;
    private  int GROUP = 0;
    private  int TABLE = 0;
    private  int CONTENT = 0;
    private  int STATEMENT = 0;
    private  int DISCUSSION = 0;
    private  int PROPOSAL = 0;
    private  int PRIVATE = 0;
    private  int DECISION = 0;
    private  int BOOK = 0;
    private  int MULTIMEDIA = 0;
    private  int OBJECT = 0;
    private  int EMAIL = 0;

    private  int PEOPLE_TABLE = 0;
    private  int GROUPS_TABLE = 0;
    private  int AUXTABLES_TABLE = 0;
    private  int STATEMENTS_TABLE = 0;
    private  int DECISIONS_TABLE = 0;
    private  int BOOKS_TABLE = 0;
    private  int MULTIMEDIA_TABLE = 0;
    private  int OBJECTS_TABLE = 0;
    private  int EMAIL_TABLE = 0;
    private  int TRANSACT_TABLE = 0;

    //current state values in client application
    private int currentSpaceNid = 0;             //current info space
    private String currentSpaceName = "";
    private int currentAuthorNid = 0;            //Nid of user logged in
    private String currentAuthorName = "";
    private int currentPerspectiveNid = 0;       //perspective being displayed
    private String currentPerspectiveName = "";
    private int homePerspectiveNid = 0;          //home perspective of user logged in
    private String homePerspectiveName = "";
    private int rootPerspectiveNid = 0;          //the top group perspective
    private String rootPerspectiveName = "";
    private int comparisonPerspectiveNid = 0;    //the bottom group comparison perspective
    private String comparisonPerspectiveName = "";
    private int currentDisplayMode = 0;          //the mode for display (see InterfaceMessageDef)
    private int currentPerspectiveOwner = 0;     //person who owns the perspective being displayed

    //lists used in client application
    private Vector listOfSpaces = new Vector();         //for display in login pane
    private Vector listOfPerspectives = new Vector();   //alpha list for selection of Perspective
    private Vector treeOfPerspectives = new Vector();   //tree for display of perspective hierarchy
    private Vector listOfKinds = new Vector();          //list for selection of Node Kind
    private Vector treeOfNodes = new Vector();

    /**
    InterfaceState() constucts an InterfaceState object.
    */
    public InterfaceState() {
    }


    /**
    *    copy data from another InterfaceState Object to this one
    */

    public void copy(InterfaceState s2) {
        setCOLAB_ROOT(s2.getCOLAB_ROOT());
        setCOLAB_ROOT_CATEGORY(s2.getCOLAB_ROOT_CATEGORY());
        setCOLAB_ROOT_PERSPECTIVE(s2.getCOLAB_ROOT_PERSPECTIVE());
        setCOLAB_ROOT_CONTENT(s2.getCOLAB_ROOT_CONTENT());
        setCOLAB_ROOT_AUTHOR(s2.getCOLAB_ROOT_AUTHOR());
        setCOLAB_COMPARISON_PERSPECTIVE(s2.getCOLAB_COMPARISON_PERSPECTIVE());
        setCOLAB_LAST(s2.getCOLAB_LAST());
        setLINK_TYPE(s2.getLINK_TYPE());
        setNODE_KIND(s2.getNODE_KIND());
        setEDIT(s2.getEDIT());
        setDELETE(s2.getDELETE());
        setDELETELINK(s2.getDELETELINK());
        setSYNONYM(s2.getSYNONYM());
        setDELETELINK(s2.getDELETELINK());
        setSYNONYM(s2.getSYNONYM());
        setPARENT(s2.getPARENT());
        setISA(s2.getISA());
        setVCOPY(s2.getVCOPY());
        setVCHILDREN(s2.getVCHILDREN());
        setPERSPECTIVE(s2.getPERSPECTIVE());
        setINFORMATION(s2.getINFORMATION());
        setPERSON(s2.getPERSON());
        setGROUP(s2.getGROUP());
        setTABLE(s2.getTABLE());
        setCONTENT(s2.getCONTENT());
        setSTATEMENT(s2.getSTATEMENT());
        setDISCUSSION(s2.getDISCUSSION());
        setPROPOSAL(s2.getPROPOSAL());
        setPRIVATE(s2.getPRIVATE());
        setDECISION(s2.getDECISION());
        setBOOK(s2.getBOOK());
        setMULTIMEDIA(s2.getMULTIMEDIA());
        setOBJECT(s2.getOBJECT());
        setEMAIL(s2.getEMAIL());
        setPEOPLE_TABLE(s2.getPEOPLE_TABLE());
        setGROUPS_TABLE(s2.getGROUPS_TABLE());
        setAUXTABLES_TABLE(s2.getAUXTABLES_TABLE());
        setSTATEMENTS_TABLE(s2.getSTATEMENTS_TABLE());
        setDECISIONS_TABLE(s2.getDECISIONS_TABLE());
        setBOOKS_TABLE(s2.getBOOKS_TABLE());
        setMULTIMEDIA_TABLE(s2.getMULTIMEDIA_TABLE());
        setOBJECTS_TABLE(s2.getOBJECTS_TABLE());
        setEMAIL_TABLE(s2.getEMAIL_TABLE());
        setTRANSACT_TABLE(s2.getTRANSACT_TABLE());
        setCurrentSpaceNid(s2.getCurrentSpaceNid());
        setCurrentAuthorNid(s2.getCurrentAuthorNid());
        setCurrentPerspectiveNid(s2.getCurrentPerspectiveNid());
        setHomePerspectiveNid(s2.getHomePerspectiveNid());
        setComparisonPerspectiveName(s2.getComparisonPerspectiveName());
        setComparisonPerspectiveNid(s2.getComparisonPerspectiveNid());
        setRootPerspectiveName(s2.getRootPerspectiveName());
        setRootPerspectiveNid(s2.getRootPerspectiveNid());
        setCurrentSpaceName(s2.getCurrentSpaceName());
        setCurrentAuthorName(s2.getCurrentAuthorName());
        setCurrentPerspectiveName(s2.getCurrentPerspectiveName());
        setHomePerspectiveName(s2.getHomePerspectiveName());
        setCurrentDisplayMode(s2.getCurrentDisplayMode());
        setCurrentPerspectiveOwner(s2.getCurrentPerspectiveOwner());
        setListOfSpaces(s2.getListOfSpaces());
        setListOfPerspectives(s2.getListOfPerspectives());
        setTreeOfPerspectives(s2.getTreeOfPerspectives());
        setTreeOfNodes(s2.getTreeOfNodes());
        setListOfKinds(s2.getListOfKinds());
    }

    /**
    *    getter and setter methods
    */

    public int getCOLAB_ROOT() {
        return COLAB_ROOT;
    }

    public int getCOLAB_ROOT_PERSPECTIVE() {
        return COLAB_ROOT_PERSPECTIVE;
    }

    public int getCOLAB_ROOT_AUTHOR() {
        return COLAB_ROOT_AUTHOR;
    }

    public int getCOLAB_ROOT_CATEGORY() {
        return COLAB_ROOT_CATEGORY;
    }

    public int getCOLAB_ROOT_CONTENT() {
        return COLAB_ROOT_CONTENT;
    }
 
    public int getCOLAB_COMPARISON_PERSPECTIVE() {
        return COLAB_COMPARISON_PERSPECTIVE;
    }

    public void setCOLAB_COMPARISON_PERSPECTIVE(int newNid) {
        COLAB_COMPARISON_PERSPECTIVE = newNid;
    }

    public void setCOLAB_LAST(int newNid) {
        COLAB_LAST = newNid;
    }

    public int getCOLAB_LAST() {
        return COLAB_LAST;
    }

    public int getNODE_KIND() {
        return NODE_KIND;
    }

    public int getCONTENT() {
        return CONTENT;
    }

    public int getINFORMATION() {
        return INFORMATION;
    }

    public int getLINK_TYPE() {
        return LINK_TYPE;
    }

    public int getPERSPECTIVE() {
        return PERSPECTIVE;
    }


    public int getTABLE() {
        return TABLE;
    }

    public int getDISCUSSION() {
        return DISCUSSION;
    }


    public int getEDIT() {
        return EDIT;
    }

    public int getDELETELINK() {
        return DELETELINK;
    }

    public int getDELETE() {
        return DELETE;
    }

    public int getPARENT() {
        return PARENT;
    }

    public int getSYNONYM() {
        return SYNONYM;
    }

    public int getISA() {
        return ISA;
    }

    public int getVCOPY() {
        return VCOPY;
    }

    public int getVCHILDREN() {
        return VCOPY;
    }

    public int getSTATEMENT() {
        return STATEMENT;
    }

    public int getPERSON() {
        return PERSON;
    }

    public int getGROUP() {
        return GROUP;
    }

    public int getPROPOSAL() {
        return PROPOSAL;
    }

    public int getPRIVATE() {
        return PRIVATE;
    }

    public int getDECISION() {
        return DECISION;
    }

    public int getBOOK() {
        return BOOK;
    }

    public int getMULTIMEDIA() {
        return MULTIMEDIA;
    }

    public int getOBJECT() {
        return OBJECT;
    }

    public int getPEOPLE_TABLE() {
        return PEOPLE_TABLE;
    }

    public int getSTATEMENTS_TABLE() {
        return STATEMENTS_TABLE;
    }

    public int getDECISIONS_TABLE() {
        return DECISIONS_TABLE;
    }

    public int getGROUPS_TABLE() {
        return GROUPS_TABLE;
    }

    public int getBOOKS_TABLE() {
        return BOOKS_TABLE;
    }

    public int getMULTIMEDIA_TABLE() {
        return MULTIMEDIA_TABLE;
    }

    public int getOBJECTS_TABLE() {
        return OBJECTS_TABLE;
    }

    public int getAUXTABLES_TABLE() {
        return AUXTABLES_TABLE;
    }

    public int getEMAIL() {
        return EMAIL;
    }

    public int getEMAIL_TABLE() {
        return EMAIL_TABLE;
    }

    public int getTRANSACT_TABLE() {
        return TRANSACT_TABLE;
    }

    public void setCurrentSpaceNid(int newCurrentSpaceNid) {
      currentSpaceNid = newCurrentSpaceNid;
    }

    public int getCurrentSpaceNid() {
      return currentSpaceNid;
    }

    public void setCurrentAuthorNid(int newCurrentAuthorNid) {
      currentAuthorNid = newCurrentAuthorNid;
    }

    public int getCurrentAuthorNid() {
      return currentAuthorNid;
    }

    public void setCurrentPerspectiveNid(int newCurrentPerspectiveNid) {
      currentPerspectiveNid = newCurrentPerspectiveNid;
    }

    public int getCurrentPerspectiveNid() {
      return currentPerspectiveNid;
    }

    public void setHomePerspectiveNid(int newHomePerspectiveNid) {
      homePerspectiveNid = newHomePerspectiveNid;
    }

    public int getHomePerspectiveNid() {
      return homePerspectiveNid;
    }

    public void setCurrentSpaceName(String newCurrentSpaceName) {
      currentSpaceName = newCurrentSpaceName;
    }

    public String getCurrentSpaceName() {
      return currentSpaceName;
    }

    public void setCurrentAuthorName(String newCurrentAuthorName) {
      currentAuthorName = newCurrentAuthorName;
    }

    public String getCurrentAuthorName() {
      return currentAuthorName;
    }

    public void setCurrentPerspectiveName(String newCurrentPerspectiveName) {
      currentPerspectiveName = newCurrentPerspectiveName;
    }

    public String getCurrentPerspectiveName() {
      return currentPerspectiveName;
    }

    public void setHomePerspectiveName(String newHomePerspectiveName) {
      homePerspectiveName = newHomePerspectiveName;
    }

    public String getHomePerspectiveName() {
      return homePerspectiveName;
    }

    public void setComparisonPerspectiveName(String newComparisonPerspectiveName) {
      comparisonPerspectiveName = newComparisonPerspectiveName;
    }

    public String getComparisonPerspectiveName() {
      return comparisonPerspectiveName;
    }

    public void setRootPerspectiveName(String newRootPerspectiveName) {
      rootPerspectiveName = newRootPerspectiveName;
    }

    public String getRootPerspectiveName() {
      return rootPerspectiveName;
    }

    public void setListOfSpaces(java.util.Vector newListOfSpaces) {
      listOfSpaces = newListOfSpaces;
    }

    public java.util.Vector getListOfSpaces() {
      return listOfSpaces;
    }

    public void setListOfPerspectives(java.util.Vector newListOfPerspectives) {
      listOfPerspectives = newListOfPerspectives;
    }

    public java.util.Vector getListOfPerspectives() {
      return listOfPerspectives;
    }

    public void setTreeOfPerspectives(java.util.Vector newTreeOfPerspectives) {
      treeOfPerspectives = newTreeOfPerspectives;
    }

    public java.util.Vector getTreeOfPerspectives() {
      return treeOfPerspectives;
    }

    public void setTreeOfNodes(java.util.Vector newTreeOfNodes) {
      treeOfNodes = newTreeOfNodes;
    }

    public java.util.Vector getTreeOfNodes() {
      return treeOfNodes;
    }

    public void setListOfKinds(java.util.Vector newListOfKinds) {
      listOfKinds = newListOfKinds;
    }

    public java.util.Vector getListOfKinds() {
      return listOfKinds;
    }

    public void setAUXTABLES_TABLE(int newAUXTABLES_TABLE) {
      AUXTABLES_TABLE = newAUXTABLES_TABLE;
    }

    public void setBOOK(int newBOOK) {
      BOOK = newBOOK;
    }

    public void setBOOKS_TABLE(int newBOOKS_TABLE) {
      BOOKS_TABLE = newBOOKS_TABLE;
    }

    public void setCOLAB_ROOT(int newCOLAB_ROOT) {
      COLAB_ROOT = newCOLAB_ROOT;
    }

    public void setCOLAB_ROOT_AUTHOR(int newCOLAB_ROOT_AUTHOR) {
      COLAB_ROOT_AUTHOR = newCOLAB_ROOT_AUTHOR;
    }

    public void setCOLAB_ROOT_CATEGORY(int newCOLAB_ROOT_CATEGORY) {
      COLAB_ROOT_CATEGORY = newCOLAB_ROOT_CATEGORY;
    }

    public void setCOLAB_ROOT_CONTENT(int newCOLAB_ROOT_CONTENT) {
      COLAB_ROOT_CONTENT = newCOLAB_ROOT_CONTENT;
    }

    public void setCOLAB_ROOT_PERSPECTIVE(int newCOLAB_ROOT_PERSPECTIVE) {
      COLAB_ROOT_PERSPECTIVE = newCOLAB_ROOT_PERSPECTIVE;
    }

    public void setCONTENT(int newCONTENT) {
      CONTENT = newCONTENT;
    }

    public void setDECISION(int newDECISION) {
      DECISION = newDECISION;
    }

    public void setDECISIONS_TABLE(int newDECISIONS_TABLE) {
      DECISIONS_TABLE = newDECISIONS_TABLE;
    }

    public void setDELETE(int newDELETE) {
      DELETE = newDELETE;
    }

    public void setDELETELINK(int newDELETELINK) {
      DELETELINK = newDELETELINK;
    }

    public void setDISCUSSION(int newDISCUSSION) {
      DISCUSSION = newDISCUSSION;
    }

    public void setPROPOSAL(int newPROPOSAL) {
      PROPOSAL = newPROPOSAL;
    }

    public void setPRIVATE(int newPRIVATE) {
      PRIVATE = newPRIVATE;
    }

    public void setEDIT(int newEDIT) {
      EDIT = newEDIT;
    }

    public void setEMAIL(int newEMAIL) {
      EMAIL = newEMAIL;
    }

    public void setEMAIL_TABLE(int newEMAIL_TABLE) {
      EMAIL_TABLE = newEMAIL_TABLE;
    }

    public void setGROUP(int newGROUP) {
      GROUP = newGROUP;
    }

    public void setGROUPS_TABLE(int newGROUPS_TABLE) {
      GROUPS_TABLE = newGROUPS_TABLE;
    }

    public void setINFORMATION(int newINFORMATION) {
      INFORMATION = newINFORMATION;
    }

    public void setISA(int newISA) {
      ISA = newISA;
    }

    public void setLINK_TYPE(int newLINK_TYPE) {
      LINK_TYPE = newLINK_TYPE;
    }

    public void setMULTIMEDIA(int newMULTIMEDIA) {
      MULTIMEDIA = newMULTIMEDIA;
    }

    public void setMULTIMEDIA_TABLE(int newMULTIMEDIA_TABLE) {
      MULTIMEDIA_TABLE = newMULTIMEDIA_TABLE;
    }

    public void setNODE_KIND(int newNODE_KIND) {
      NODE_KIND = newNODE_KIND;
    }

    public void setOBJECT(int newOBJECT) {
      OBJECT = newOBJECT;
    }

    public void setOBJECTS_TABLE(int newOBJECTS_TABLE) {
      OBJECTS_TABLE = newOBJECTS_TABLE;
    }

    public void setPARENT(int newPARENT) {
      PARENT = newPARENT;
    }

    public void setPEOPLE_TABLE(int newPEOPLE_TABLE) {
      PEOPLE_TABLE = newPEOPLE_TABLE;
    }

    public void setPERSON(int newPERSON) {
      PERSON = newPERSON;
    }

    public void setPERSPECTIVE(int newPERSPECTIVE) {
      PERSPECTIVE = newPERSPECTIVE;
    }

    public void setSTATEMENT(int newSTATEMENT) {
      STATEMENT = newSTATEMENT;
    }

    public void setSTATEMENTS_TABLE(int newSTATEMENTS_TABLE) {
      STATEMENTS_TABLE = newSTATEMENTS_TABLE;
    }

    public void setSYNONYM(int newSYNONYM) {
      SYNONYM = newSYNONYM;
    }

    public void setTABLE(int newTABLE) {
      TABLE = newTABLE;
    }

    public void setTRANSACT_TABLE(int newTRANSACT_TABLE) {
      TRANSACT_TABLE = newTRANSACT_TABLE;
    }

    public void setVCHILDREN(int newVCHILDREN) {
      VCHILDREN = newVCHILDREN;
    }

    public void setVCOPY(int newVCOPY) {
      VCOPY = newVCOPY;
    }

    public void setComparisonPerspectiveNid(int newComparisonPerspectiveNid) {
        comparisonPerspectiveNid = newComparisonPerspectiveNid;
    }

    public int getComparisonPerspectiveNid() {
        return comparisonPerspectiveNid;
    }

    public void setRootPerspectiveNid(int newRootPerspectiveNid) {
        rootPerspectiveNid = newRootPerspectiveNid;
    }

    public int getRootPerspectiveNid() {
        return rootPerspectiveNid;
    }

    public void setCurrentDisplayMode(int newCurrentDisplayMode) {
        currentDisplayMode = newCurrentDisplayMode;
    }

    public int getCurrentDisplayMode() {
        return currentDisplayMode;
    }

    public void setCurrentPerspectiveOwner(int newCurrentPerspectiveOwner) {
        currentPerspectiveOwner = newCurrentPerspectiveOwner;
    }

    public int getCurrentPerspectiveOwner() {
        return currentPerspectiveOwner;
    }

}
