/**
*  Title:           LinkDataRow for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     LinkDataRow defines a class of DataRows with content from the links table of the database.
*
*              Structure of a row of data from the links table:
*                1. int lid
*                2. int type
*                3. int perspective
*                4. int fromnode
*                5. int tonode
*                6. int seq
*                7. int author
*                8. date dt_create
*                *. int level   (not in links table)
*
*   link_types:
*         EDIT: (lid, EDIT, p, oldNid, newNid, seq, author, time)
*         DELETE: (lid, DELETE, p, oldNid, 0, seq, author, time)
*         SYNONYM: (lid, SYNONYM, p, categoryNid, synonymNid, seq, author, time)
*         PARENT: (lid, PARENT, p, childNid, parentNid, seq, author, time)
*         ISA: (lid, ISA, p, subCategoryNid, superCategoryNid, seq, author, time)
*         VCOPY: (lid, VCOPY, p2, oldNid, 0, seq, author, time)
*
*/

package pow2000;

import java.sql.ResultSet;
import java.sql.Timestamp;
import java.sql.SQLException;

public class LinkDataRow implements Cloneable{
    private int lid = 0;                //the lid of this item in the links table
    private int type = 0;               //the link type of this item
    private int perspective = 0;        //the original perspective for this item
    private int from = 0;               //the node from which this relationship is defined
    private int to = 0;                 //the node to which this relationship is defined
    private int sequence = 0;           //sequence of this item under it's parent
    private int author = 0;             //author of this item
    private Timestamp create = null;    //date and time this item was created
    private int level = 0;

    public LinkDataRow() {
            jbInit();
    }

    /**
    LinkDataRow(rs) encapsulates the structure of the links table.
    It takes a resutset with at least 10 fields and stores it in a LinkDataRow.
    The other COLAB components use the getter and setter methods of LinkDataRow to
    access and manipulate table data using the names defined in this class.
    */
    public LinkDataRow(ResultSet rs) throws SQLException{
        setLid(rs.getInt(1));
        setType(rs.getInt(2));
        setPerspective(rs.getInt(3));
        setFrom(rs.getInt(4));
        setTo(rs.getInt(5));
        setSequence(rs.getInt(6));
        setAuthor(rs.getInt(7));
        setCreate(rs.getTimestamp(8));
        jbInit();
    }

    public LinkDataRow(int theLid, int theType, int thePerspective, int theFrom,
                       int theTo, int theSequence, int theAuthor, Timestamp theCreate) {
        setLid(theLid);
        setType(theType);
        setPerspective(thePerspective);
        setFrom(theFrom);
        setTo(theTo);
        setSequence(theSequence);
        setAuthor(theAuthor);
        setCreate(theCreate);
        jbInit();
    }

    public synchronized Object clone() {
        LinkDataRow theClone = new LinkDataRow(lid, type, perspective, from, to, sequence, author, create);
        return theClone;
    }

    private void jbInit() {
    }

    public static void main(String[] args) {
        LinkDataRow linkDataRow1 = new LinkDataRow();
    }


    public String toString() {
        String theString = "Lid: " + lid;
        return theString;
}

    public String asString() {
        String theString = "LinkDataRow";
        theString += ": Lid= " + lid;
        theString += "; Type= " + type;
        theString += "; Persp= " + perspective;
        theString += "; From= " + from;
        theString += "; To= " + to;
        theString += "; Seq= " + sequence;
        theString += "; Author= " + author;
        theString += "; Create= " + create;
        return theString;
    }

    public String getStringValue(String columnName) {
        if (columnName.equals("lid")) {return "" + lid;}
        else if (columnName.equals("type")) {return "" + type;}
        else if (columnName.equals("perspective")) {return "" + perspective;}
        else if (columnName.equals("from")) {return "" + from;}
        else if (columnName.equals("to")) {return "" + to;}
        else if (columnName.equals("sequence")) {return "" + sequence;}
        else if (columnName.equals("author")) {return "" + author;}
        else if (columnName.equals("create")) {return "" + create;}
        else {return "";}
    }

    public void incrLevel() {
        setLevel(getLevel() + 1);
    }

    public void setLid(int newLid) {
        lid = newLid;
    }

    public int getLid() {
        return lid;
    }

    public void setType(int newType) {
        type = newType;
    }

    public int getType() {
        return type;
    }

    public void setPerspective(int newPerspective) {
        perspective = newPerspective;
    }

    public int getPerspective() {
        return perspective;
    }

    public void setFrom(int newFrom) {
        from = newFrom;
    }

    public int getFrom() {
        return from;
    }

    public void setTo(int newTo) {
        to = newTo;
    }

    public int getTo() {
        return to;
    }

    public void setSequence(int newSequence) {
        sequence = newSequence;
    }

    public int getSequence() {
        return sequence;
    }

    public void setAuthor(int newAuthor) {
        author = newAuthor;
    }

    public int getAuthor() {
        return author;
    }

    public void setCreate(Timestamp newCreate) {
        create = newCreate;
    }

    public Timestamp getCreate() {
        return create;
    }

    public void setLevel(int newLevel) {
        level = newLevel;
    }

    public int getLevel() {
        return level;
    }

}

