/**
*  Title:           ListMgr for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     computes lists of contents for displays
*/

package pow2000;

import java.sql.*;
import java.util.*;
import java.lang.*;
import com.objectspace.jgl.OrderedSet;
import com.objectspace.jgl.BinaryPredicate;
import com.objectspace.jgl.OrderedSetIterator;

public class ListMgr  {
    DatabaseMgr databaseMgr1;
    VisiblePerspectivePredicate vp = new VisiblePerspectivePredicate();
    VisibleOrderedSet foundVisibles = new VisibleOrderedSet(vp, false);    //initialize foundVisibles

    public ListMgr() throws Exception {
        databaseMgr1 = new DatabaseMgr();
        jbInit();
    }

    public ListMgr(DatabaseMgr newDatabaseMgr) throws Exception {
        databaseMgr1 = newDatabaseMgr;
        jbInit();
    }

    public static void main(String[] args) throws Exception {
        ListMgr listMgr1 = new ListMgr();
    }

    private void jbInit() {
     }

    /**
    *  getPerspectiveTree() returns a Vector of TreeDataRows.
    *  Each row has a perspective under its parent.
    */
    public Vector getPerspectiveTree(Statement stmt) throws Exception {
        Vector v = databaseMgr1.perspectiveTree(databaseMgr1.state1.getCOLAB_ROOT_PERSPECTIVE(), stmt);
        return indexParents(v);
    }

    /**
    *  getPerspectiveList() returns a Vector of NodeDataRows.
    *  Each row has a perspective, with its name indented.
    */
    public Vector getPerspectiveList(int p, Statement stmt) throws Exception {
        Vector v = new Vector();
        Vector inList = databaseMgr1.perspectiveHierarchy(p, stmt);
        for (int i = 0; i < inList.size(); i++) {
            LinkDataRow linkRow = (LinkDataRow) inList.elementAt(i);  //from_node = perspective Nid, Level = indentations
            int thePerspective = linkRow.getLid();
            int level = linkRow.getLevel();
            NodeDataRow nodeRow = databaseMgr1.getNodeRow(thePerspective, stmt);
            String name = nodeRow.getDescription();
            for (int j = 0; j < level; j++) {name = "       " + name;}    //indent
            nodeRow.setName(name);
            v.addElement(nodeRow);
        }
        return v;
    }

    /**
    *  getPerspectiveAlphaList() returns a Vector of NodeDataRows.
    *  Each defined perspective has one row, in alpha order.
    */
    public Vector getPerspectiveAlphaList(Statement stmt) throws Exception {
        NodeAlphaPredicate bp = new NodeAlphaPredicate();
        NodeOrderedSet s = new NodeOrderedSet(bp, false);
        //get list of NodeDataRows for all perspectives
        String theQuery = "SELECT * FROM nodes WHERE kind = " + databaseMgr1.state1.getPERSPECTIVE();
        ResultSet rs = stmt.executeQuery(theQuery);
        while (rs.next()) {
            s.add(new NodeDataRow(rs));
        }
        //put in a Vector
        Vector v = new Vector();
        Enumeration iterator = s.elements();
        while (iterator.hasMoreElements()) {
            NodeDataRow row = (NodeDataRow) iterator.nextElement();
            v.addElement(row);
        }
        return v;
    }

    /**
    *  getKindAlphaList() returns a Vector of NodeDataRows.
    *  Each defined kind has one row, in alpha order.
    */
    public Vector getKindAlphaList(Statement stmt) throws Exception {
        NodeAlphaPredicate bp = new NodeAlphaPredicate();
        NodeOrderedSet s = new NodeOrderedSet(bp, false);
        //get list of NodeDataRows for all perspectives
        String theQuery = "SELECT * FROM nodes WHERE kind = " + databaseMgr1.state1.getNODE_KIND() +
                                               " AND nid > " + databaseMgr1.state1.getCOLAB_LAST();
        ResultSet rs = stmt.executeQuery(theQuery);
        while (rs.next()) {
            s.add(new NodeDataRow(rs));
        }
        //put in a Vector
        Vector v = new Vector();
        Enumeration iterator = s.elements();
        while (iterator.hasMoreElements()) {
            NodeDataRow row = (NodeDataRow) iterator.nextElement();
            row.setName(row.getName().toLowerCase());       //force lower case for standard kinds
            v.addElement(row);
        }
        return v;
    }

    /**
    *  getSpacesList() returns a Vector of spaces names.
    */
    public Vector getSpacesList(Statement stmt) throws Exception {
        Vector v = new Vector();
        String theQuery = "SELECT name FROM spaces";
        ResultSet rs = stmt.executeQuery(theQuery);
        while (rs.next()) {
            v.addElement(rs.getString(1));
        }
        return v;
    }

    public String getPerspectiveName(int p, Statement stmt) throws Exception {
        String theString = "";
        theString = databaseMgr1.getNodeRow(p, stmt).getDescription();
        return theString;
    }

    public Vector getLogonsList(Statement stmt) throws SQLException, ClassNotFoundException {
        String theString = "" + databaseMgr1.state1.getPERSON();
        Vector theList = databaseMgr1.queryNodesOrLinks(databaseMgr1.GET_ID, databaseMgr1.GET_LIST, databaseMgr1.GET_NODES, "kind", theString, "", "", stmt);
        return theList;
    }

    /**
    getNodeRow(Nid) returns the NodeDataRow from the nodes table with the given Nid.
    Search for lowerBound in the OrderedSet is guaranteed to be O(log N) using red & black trees.
    */
    public NodeDataRow getNodeRow(int theNid, NodeOrderedSet nodesCache) {
        NodeDataRow targetRow = new NodeDataRow();
        targetRow.setNid(theNid);
        OrderedSetIterator i = nodesCache.lowerBound(targetRow);
        if (i.atEnd()) {
            return new NodeDataRow();
        }
        else {
            return (NodeDataRow) i.get();
        }
    }

    /**
    visible() is a recursive function returning an OrderedSet of Integers
    representing all nodes visible in perspective p
    */
    public OrderedSet visible(NodeOrderedSet nodesCache, LinkOrderedSet linksCache, int p, int startNid, int mode) {
        OrderedSet theVisible = new OrderedSet();
        NodeDataRow nodeRow = new NodeDataRow();
        LinkDataRow linkRow = new LinkDataRow();
        VisibleDataRow foundVisibleDataRow = new VisibleDataRow();
        OrderedSet foundOrderedSet = new OrderedSet();
        OrderedSet newOrderedSet = new OrderedSet();
        //add nid of nodes in p
        Enumeration iterator = nodesCache.elements();
        while (iterator.hasMoreElements()) {
            nodeRow = (NodeDataRow) iterator.nextElement();
            if (nodeRow.getPerspective() == p) {
                if (AddInMode(nodeRow, p, startNid, mode)) {           //check for addInMode below
                    theVisible.add(new Integer(nodeRow.getNid()));
                }
            }
        }
        //find parents of p, recurse, and union results
        iterator = linksCache.elements();
        while (iterator.hasMoreElements()) {
            linkRow = (LinkDataRow) iterator.nextElement();
            if (linkRow.getFrom() == p & linkRow.getType() == databaseMgr1.state1.getPARENT()) {
                int aParent = linkRow.getTo();
                if (aParent > 0) {
                    //union with list already calculated in foundVisibles or compute it if not found
                    foundVisibleDataRow = foundVisibles.find(aParent);
                    int foundP = foundVisibleDataRow.getPerspective();
                    if (foundP == 0 || foundP != aParent) {                     //signalled not already found
                        newOrderedSet = visible(nodesCache, linksCache, aParent, startNid, mode);
                        theVisible = theVisible.union(newOrderedSet);
                        foundVisibles.add(new VisibleDataRow(aParent, newOrderedSet));   //save returned list for later use
                    }
                    else {
                        theVisible = theVisible.union(foundVisibleDataRow.getOrderedSet());
                    }
                }
            }
        }
        //delete nodes ("from") that are deleted or edited in p
        //and add nodes ("from") that are VCOPYed in p
        //since linksCache is in chronological (lid) order, multiple VCOPY and DELETE should be OK
        iterator = linksCache.elements();
        while (iterator.hasMoreElements()) {
            linkRow = (LinkDataRow) iterator.nextElement();
            if (mode != InterfaceMessageDef.GET_NODES_ARCHIVE) {
            //if (DeleteInMode(mode, viewer)) {
                if ((linkRow.getPerspective() == p & linkRow.getType() == databaseMgr1.state1.getDELETE()) |
                    (linkRow.getPerspective() == p & linkRow.getType() == databaseMgr1.state1.getEDIT()) ) {
                    theVisible.remove(new Integer(linkRow.getFrom()));
                }
            }
            if (linkRow.getPerspective() == p & linkRow.getType() == databaseMgr1.state1.getVCOPY()) {
                nodeRow = getNodeRow(linkRow.getFrom(), nodesCache);
                if (AddInMode(nodeRow, p, startNid, mode)) {
                    theVisible.add(new Integer(linkRow.getFrom()));
                }
            }
        }
        return theVisible;
    }

    /**
    AddInMode()
    modes:
        GET_NODES_GROUP = 1;          -- no Discussion, Proposal, Decision, viewer's Private
        GET_NODES_DISCUSSION = 2;     -- no Proposal, Decision, viewer's Private
        GET_NODES_NEGOTIATION = 3;    -- viewer's Private
        GET_NODES_PERSONAL = 4;       -- viewer's Discussions, Proposals, Decisions, Private
        GET_NODES_OTHER = 5;          -- owner's Discussions, Proposals, Decisions, viewer's Private
        GET_NODES_COMPARISON = 6;     -- viewer's Private
        GET_NODES_ARCHIVE = 7;        -- viewer's Private, but include deleted notes
    Kinds:
        Discussion   -- visible in Discussion and Negotiation mode, and in author's or owner's perspective
        Proposal     -- visible in Negotiation mode and in author's or owner's perspective
        Decision     -- visible in Negotiation mode and in author's or owner's perspective
        Private      -- visible to author only
        Statement, Comment, Heading, Argument, etc. -- always visible
    */
    public boolean AddInMode(NodeDataRow nodeRow, int p, int startNid, int mode) {
        int theKind = nodeRow.getKind();
        //process Discussion notes
        if (theKind == databaseMgr1.state1.getDISCUSSION()) {
            if (mode == InterfaceMessageDef.GET_NODES_GROUP) {return false; }
            if (mode == InterfaceMessageDef.GET_NODES_PERSONAL) {
                if (databaseMgr1.state1.getCurrentAuthorNid() == nodeRow.getAuthor()) {return true; }
                else {return false; }
            }
            if (mode == InterfaceMessageDef.GET_NODES_OTHER) {
                if (databaseMgr1.state1.getCurrentPerspectiveOwner() == nodeRow.getAuthor()) {return true; }
                else {return false; }
            }
            return true;
        }
        //process Proposal or Decision notes
        if (theKind == databaseMgr1.state1.getPROPOSAL() || theKind == databaseMgr1.state1.getDECISION()) {
            if (mode == InterfaceMessageDef.GET_NODES_GROUP) {return false; }
            if (mode == InterfaceMessageDef.GET_NODES_DISCUSSION) {return false; }
            if (mode == InterfaceMessageDef.GET_NODES_PERSONAL) {
                if (databaseMgr1.state1.getCurrentAuthorNid() == nodeRow.getAuthor()) {return true; }
                else {return false; }
            }
            if (mode == InterfaceMessageDef.GET_NODES_OTHER) {
                if (databaseMgr1.state1.getCurrentPerspectiveOwner() == nodeRow.getAuthor()) {return true; }
                else {return false; }
            }
            return true;
        }
        //process Private notes
        if (theKind == databaseMgr1.state1.getPRIVATE()) {
            if (databaseMgr1.state1.getCurrentAuthorNid() == nodeRow.getAuthor()) {return true; }
            else return false;
        }
        //process all other notes
        return true;
    }

    /**
    DeleteInMode() --
    */
    public boolean DeleteInMode(int mode, int viewer) {
        //remove deleted and edited versions unless in archive mode
        return (mode != InterfaceMessageDef.GET_NODES_ARCHIVE);
    }

    /**
    getTree() -- take OrderedSet of visible nid Integers and put their descriptions in a Vector of TreeDataRows.
    Note that the TreeDataRows are here set to contain the contents of the NodeDataRows plus String equivalents.
    Formatting for display -- including recency computation -- is done in the Client.
    */
    public Vector getTree(OrderedSet theList, NodeOrderedSet nodesCache, LinkOrderedSet linksCache, int p) throws Exception {
        BinaryPredicate thePredicate = new TreeSeqPredicate();       //maintain in order by sequence
        OrderedSet inList = new OrderedSet(thePredicate, true);
        Vector tempList = new Vector();
        Vector outList = new Vector();
        String kindName;
        String authorName;
        String editorName;
        String perspectiveName;
        String dateName;
        if (theList.size() > 0) {
            Enumeration iterator = theList.elements();
            while (iterator.hasMoreElements()) {                    //loop thru nids returned by visible()
                Integer theInteger = (Integer) iterator.nextElement();
                int theNid = theInteger.intValue();
                NodeDataRow nodeRow = nodesCache.find(theNid);            //find row in nodesCache for this nid
                int kindNid = nodeRow.getKind();
                NodeDataRow kindRow = databaseMgr1.getKindCache().find(kindNid);
                int authorNid = nodeRow.getAuthor();
                NodeDataRow authorRow = databaseMgr1.getPeopleCache().find(authorNid);
                int editorNid = nodeRow.getEditor();
                NodeDataRow editorRow = databaseMgr1.getPeopleCache().find(editorNid);
                int perspectiveNid = nodeRow.getPerspective();
                NodeDataRow perspectiveRow = databaseMgr1.getPerspectiveCache().find(perspectiveNid);
                if (kindRow.getNid() == 0) {kindName = "";}
                    else {kindName = kindRow.getName().toLowerCase();}
                if (kindName.length() > 5 && kindName.substring(0,5).compareTo("COLAB") == 0) {kindName = "";}
                if (authorRow.getNid() == 0) {authorName = "";}
                    else {authorName = authorRow.getName();}
                if (authorName.length() > 5 && authorName.substring(0,5).compareTo("COLAB") == 0) {authorName = "";}
                if (editorRow.getNid() == 0) {editorName = "";}
                    else {editorName = editorRow.getName();}
                if (editorName.length() > 5 && editorName.substring(0,5).compareTo("COLAB") == 0) {editorName = "";}
                if (perspectiveRow.getNid() == 0) {perspectiveName = "";}
                    else {perspectiveName = perspectiveRow.getName().toLowerCase();}
                if (perspectiveName.length() > 5 && perspectiveName.substring(0,5).compareTo("COLAB") == 0) {perspectiveName = "";}
                //get position coordinates
                PositionDataRow pRow = getPRow(theNid, p);
                // create a new entry for this nid with each of its parents
                Vector theParents = getParentNodes(theNid, linksCache, p);     //find this node's parents and sequence among its children
                for (int i = 0; i < theParents.size(); i++) {
                    ParentSeqRow psRow = (ParentSeqRow) theParents.elementAt(i);
                    TreeDataRow theRow = new TreeDataRow();               //create a new row for the display tree
                    theRow.setNid(theNid);
                    theRow.setKind(kindNid);
                    theRow.setKindName(kindName);
                    theRow.setPerspective(perspectiveNid);
                    theRow.setPerspectiveName(perspectiveName);
                    theRow.setName(nodeRow.getName());
                    theRow.setDescription(nodeRow.getDescription());
                    theRow.setAuthor(authorNid);
                    theRow.setAuthorName(authorName);
                    theRow.setCreated(nodeRow.getCreated());
                    theRow.setEditor(editorNid);
                    theRow.setEditorName(editorName);
                    theRow.setModified(nodeRow.getModified());
                    theRow.setParent(psRow.getParent());
                    theRow.setX(pRow.getX());
                    theRow.setY(pRow.getY());
                    theRow.setSequence(psRow.getSequence());
                    inList.add(theRow);
                }
            }
        }
        TreeDataRow rootRow = new TreeDataRow();
        rootRow.setNid(databaseMgr1.state1.getCOLAB_ROOT_CONTENT());
        outList = reorder(inList, rootRow);    //reorder starting from COLAB_ROOT
        outList = indexParents(outList);       //set pointers within Vector
        return outList;
    }

    /**
    getPRow() returns a PositionDataRow
    */
    public PositionDataRow getPRow (int nid, int p) {
        Enumeration iterator = databaseMgr1.getPositionCache().elements();
        while (iterator.hasMoreElements()) {
            PositionDataRow pRow = (PositionDataRow) iterator.nextElement();
            if (pRow.getNid() == nid & pRow.getPerspective() == p) {
                return pRow;
            }
        }
        return new PositionDataRow();
    }

    /**
    perspectiveInheritance returns a Vector of Integer(nids) of ancestors of perspective p
    */
    public Vector perspectiveInheritance (LinkOrderedSet linksCache, int p) {
        Vector outList = new Vector();
        outList.addElement(new Integer(p));        //add row for p
        Vector parentList = perspectiveParents(linksCache, p);
        Enumeration iterator = parentList.elements();
        while (iterator.hasMoreElements()) {
            Integer pInteger = (Integer) iterator.nextElement();
            Vector v = perspectiveInheritance(linksCache, pInteger.intValue());
            for (int i = 0; i < v.size(); i++) {
                outList.addElement(v.elementAt(i));
            }
        }
        return outList;
    }

    /**
    perspectiveParents returns a Vector of Integer(nids) of parents of perspective p
   */
    public Vector perspectiveParents (LinkOrderedSet linksCache, int p) {
        Vector v = new Vector();
        Enumeration iterator = linksCache.elements();
        while (iterator.hasMoreElements()) {
            LinkDataRow linkRow = (LinkDataRow) iterator.nextElement();
            if (linkRow.getType() == databaseMgr1.state1.getPARENT() & linkRow.getFrom() == p) {
                v.addElement(new Integer(linkRow.getTo()));
            }
        }
        return v;
    }

    /**
    For a given nid in perspective p, getParentNodes() returns a Vector of ParentSeqRows
    containing a parent and its sequence among that parent's children.
    Implements multiple parents in directed acyclical graph (dag) of content.
    */
    public Vector getParentNodes(int nid, LinkOrderedSet linksCache, int p) throws Exception {
        Vector v = new Vector();                          //return Vector with 0 elements if no parents fourd
        ParentSeqRow psRow;
        LinkDataRow linkRow;
        LinkDataRow perspectiveRow;
        int parentType = databaseMgr1.state1.getPARENT();
        Enumeration iterator = linksCache.elements();
        while (iterator.hasMoreElements()) {
            linkRow = (LinkDataRow) iterator.nextElement();
            if (linkRow.getFrom() == nid) {                  //if nid is the child
                if (linkRow.getType() == parentType) {       //if it is a parent link
                                                             //if perspective is p or an ancestor of p
                    for (int i = 0; i < databaseMgr1.getPerspectiveInheritanceCache().size(); i++) {
                        perspectiveRow = (LinkDataRow) databaseMgr1.getPerspectiveInheritanceCache().elementAt(i);
                        if (p == perspectiveRow.getLid()) {
                            if (linkNotDeleted(linkRow, linksCache)) {          //if this link not deleted
                                psRow = new ParentSeqRow(linkRow.getTo(), linkRow.getSequence());
                                v.addElement(psRow);         //add parent found
                            }
                        }
                    }
                }

            }
        }
        return v;
    }

    /**
    linkNotDeleted()
    */
    private boolean linkNotDeleted(LinkDataRow linkRow, LinkOrderedSet linksCache) throws Exception {
        boolean notDeleted = true;
        LinkDataRow deleteLinkRow;
        int deleteType = databaseMgr1.state1.getDELETELINK();
        Enumeration iterator = linksCache.elements();
        while (iterator.hasMoreElements()) {
            deleteLinkRow = (LinkDataRow) iterator.nextElement();
            if (deleteLinkRow.getFrom() == linkRow.getFrom()) {     //if same child
                if (deleteLinkRow.getTo() == linkRow.getTo()) {     //if same parent
                    if (deleteLinkRow.getType() == deleteType) {    //if it is a deleteLink link
                        int linkP = linkRow.getPerspective();
                        int deleteP = deleteLinkRow.getPerspective();
                                                                    //if linkP = deleteP or an ancestor of deleteP
                        Vector deleteList = perspectiveInheritance (linksCache, deleteP); //look thru deleteList for linkP
                        Enumeration deleteIterator = deleteList.elements();
                        while (deleteIterator.hasMoreElements()) {
                            Integer pInteger = (Integer) deleteIterator.nextElement();
                            if (pInteger.intValue() == linkP) {
                                return false;                        //signal link deleted
                            }
                        }
                    }
                }
            }
        }
        return notDeleted;
    }

    /**
    reorder() takes an ordered (by seq) list of visible nodes and a parent node from the list (initially getCOLAB_ROOT_CONTENT())
    and returns a depth-first sort of the list for display.
    If a node has several parents, it will appear in the list several times, once with each parent.
    */
    private Vector reorder(OrderedSet inList, TreeDataRow parentRow) throws Exception {
        //create outList Vector to return
        Vector outList = new Vector();
        //add parentRow to outList
        outList.addElement((TreeDataRow) parentRow.clone());
        //iterate through inList looking for children of parentRow
        if (inList.size() > 0) {
            Enumeration iterator = inList.elements();
            while (iterator.hasMoreElements()) {                    //loop thru inList of potential child nodes
                TreeDataRow childRow = (TreeDataRow) iterator.nextElement();
                //if child found, recurse and add results to outList (depth-first)
                if (parentRow.getNid() == childRow.getParent()) {        //if found a child of this parent
                    Vector childList = reorder(inList, childRow);
                    for (int j = 0; j < childList.size(); j++) {
                        outList.addElement(childList.elementAt(j));
                    }
                }
            }
        }
        return outList;
    }

    private Vector indexParents (Vector outList) {
        if (outList.size() > 0) {
            for (int i = 1; i < outList.size(); i++) {
                TreeDataRow theRow = (TreeDataRow) outList.elementAt(i);
                int theIndex = getParentIndex(outList, theRow.getParent(), i);
                theRow.setParentIndex(theIndex);
            }
        }
        return outList;
    }

    private int getParentIndex(Vector outList, int theNid, int position) {
        if (outList.size() > 0) {
            for (int i = position - 1; i > -1; i--) {
                TreeDataRow theRow = (TreeDataRow) outList.elementAt(i);
                if (theRow.getNid() == theNid) {
                    return i;
                }
            }
        }
        return -1;    //default: parent is top level COLAB_ROOT
    }

    public Vector getNodesForDisplay(int p, int startNid, int mode, Statement stmt) throws Exception {
        long start = System.currentTimeMillis();
//System.out.println("call getNodesForDisplay");
        Vector theNodes;
        if (databaseMgr1.getPerspectiveInheritanceCachePerspectiveNid() != p) {
            databaseMgr1.setPerspectiveInheritanceCache(databaseMgr1.perspectiveInheritance(p, stmt));
        }
        NodeOrderedSet nodesCache = databaseMgr1.cacheNodes(p, stmt);                // refresh database cache
        LinkOrderedSet linksCache = databaseMgr1.cacheLinks(p, stmt);
        databaseMgr1.state1.setCurrentPerspectiveOwner(databaseMgr1.getCurrentPerspectiveOwner(p, stmt));
        VisiblePerspectivePredicate vp = new VisiblePerspectivePredicate();
        foundVisibles = new VisibleOrderedSet(vp, false);    //initialize foundVisibles
        OrderedSet theVisible = visible(nodesCache, linksCache, p, startNid, mode);
        theNodes = getTree(theVisible, nodesCache, linksCache, p);
        long end = System.currentTimeMillis() - start;
//databaseMgr1.db("time to get nodes = " + end/100 + "/10 secs.");
        return theNodes;
    }

 }

