/**
*  Title:           NodeDataRow for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1998
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     NodeDataRow defines a class of DataRows with content from the nodes table of the database.
*
*              Structure of a row of data from the nodes table:
*                1. int nid
*                2. int kind
*                3. int perspective
*                4. int suptable
*                5. int content
*                6. String name
*                7. String description
*                8. int author
*                9. date created
*                10. int editor
*                11. date modified
*/

package pow2000;

import java.sql.ResultSet;
import java.sql.Timestamp;
import java.sql.SQLException;
import java.io.Serializable;

public class NodeDataRow implements Cloneable, Serializable {
    private int nid = 0;                //the nid of this item in the links table
    private int kind = 0;               //the node kind of this item
    private int perspective = 0;        //the original perspective for this item
    private int suptable = 0;           //the table with additional info for this item
    private int content = 0;            //the row of the table with additional info for this item
    private String name = " ";          //the name of this item
    private String description = " ";   //text contents or description of this item
    private int author = 0;             //author of this item
    private Timestamp created = null;   //timestamp of cretion time
    private int editor = 0;             //author of this item
    private Timestamp modified = null;  //timestamp of last edit

    public NodeDataRow() {
            jbInit();
    }

    /**
    NodeDataRow(rs) encapsulates the structure of the nodes table.
    It takes a resutset with at least 10 fields and stores it in a NodeDataRow.
    The other COLAB components use the getter and setter methods of NodeDataRow to
    access and manipulate table data using the names defined in this class.
    */
    public NodeDataRow(ResultSet rs) throws SQLException{
        setNid(rs.getInt(1));
        setKind(rs.getInt(2));
        setPerspective(rs.getInt(3));
        setSuptable(rs.getInt(4));
        setContent(rs.getInt(5));
        setName(decode(rs.getString(6)));
        setDescription(decode(rs.getString(7)));
        setAuthor(rs.getInt(8));
        setCreated(rs.getTimestamp(9));
        setEditor(rs.getInt(10));
        setModified(rs.getTimestamp(11));
        jbInit();
    }

    public NodeDataRow(int theNid, int theKind, int thePerspective, int theSuptable, int theContent, String theName,
                       String theDescription, int theAuthor, Timestamp theCreated, int theEditor, Timestamp theModified) {
        setNid(theNid);
        setKind(theKind);
        setPerspective(thePerspective);
        setSuptable(theSuptable);
        setContent(theContent);
        setName(theName);
        setDescription(theDescription);
        setAuthor(theAuthor);
        setCreated(theCreated);
        setEditor(theEditor);
        setModified(theModified);
        jbInit();
    }

    public synchronized Object clone() {
        NodeDataRow theClone = new NodeDataRow(nid, kind, perspective, suptable, content, name, description, author, created, editor, modified);
        return theClone;
    }

    private void jbInit() {
    }

    public static void main(String[] args) {
        NodeDataRow NodeDataRow1 = new NodeDataRow();
    }


    public String toString() {
        String theString;
        if (name == null || name.trim().length() == 0 ) {theString = description; }
        else {theString = name; }
        if (theString == null) {theString = "  "; }
        return theString;
    }

    public String asString() {
        String theString = "NodeDataRow";
        theString += ": Nid= " + nid;
        theString += "; Kind= " + kind;
        theString += "; Persp= " + perspective;
        theString += "; Suptable= " + suptable;
        theString += "; Content= " + content;
        theString += "; Name= " + name;
        theString += "; Descr= " + description;
        theString += "; Author= " + author;
        theString += "; Created= " + created;
        theString += "; Editor= " + editor;
        theString += "; Modified= " + modified;
        return theString;
    }

    public String getStringValue(String columnName) {
        if (columnName.equals("nid")) {return "" + nid;}
        else if (columnName.equals("kind")) {return "" + kind;}
        else if (columnName.equals("perspective")) {return "" + perspective;}
        else if (columnName.equals("suptable")) {return "" + suptable;}
        else if (columnName.equals("content")) {return "" + content;}
        else if (columnName.equals("name")) {return "" + name;}
        else if (columnName.equals("description")) {return "" + description;}
        else if (columnName.equals("author")) {return "" + author;}
        else if (columnName.equals("created")) {return "" + created;}
        else if (columnName.equals("editor")) {return "" + editor;}
        else if (columnName.equals("modified")) {return "" + modified;}
        else {return "";}
    }

    public void setNid(int newNid) {
        nid = newNid;
    }

    public int getNid() {
        return nid;
    }

    public void setKind(int newKind) {
        kind = newKind;
    }

    public int getKind() {
        return kind;
    }

    public void setPerspective(int newPerspective) {
        perspective = newPerspective;
    }

    public int getPerspective() {
        return perspective;
    }

    public void setSuptable(int newSuptable) {
        suptable = newSuptable;
    }

    public int getSuptable() {
        return suptable;
    }

    public void setContent(int newContent) {
        content = newContent;
    }

    public int getContent() {
        return content;
    }

    public void setName(String newName) {
        name = newName;
    }

    public String getName() {
        return name;
    }

    public void setDescription(String newDescription) {
        description = newDescription;
    }

    public String getDescription() {
        return description;
    }

    public void setAuthor(int newAuthor) {
        author = newAuthor;
    }

    public int getAuthor() {
        return author;
    }

    public void setCreated(Timestamp newCreated) {
        created = newCreated;
    }

    public Timestamp getCreated() {
        return created;
    }

    public void setEditor(int newEditor) {
        editor = newEditor;
    }

    public int getEditor() {
        return editor;
    }

    public void setModified(Timestamp newModified) {
        modified = newModified;
    }

    public Timestamp getModified() {
        return modified;
    }

    /**
    decode(msg) decodes a \' as ' from SQL.
    */
    public String decode(String msg) {
        if (msg == null) {return " "; }
        String str = msg;
        if (str.length() < 1) {return " "; }
        int index = str.indexOf("\\" + "\'", 0);
        while (index > -1) {
            str = str.substring(0, index) + str.substring(index + 1);
            //str = str.substring(0, index) + str.substring(index);
            index = str.indexOf("\\" + "\'", index + 2);
        }
        return str;
    }

}

