/**
*  Title:           PositionDataRow
*  Version:         1.0
*  Copyright:       Copyright (c) 1999
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     PositionDataRow defines a class of DataRows with content from the positions table of the database.
*
*              Structure of a row of data from the nodes table:
*                1. int id
*                2. int nid
*                3. int perspective
*                4. int x
*                5. int y
*/

package pow2000;

import java.sql.ResultSet;
import java.sql.Timestamp;
import java.sql.SQLException;
import java.io.Serializable;

public class PositionDataRow implements Cloneable, Serializable {
    private int id = 0;                //the id of this item in the positions table
    private int nid = 0;               //the nid of this item in the nodes table
    private int perspective = 0;       //the perspective for this position
    private int x = 0;                 //the x coordinate of nid in perspective
    private int y = 0;                 //the y coordinate of nid in perspective

    public PositionDataRow() {
        setX(0);
        setY(0);
        jbInit();
    }

    /**
    PositionDataRow(rs) encapsulates the structure of the positions table.
    It takes a resutset with 5 fields and stores it in a PositionDataRow.
    The other COLAB components use the getter and setter methods of PositionDataRow to
    access and manipulate table data using the names defined in this class.
    */
    public PositionDataRow(ResultSet rs) throws SQLException{
        setId(rs.getInt(1));
        setNid(rs.getInt(2));
        setPerspective(rs.getInt(3));
        setX(rs.getInt(4));
        setY(rs.getInt(5));
        jbInit();
    }

    public PositionDataRow(int theId, int theNid, int thePerspective, int theX, int theY) {
        setId(theId);
        setNid(theNid);
        setPerspective(thePerspective);
        setX(theX);
        setY(theY);
        jbInit();
    }

    public synchronized Object clone() {
        PositionDataRow theClone = new PositionDataRow(id, nid, perspective, x, y);
        return theClone;
    }

    private void jbInit() {
    }

    public static void main(String[] args) {
        PositionDataRow positionDataRow1 = new PositionDataRow();
    }


    public String toString() {
        return "id, nid, p, x, y = " + id + ", " + nid + ", " + perspective + ", " + x + ", " + y;
    }

    public String asString() {
        String theString = "PositionDataRow";
        theString += ": Id= " + id;
        theString += "; Nid= " + nid;
        theString += "; Persp= " + perspective;
        theString += "; X= " + x;
        theString += "; Y= " + y;
        return theString;
    }

    public void setId(int newId) {
        id = newId;
    }

    public int getId() {
        return id;
    }

    public void setNid(int newNid) {
        nid = newNid;
    }

    public int getNid() {
        return nid;
    }

    public void setPerspective(int newPerspective) {
        perspective = newPerspective;
    }

    public int getPerspective() {
        return perspective;
    }

    public void setX(int newX) {
        x = newX;
    }

    public int getX() {
        return x;
    }

    public void setY(int newY) {
        y = newY;
    }

    public int getY() {
        return y;
    }

}

