/**
*  Title:           TreeDataRow for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 1999
*  Author:          Gerry Stahl
*  Organization:    L3D
*  Description:     structure of a row of data for a tree
*                1. nid                nid in nodes table
*                2. kind               node kind
*                3. kindName           String name of kind
*                4. perspective        home perspective of node
*                5. perspectiveName    String name of perspective
*                6. name               name field of node
*                7. description        description field of node
*                8. author             original author of node
*                9. authorName         String name of author
*                10. created           creation date and time
*                11. editor            last editor of node
*                12. editorName        String name of editor
*                13. modified          last modification date and time
*                14. parent            nid of parent node
*                15. node              pointer to node in tree
*                16. parentIndex       index to parent node in Vector
*                17. dummy             dummy int
*                18. x                 x coordinate of node in this perspective
*                19. y                 y coordinate of node in this perspective
*                20. sequence          sequence of node among children of its parent
*/

package pow2000;

import javax.swing.tree.DefaultMutableTreeNode;
import java.io.Serializable;
import java.sql.Timestamp;

public class TreeDataRow implements Cloneable, Serializable {
    private int nid = 0;
    private int kind = 0;
    private String kindName = "";
    private int perspective = 0;
    private String perspectiveName = "";
    private String name = "";
    private String description = "";
    private int author = 0;
    private String authorName = "";
    private Timestamp created = null;
    private int editor = 0;
    private String editorName = "";
    private Timestamp modified = null;
    private int parent = 0;
    private DefaultMutableTreeNode node = null;
    private int parentIndex = 0;
    private int dummy = 0;
    private int x = 0;
    private int y = 0;
    private int sequence = 0;

    public TreeDataRow() {
        jbInit();
    }

    public TreeDataRow(int newNid, int newKind,  String newKindName, int newPerspective, String newPerspectiveName,
                        String newName, String newDescription, int newAuthor, String newAuthorName, Timestamp newCreated,
                        int newEditor, String newEditorName, Timestamp newModified, int newParent, DefaultMutableTreeNode newNode,
                        int newParentIndex, int newDummy, int newX, int newY, int newSequence) {
        setNid(newNid);
        setKind(newKind);
        setKindName(newKindName);
        setPerspective(newPerspective);
        setPerspectiveName(newPerspectiveName);
        setName(newName);
        setDescription(newDescription);
        setAuthor(newAuthor);
        setAuthorName(newAuthorName);
        setCreated(newCreated);
        setEditor(newEditor);
        setEditorName(newEditorName);
        setModified(newModified);
        setParent(newParent);
        setNode(newNode);
        setParentIndex(newParentIndex);
        setDummy(newDummy);
        setX(newX);
        setY(newY);
        setSequence(newSequence);
        jbInit();
    }

    public synchronized Object clone() {
        TreeDataRow theClone = new TreeDataRow(nid, kind, kindName, perspective, perspectiveName,
                                                name, description, author, authorName, created,
                                                editor, editorName, modified, parent, node,
                                                parentIndex, dummy, x, y, sequence);
        return theClone;
    }

    private void jbInit() {
    }

    public static void main(String[] args) {
        TreeDataRow TreeDataRow1 = new TreeDataRow();
    }

    public String toString() {
        return description;
    }

    public String asString() {
        String theString = "TreeDataRow";
        theString += "; Nid= " + nid;
        theString += "; Kind= " + kind;
        theString += "; KindName= " + kindName;
        theString += "; Persp= " + perspective;
        theString += "; PerspName= " + perspectiveName;
        theString += ": Name= " + name;
        theString += ": Desc= " + description;
        theString += "; Parent= " + parent;
        theString += "; ParIndex= " + parentIndex;
        return theString;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String newDescription) {
        description = newDescription;
    }

    public String getName() {
        return name;
    }

    public void setName(String newName) {
        name = newName;
    }

    public int getParent() {
        return parent;
    }

    public void setParent(int newParent) {
        parent = newParent;
    }

    public DefaultMutableTreeNode getNode() {
        return node;
    }

    public void setNode(DefaultMutableTreeNode newNode) {
        node = newNode;
    }

    public int getNid() {
        return nid;
    }

    public void setNid(int newNid) {
        nid = newNid;
    }

    public int getPerspective() {
        return perspective;
    }

    public void setPerspective(int newPerspective) {
        perspective = newPerspective;
    }

    public String getPerspectiveName() {
        return perspectiveName;
    }

    public void setPerspectiveName(String newPerspectiveName) {
        perspectiveName = newPerspectiveName;
    }

    public int getKind() {
        return kind;
    }

    public void setKind(int newKind) {
        kind = newKind;
    }

    public int getDummy() {
        return dummy;
    }

    public void setDummy(int newDummy) {
        dummy = newDummy;
    }

    public void setParentIndex(int newParentIndex) {
        parentIndex = newParentIndex;
    }

    public int getParentIndex() {
        return parentIndex;
    }

    public void setKindName(String newKindName) {
        kindName = newKindName;
    }

    public String getKindName() {
        return kindName;
    }

    public void setAuthorName(String newAuthorName) {
        authorName = newAuthorName;
    }

    public String getAuthorName() {
        return authorName;
    }

    public void setAuthor(int newAuthor) {
        author = newAuthor;
    }

    public int getAuthor() {
        return author;
    }

    public void setEditor(int newEditor) {
        editor = newEditor;
    }

    public int getEditor() {
        return editor;
    }

    public void setCreated(java.sql.Timestamp newCreated) {
        created = newCreated;
    }

    public java.sql.Timestamp getCreated() {
        return created;
    }

    public void setModified(java.sql.Timestamp newModified) {
        modified = newModified;
    }

    public java.sql.Timestamp getModified() {
        return modified;
    }

    public void setEditorName(String newEditorName) {
        editorName = newEditorName;
    }

    public String getEditorName() {
        return editorName;
    }

    public void setX(int newX) {
        x = newX;
    }

    public int getX() {
        return x;
    }

    public void setY(int newY) {
        y = newY;
    }

    public int getY() {
        return y;
    }

    public void setSequence(int newSequence) {
        sequence = newSequence;
    }

    public int getSequence() {
        return sequence;
    }

}

 