/**
*  Title:           xmlconv for COLAB
*  Version:         1.0
*  Copyright:       Copyright (c) 2000
*  Author:          Gerry Stahl & Aaron Martin
*  Organization:    L3D
*  Description:     Class for XML DTD
*/

/***********************************************************************
***                                                                 ****
***  Filename: xmlconv.java                                         ****
***  Created: 02/02/2000                                            ****
***  Creator: Aaron Martin                                          ****
***  Absract -                                                      ****
***     This is a program that will convert recursive xml file to   ****
***     a non-recursive xml file and vice versa                     ****
***                                                                 ****
***  Syntax  -                                                      ****
***     xmlconv -n recursiveXMLfile.xml outputXMLfile.xml           ****
***         The -n flag specifies the conversion to take place      ****
***         in this case we are converting to a non-recursive       ****
***         xml file to a recursive one.  The first arg specifes    ****
***         the input XML to be converted.  The second arg flag     ****
***         is the output file.
***                                                                 ****
***     xmlconv -r nonrecursiveXMLfile.xml outputXMLfile.xml        ****
***         The -r flag specifies the conversion to take place,     ****
***         in this case the flag indicates that we are converting  ****
***         to a recursive xml from a non recursive XML input.      ****
***                                                                 ****
***  INVARIANT:                                                     ****
***     Converting from RECURSIVE XML to NON-RECURSIVE XML -        ****
***       1.  Because the recursive nodes do not contain unique id's ***
***           Notes that are the same but have different parents    ****
***           will have different unique identifers                 ****
***                                                                 ****
***     Converting from NON-RECURSIVE XML to RECURSIVE XML -        ****
***       1.  The unique indefiers will be lost during the conversion **
***       2.  Before a link is added both notes must be added to the ***
***           dataset                                               ****
***                                                                 ****
***   Version History:                                              ****
***     *******************     1.0.0.0   ***********************   ****
***     02/02/2000 - aamartin - Started                             ****
***                                                                 ****
************************************************************************/

package xml2000;

import java.io.*;
import com.ibm.xml.parsers.DOMParser;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.msg.*;
import java.io.IOException;


public class xmlconv {

  private static BufferedWriter xmlOut;
  private static Document doc;
  private static DOMParser parser;

  public static void main(String args[]) {

    if(args.length != 3) {
      message("Usage: xmlconv -rn inputFile outputFile");
      return;
    }

    Dataset master = new Dataset();
    xmlconv bob = new xmlconv(args[1], master);
    StringBuffer buf = new StringBuffer();


  //  master.generateNRXML(0);
    try {
      xmlOut = new BufferedWriter(new FileWriter(args[2]));
      if(args[0].compareTo("-n") == 0)
        master.generateNRXML(0, buf);
      else if (args[0].compareTo("-r") == 0) {
        master.generateRXML(0, buf);
      }
    }
    catch (Exception ex) {
      System.err.println("error with file stuff");
    }

    try {
         xmlOut.write(buf.toString());
           xmlOut.close();
       } catch (IOException ex) {
      System.err.println("Cannont write XML output file.");
           }
  }

   xmlconv (String filename, Dataset master) {
   parser = new DOMParser();

    try {
      parser.parse(filename);
    } catch (SAXException se) {
      se.printStackTrace();
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }

    doc = parser.getDocument();
    xmlGenerator(doc.getDocumentElement(), 0, master);
  }


  public static void xmlGenerator(Node node, int depth, Dataset master) {
    int currentNoteID;
    int depth1;  //cut me out

    int i, childCount;

    NodeList children;

    Node child;
    //Dataset master = new Dataset();
    spacer(depth);
    String text, value="";
    String NodeName;

    depth1 = depth;  //cut me out too

    parseAttributes(master, node.getAttributes());

    switch (node.getNodeType())
    {
      case Node.DOCUMENT_NODE:
        text = node.getNodeValue();
        break;
      case Node.ELEMENT_NODE:
        text = node.getNodeName();
        if (text.compareToIgnoreCase("DATASET") == 0) {
          if (node.hasChildNodes())  {
            children = node.getChildNodes();
            childCount = children.getLength();
            for(i = 0; i < childCount; i++) {
              child = children.item(i);
              NodeName = child.getNodeName();
              if (NodeName.compareToIgnoreCase("description") == 0) {
                if(child.hasChildNodes())
                  value = child.getFirstChild().getNodeValue();
                master.addDescription(value);
              }
              if (NodeName.compareToIgnoreCase("comment") == 0) {
                if(child.hasChildNodes())
                  value = child.getFirstChild().getNodeValue();
                master.addComment(value);
              }
              if (NodeName.compareToIgnoreCase("note") == 0) {
                Note tempNote = new Note(getNID(child.getAttributes()));
                parseNote(child, tempNote);//might have problems with refrencing
                master.addNote(tempNote);
              }
              if (NodeName.compareToIgnoreCase("link") == 0) {
                parseLink(child, master);
             }
            }
          }
        }
        break;
      case Node.ATTRIBUTE_NODE:
        text = node.getNodeName();
        break;
      case Node.PROCESSING_INSTRUCTION_NODE:
      case Node.COMMENT_NODE:
      case Node.TEXT_NODE:
      default:
        text = node.toString();
        break;
    }
    return;

  }

  static void parseLink(Node node, Dataset master) {
    String NodeName;
    NodeList children;
    int childCount;
    Node child;
    Link link = new Link();
    int to, from;

    parseLinkAttr(node.getAttributes(), link);

    if(node.hasChildNodes()) {
      children = node.getChildNodes();
      childCount = children.getLength();
      for(int i=0; i < childCount; i++) {
        child = children.item(i);
        NodeName = child.getNodeName();
        if(NodeName.compareToIgnoreCase("flocator") == 0)
          parseFlocatorAttr(child.getAttributes(), link);
        if(NodeName.compareToIgnoreCase("tlocator") == 0)
          parseTlocatorAttr(child.getAttributes(), link);
      }
    }
    master.addLink(link.getFromHref(), link.getToHref());
  }

  static void parseFlocatorAttr(NamedNodeMap attributes, Link link) {
    int i, attrCount;
    String attrName, attrValue;

    attrCount = attributes.getLength();

    for(i =0; i<attrCount; i++) {
      attrName = attributes.item(i).getNodeName();
      attrValue = attributes.item(i).getNodeValue();
      if(attrName.compareToIgnoreCase("xml:link") == 0)
        link.setFlocatorXML_Link(attrValue);
      if(attrName.compareToIgnoreCase("href") == 0)
        link.setFlocator((int) Float.valueOf(attrValue).floatValue());
      if(attrName.compareToIgnoreCase("role") == 0)
        link.setFlocatorRole(attrValue);
    }
  }

  static void parseTlocatorAttr(NamedNodeMap attributes, Link link) {
    int i, attrCount;
    String attrName, attrValue;

    attrCount = attributes.getLength();

    for(i =0; i<attrCount; i++) {
      attrName = attributes.item(i).getNodeName();
      attrValue = attributes.item(i).getNodeValue();
      if(attrName.compareToIgnoreCase("xml:link") == 0)
        link.setTlocatorXML_Link(attrValue);
      if(attrName.compareToIgnoreCase("href") == 0)
        link.setTlocator((int) Float.valueOf(attrValue).floatValue());
      if(attrName.compareToIgnoreCase("role") == 0)
        link.setTlocatorRole(attrValue);
    }
  }

  static void parseLinkAttr(NamedNodeMap attributes, Link link) {
    int i, attrCount;
    String attrName, attrValue;

    attrCount = attributes.getLength();

    for(i =0; i < attrCount; i++) {
      attrName = attributes.item(i).getNodeName();
      attrValue = attributes.item(i).getNodeValue();
      if(attrName.compareToIgnoreCase("xml:link") == 0)
        link.setxmllinkAttr(attrValue);
      if(attrName.compareToIgnoreCase("inline") == 0)
        link.setInline(attrValue);
      if(attrName.compareToIgnoreCase("role") == 0)
        link.setLinkRole(attrValue);
      if(attrName.compareToIgnoreCase("role") == 0)
        link.setLinkRole(attrValue);
      if(attrName.compareToIgnoreCase("title") == 0)
        link.setTitle(attrValue);
      if(attrName.compareToIgnoreCase("show") == 0)
        link.setShow(attrValue);
      if(attrName.compareToIgnoreCase("actuate") == 0)
        link.setActuate(attrValue);
      if(attrName.compareToIgnoreCase("behavior") == 0)
        link.setBehavior(attrValue);
      if(attrName.compareToIgnoreCase("content-role") == 0)
        link.setContent_Role(attrValue);
      if(attrName.compareToIgnoreCase("content-title") == 0)
        link.setContent_Title(attrValue);
      if(attrName.compareToIgnoreCase("lid") == 0)
        link.setLinkID((int) Float.valueOf(attrValue).floatValue());
      if(attrName.compareToIgnoreCase("type") == 0)
        link.setType(attrValue);
      if(attrName.compareToIgnoreCase("perspective") == 0)
        link.setPerspective(attrValue);
      if(attrName.compareToIgnoreCase("seq") == 0)
        link.setSeq(attrValue);
      if(attrName.compareToIgnoreCase("author") == 0)
        link.setAuthor(attrValue);
      if(attrName.compareToIgnoreCase("created") == 0)
        link.setCreated(attrValue);
    }
  }

  static void parseNote(Node node, Note note) {
    int depth1;  //cut me out

    int i, temp, childCount, authorCount;

    NodeList children, authorValues;

    Node child, authorNode;
    String text, value="";
    String NodeName, currentAuthorElement;

    depth1 = 1;  //cut me out too
    parseAttributes(note, node.getAttributes());

    switch (node.getNodeType())
    {
      case Node.DOCUMENT_NODE:
        text = node.getNodeValue();
        break;
      case Node.ELEMENT_NODE:
        text = node.getNodeName();
        if (text.compareToIgnoreCase("note") == 0) {
          if (node.hasChildNodes())  {
            children = node.getChildNodes();
            childCount = children.getLength();
            for(i = 0; i < childCount; i++) {
              child = children.item(i);
              NodeName = child.getNodeName();
              if (NodeName.compareToIgnoreCase("note") == 0) {
                if(child.hasChildNodes())
                  value = child.getFirstChild().getNodeValue();
                Note newChild = new Note(getNID(child.getAttributes()));
        //        Note newChild = new Note();
                parseNote(child, newChild);
                //parseAttributes(note, child.getAttributes());
                note.addChildNote(newChild);
              }
              if (NodeName.compareToIgnoreCase("title") == 0) {
                if(child.hasChildNodes())
                  value = child.getFirstChild().getNodeValue();
                note.setTitle(value);
              }
              if (NodeName.compareToIgnoreCase("content") == 0) {
                if(child.hasChildNodes())
                  value = child.getFirstChild().getNodeValue();
                note.setContent(value);
              }
              if (NodeName.compareToIgnoreCase("author") == 0 ) {
                author newAuthor = new author();
                parseAuthor(child, newAuthor);
                note.addAuthor(newAuthor);
              }
              if (NodeName.compareToIgnoreCase("additional") == 0) {
                if(child.hasChildNodes())
                  value = child.getFirstChild().getNodeValue();
                note.addAdditional(value);
              }

            }
          }
        }
        break;
      case Node.ATTRIBUTE_NODE:
        text = node.getNodeName();
        break;
      case Node.PROCESSING_INSTRUCTION_NODE:
      case Node.COMMENT_NODE:
      case Node.TEXT_NODE:
      default:
        text = node.toString();
        break;
    }
  }

  static int getNID(NamedNodeMap attributes) {
    for(int i=0; i<attributes.getLength(); i++) {
      if(attributes.item(i).getNodeName().compareToIgnoreCase("noteid") == 0)
        return (int) Float.valueOf(attributes.item(i).getNodeValue()).floatValue();
    }
    return Note.newNoteID();
  }

  static void parseAuthor(Node node, author newAuthor) {

   NodeList children;
   int i, temp, childcount;
   String NodeName, value;

   Node child;

   NamedNodeMap test;
   test = node.getAttributes();
   temp = test.getLength();

   switch (node.getNodeType())
   {
    case Node.ELEMENT_NODE:
      if (node.hasChildNodes()) {
        children = node.getChildNodes();
        childcount = children.getLength();
        for(i=0; i<childcount; i++) {
          child = children.item(i);
          NodeName = child.getNodeName();
          if(NodeName.compareToIgnoreCase("name") == 0) {
            value = child.getFirstChild().getNodeValue();
            newAuthor.setName(value);
          }
          if(NodeName.compareToIgnoreCase("email") == 0) {
            value = child.getFirstChild().getNodeValue();
            newAuthor.setEmail(value);
          }
          if(NodeName.compareToIgnoreCase("address") == 0) {
            value = child.getFirstChild().getNodeValue();
            newAuthor.addAddress(value);
          }
        }
      }
      break;
    default:
       System.out.println("don't know how to handle author element");
   }
  }

  static void parseAttributes(Dataset master, NamedNodeMap attributes) {
    int i, attributeLength;
    String attributeName;

   attributeLength = attributes.getLength();
   for (i=0; i < attributeLength; i++) {
    attributeName = attributes.item(i).getNodeName();
    if(attributeName.compareToIgnoreCase("xml_version") == 0)
      master.setXMLVersion(attributes.item(i).getNodeValue());
    if(attributeName.compareToIgnoreCase("system") == 0)
      master.setSystem(attributes.item(i).getNodeValue());
    if(attributeName.compareToIgnoreCase("created") == 0)
      master.setCreated(attributes.item(i).getNodeValue());
    if(attributeName.compareToIgnoreCase("creator") == 0)
      master.setCreator(attributes.item(i).getNodeValue());
   }
  }

  static void parseAttributes(Note note, NamedNodeMap attributes) {
    int i, attributeLength;
    String attributeName;

   attributeLength = attributes.getLength();
   for (i=0; i < attributeLength; i++) {
    attributeName = attributes.item(i).getNodeName();
    if(attributeName.compareToIgnoreCase("link_type") == 0)
      note.setlink_type(attributes.item(i).getNodeValue());
    if(attributeName.compareToIgnoreCase("sequence") == 0)
      note.setSequence(attributes.item(i).getNodeValue());
    if(attributeName.compareToIgnoreCase("note_kind") == 0)
      note.setNote_kind(attributes.item(i).getNodeValue());
    if(attributeName.compareToIgnoreCase("created") == 0)
      note.setCreated(attributes.item(i).getNodeValue());
   }
  }

  static void crap() {
      int i = 0;
      char c;
      try {
      while (i != -1) {
        i = System.in.read();
        c = (char) i;
      }
    } catch (IOException ioe) {
        System.out.println("IO error" + ioe);
    }
  }

  private static void spacer(int spaces) {
    int i;
    for (i = 0; i < spaces; i++)
      System.out.print(" ");
  }

  static void message(String msg) {
    System.out.println(msg);
    System.out.println("");
  }

}

